//  20/08/1998 : G.Barrand : creation.
//  30/01/2004 : L.Garnier : addition of methods to modifie hatch parameters


#ifndef HEPVis_SbStyle_h
#define HEPVis_SbStyle_h

#include <Inventor/SbColor.h>
#include <Inventor/SbString.h>
 
#include <HEPVis/SbStyles.h>

#include <vector>

class SbStyleColor : public std::pair<SbString,SbColor> {
public:
  SbStyleColor();
  SbStyleColor(const SbString& aName,const SbColor& aColor);
  SbStyleColor(const SbString& aName,float aRed,float aGreen,float aBlue);
  SbStyleColor(const SbStyleColor& aFrom);
  virtual ~SbStyleColor();
  SbStyleColor& operator=(const SbStyleColor& aFrom);
};

class SbStyleColormap : public std::vector<SbStyleColor> {
public:
  SbColor getColor(int) const;
  SbString getStringColor(int) const;
};

class SbStyle {
public:
  SbStyle();
  SbStyle(const SbStyle&);
  virtual ~SbStyle();
  SbStyle& operator=(const SbStyle&);
  void copy(const SbStyle&);
  SbBool isEqual(const SbStyle&);
  virtual void reset();
  void dump();
  //SbBool setFromScript(const SbString&);
  //SbString getScript() const;
  SbBool setFromString(const SbString&);
  SbString getString() const;
  void setColor(const SbColor&);
  SbColor getColor() const;
  void setHighlightColor(const SbColor&);
  SbColor getHighlightColor() const;
  void setLinePattern(SbLinePattern);
  SbLinePattern getLinePattern() const;
  void setLineWidth(int);
  int getLineWidth() const;
  float getTransparency() const;
  void setTransparency(float);
  void setMarkerStyle(SbMarkerStyle);
  SbMarkerStyle getMarkerStyle() const;
  void setMarkerSize(int);
  int getMarkerSize() const;
  void setFontSize(int);
  int getFontSize() const;
  void setAreaStyle(SbAreaStyle);
  SbAreaStyle getAreaStyle() const;
  void setModeling(const SbString&);
  const SbString& getModeling() const;
  void setTickModeling(const SbString&);
  const SbString& getTickModeling() const;
  void setEncoding(const SbString&);
  const SbString& getEncoding() const;
  void setSmoothing(SbBool);
  SbBool getSmoothing() const;
  void setHinting(SbBool);
  SbBool getHinting() const;
  void setPaintingPolicy(SbPaintingPolicy);
  SbPaintingPolicy getPaintingPolicy() const;
  void setHatchingPolicy(SbHatchingPolicy);
  SbHatchingPolicy getHatchingPolicy() const;
  void setProjectionType(SbProjectionType);
  SbProjectionType getProjectionType() const;
  void setFontName(const SbString&);
  const SbString& getFontName() const;
  void setDivisions(int);
  int getDivisions()const;
  void setOffset(float);
  float getOffset()const;
  void setStripWidth(float);
  float getStripWidth()const;
  void setAngle(float);
  float getAngle() const;
  void setScale(float);
  float getScale() const;
  void setSpacing(float);
  float getSpacing() const;
  void setMultiNodeLimit(int);
  int getMultiNodeLimit() const;
  void setVisible(SbBool);
  SbBool getVisible()const;
  void setBarOffset(float);
  float getBarOffset() const;
  void setBarWidth(float);
  float getBarWidth() const;
  void setCut(const SbString&);
  SbString getCut() const;
  void setOptions(const SbString&);
  SbString getOptions() const;
public:
  static SbStyleColormap& getDefaultColormap();
  static SbStyleColormap& getROOT_Colormap();
  static SbBool stringToColor(const SbString&,SbColor&);
  static SbBool colorToString(const SbColor&,SbString&);
  static int getValueColorMapping (const SbString&,float*&,SbColor*&);
  static char* markerStyleToString(SbMarkerStyle);
  static char* areaStyleToString(SbAreaStyle);
  static char* paintingPolicyToString(SbPaintingPolicy);
  static char* hatchingPolicyToString(SbHatchingPolicy);
  static char* projectionTypeToString(SbProjectionType);
private: 
  SbColor fColor;
  SbColor fHighlightColor;
  int fLineWidth;
  int fMarkerSize;
  int fFontSize;
  SbLinePattern fLinePattern;
  SbMarkerStyle fMarkerStyle;
  SbAreaStyle fAreaStyle;
  SbString fModeling; 
  SbString fTickModeling; 
  SbString fEncoding; 
  SbBool fSmoothing;
  SbBool fHinting;
  SbString fCut;
  SbPaintingPolicy fPaintingPolicy;
  SbHatchingPolicy fHatchingPolicy;
  SbProjectionType fProjectionType;
  SbString fFontName;
  int fMultiNodeLimit;
  int fDivisions;
  float fTransparency;
  float fSpacing;
  float fAngle;
  float fScale;
  float fOffset;
  float fStripWidth;
  SbBool fVisible;
  float fBarOffset;
  float fBarWidth;
  SbString fOptions;
};

#endif



