/*-----------------------------HEPVis----------------------------------------*/
/*                                                                           */
/* Node:             SoXtGui                                            */
/* Description:      HEP GUI Class                                           */
/*                  (draws heavily on Jeff Kallenbach's SoXtHepViewer class) */
/*                                                                           */
/* Author:           Lucas Taylor  July 3 1998                               */
/*                                                                           */
/*---------------------------------------------------------------------------*/
#ifndef SoXtGui_H
#define SoXtGui_H
#include <Inventor/Xt/SoXtComponent.h>
#include <Inventor/Xt/viewers/SoXtFullViewer.h>
#include <Inventor/SoPrimitiveVertex.h>
#include <Inventor/SbLinear.h>
#include <Inventor/SbString.h>
#include <Xm/PushB.h>

#include <HEPVis/Xt/Twig.h>

class SoXtExaminerViewer;
class SoXtPlaneViewer;
class SoXtWalkViewer;
class SoXtFlyViewer;
class SoXtPrintDialog;
class SoSeparator;
class SoNode;
class SoGroup;
class SoLabel;
class SoEnvironment;
class SoDirectionalLight;
class SoPerspectiveCamera;
class SoXtColorEditor;
class SoXtMaterialEditor;         // tp
class SbColor;
class SoText2;
class SoTimerSensor;

#ifdef __KCC
class InfoStreamBuf;
class InfoStream;
#endif //__KCC

class SoXtGui: public SoXtComponent
{
  
public:

  typedef int (*TrigCallbackProc) (void *);
  typedef int (*QuitCallbackProc) (void *);
  
  typedef enum {EXAMINER, PLANE, WALK, FLY} ViewStyle;

  //
  // Constructor
  //
  SoXtGui(Widget parent=NULL, 
		const char *name="HEPVis GUI", 
		SbBool buildInsideParent=TRUE);
  //
  // Destructor
  //
  virtual ~SoXtGui();
  
  virtual void setSceneGraph(SoNode *); 
  virtual SoNode *getSceneGraph() const; 

  //
  // Access to the viewer:
  //
  virtual SoXtFullViewer *getCurrentViewer() const;
  virtual SoXtFullViewer *getViewer(int viewNumber) const;

  //
  // Access to the material editor:
  //
  virtual SoXtMaterialEditor* getMaterialEditor() const;

  //
  // Access to the menu bar:
  //
  virtual Widget getMenuBar() const;

  //
  // Access to the file menu:
  //
  virtual Widget getFileMenu() const;

  //
  // Access to the viewer menu:
  //
  virtual Widget getViewMenu() const;

  //
  // Access to the help menu:
  //
  virtual Widget getHelpMenu() const;
    
  //
  // Define URL for WWW help
  //
  virtual void setHelpURL(SbString);
        
  //
  // Access to the text window
  //
  virtual void addText(char *TextItem);
  virtual void clearText();
  virtual void showTextArea();
  virtual void hideTextArea();
  
  //
  // Resize window
  //
  virtual void resizeMain(short hgt, short wid);
  
  //
  // Callbacks
  //
  virtual void setTrigCallback (TrigCallbackProc procedure, void * call_data);
  virtual void setQuitCallback (QuitCallbackProc procedure, void * call_data);

  //
  // This is for error messages:
  //
  void setErrorMessage (char *);

  //
  // This is to build a new widget (a menu for now) for controlling Twigs:
  //
  void createTwigWidget (Twig *);



protected:
  
  //
  // These are for printing...
  //
  SoXtPrintDialog *printDialog;
  static void printCallback(Widget w, XtPointer callData, XmPushButtonCallbackStruct *callbackStruct);
  static void print(SoXtGui *);
  static void beforePrint(void *, SoXtPrintDialog *);
  static void afterPrint(void *, SoXtPrintDialog *);
  static void saveIVCallback(Widget w, XtPointer callData, XmPushButtonCallbackStruct *callbackStruct);
  static void saveIV(Widget, void *, XmFileSelectionBoxCallbackStruct *);  

  //
  // This get "New event"
  //
  static void invokeTrig (Widget w, XtPointer callData, XmPushButtonCallbackStruct *callbackStruct);

  //
  // This "Exits"
  //
  static void invokeQuit (Widget w, XtPointer callData, XmPushButtonCallbackStruct *callbackStruct);

  //
  // These are for the file dialog...
  //
  static int  trigCallback(void *);
  static void openCallback(Widget w, XtPointer callData,XmPushButtonCallbackStruct *callbackStruct);
  static void open(Widget, void *, XmFileSelectionBoxCallbackStruct *);  
  static int  quitCallback(void *);
  //
  // Check for valid input
  //
  static void verifyNumeric(Widget, XtPointer, XmTextVerifyCallbackStruct *);

  //
  // These are for editing the background color
  //
  static void editBackgroundColor(Widget w, XtPointer clientData, XmPushButtonCallbackStruct *callData);
  static void backgroundColorCB(void *This, const SbColor *color);

  //
  // These are for the material editor
  //
  static void editMaterial(Widget w, XtPointer clientData, XmPushButtonCallbackStruct *callData);
  //  static void backgroundColorCB(void *This, const SbColor *color);          // not yet implemented

  //
  // This spawns a new view
  //
  static void spawnView
    (Widget w, XtPointer clientData, XmPushButtonCallbackStruct *);

  //
  // This toggles Antialiasing
  //
  static void toggleAACB
   (Widget w, XtPointer callData, XmToggleButtonCallbackStruct *callbackStruct);

  //
  // This toggles Perf Meter
  //
  static void togglePMCB
   (Widget w, XtPointer callData, XmToggleButtonCallbackStruct *callbackStruct);

  //
  // This toggles the Axes (Labels)
  //
  static void toggleAxesCB
   (Widget w, XtPointer callData, XmToggleButtonCallbackStruct *callbackStruct);

  //
  // This toggles the view
  //
  static void toggleViewCB
   (Widget w, XtPointer callData, XmToggleButtonCallbackStruct *callbackStruct);

  //
  // This toggles the transparency
  //
  static void toggleTransCB
   (Widget w, XtPointer callData,XmToggleButtonCallbackStruct *callbackStruct);

  //
  // This toggles the highlight style
  //
  static void toggleHLCB
   (Widget w, XtPointer callData, XmToggleButtonCallbackStruct *callbackStruct);

  //
  // This toggles display of the text window
  //
  static void toggleShowTextWindow
   (Widget w, XtPointer callData, XmToggleButtonCallbackStruct *callbackStruct);


  //
  // This exits the program
  //
  static void close
   (Widget w, XtPointer callData, XmPushButtonCallbackStruct *callbackStruct);

  //
  // This clears the text window of text
  //
  static void clearTextCB
   (Widget w, XtPointer callData, XmPushButtonCallbackStruct *callbackStruct);

  //
  // This toggles the lights
  //
  static void toggleLights
    (Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *callData);

  //
  // This is the WWW help callback
  //
  static void helpButtonCB 
    (Widget w, XtPointer client_data, XtPointer call_data);

  //
  // This is the print IV Tree callback (only useful for Twig's)
  //
  static void printTreeCB 
    (Widget w, XtPointer client_data, XmPushButtonCallbackStruct *callbackStruct);

  //
  // This is the make Twig dialog callback (only useful for Twig's)
  //
  static void createTwigDialog 
    (Widget w, XtPointer client_data, XmPushButtonCallbackStruct *callbackStruct);

  //
  // This is for the scene
  //
  SoSeparator *ancestor;
  SoNode      *sceneGraph;

  //
  // This is the background color editor
  //
  SoXtColorEditor *_backgroundColorEditor;

  //
  // This is the material editor
  //
  SoXtMaterialEditor* _materialEditor;

  //
  // These are the lights
  //
  SoDirectionalLight *_leftFlood;
  SoDirectionalLight *_rightFlood;
  SoDirectionalLight *_topFlood;
  SoDirectionalLight *_bottomFlood;

  virtual void create(Widget);
  
  SoXtFullViewer      *_theFullViewer;
  SoXtExaminerViewer  *_theExaminerViewer;
  SoXtPlaneViewer     *_thePlaneViewer;
  SoXtWalkViewer      *_theWalkViewer;
  SoXtFlyViewer       *_theFlyViewer;

  Widget _MainWindow;
  Widget _panedWindow;
  Widget _theTextArea;
  Widget _filterForm;
  
  Widget _menuBar;
  Widget _userMenuBar;
  
  Widget _fileMenuButton;
  Widget _fileMenu;
  Widget _trigButton;
  Widget _openButton;
  Widget _saveButton;
  Widget _printButton;
  Widget _closeButton;
  Widget _quitButton;
  
  Widget _viewMenuButton;
  Widget _viewMenu;
  Widget _pulldownMenu;
  Widget _viewStyle;
  Widget _examinerButton;
  Widget _planeButton;
  Widget _walkButton;
  Widget _flyButton;
  Widget _newExaminerButton;
  Widget _newPlaneButton;
  Widget _newWalkButton;
  Widget _newFlyButton;
  Widget _lightsButton;
  Widget _lightsMenu;
  Widget _headlightButton;
  Widget _leftFloodButton;
  Widget _rightFloodButton;
  Widget _topFloodButton;
  Widget _bottomFloodButton;
  Widget _transparencyButton;
  Widget _transparencyMenu;
  Widget _screenDoorButton;
  Widget _blendedButton;
  Widget _sortedButton;
  Widget _sortedBlendedButton;
  Widget _antialiasingButton;
  Widget _perfMeterButton;
  Widget _axesButton;
  Widget _clearTextButton;
  Widget _showTextWindowButton;
  Widget _editBackgroundButton;
  Widget _editMaterialButton;         // tp
  Widget _highlightButton;
  Widget _highlightMenu;
  Widget _noHighlightButton;
  Widget _boxHighlightButton;
  Widget _linesHighlightButton;
  Widget _helpMenuButton;
  Widget _helpMenu;
  Widget _helpPopupButton;

  // Widgets associated with Twig Widget

  Widget _twigMenuButton;  
  Widget _twigMenu;

  
  
private: 
  
  static String         _defaultSoXtGuiResources[];
  
  //
  // Lights, camera, environment!
  //
  SoGroup 	    	*lightsCameraEnvironment;
  SoLabel 	    	*envLabel;
  SoEnvironment   	*environment;
  SoGroup 	    	*lightGroup;
  SoPerspectiveCamera   *perspectiveCamera;

  //
  // Callbacks
  //
  TrigCallbackProc _trigCallback;
  QuitCallbackProc _quitCallback;

  //
  //Labels
  //
  static void addLabels(SoXtGui *This);

  //
  // Perf Meter
  //
  static void instrumentScene(SoXtGui *This);
  static void instrumentViewer(SoXtGui *viewer);
  static void unInstrumentViewer(SoXtGui *viewer);
  static void overlayViewportCB(void *, SoAction *);
  static void perfSensorCB(void *, SoSensor *);
  static void viewStartCB(void *, SoXtViewer *);
  static void countUpdatesCB(void *, SoAction *);
  static  int countTriangles(SoNode *);
  static void countTriangleCB(void *, SoCallbackAction *, 
		  const SoPrimitiveVertex *, const SoPrimitiveVertex *, 
		  const SoPrimitiveVertex *);

  SoTimerSensor *perfSensor;
  SoText2 *perfMeterText;
  int Metered;
  int numTris;
  int numFramesRendered;

  SbBool wasAnimating;
  SbTime startTime; 
  double lastFrames;

  void *_trigCallData;
  void *_quitCallData;
  
  static SbString *helpURL; 
 
  //
  // The Pointer for the top of any selectable Twigs
  //
  Twig *theTopTwig;



};
#ifdef __KCC // InfoStreamBuf requires fully templated iostreams
#include <ostream>
#include <streambuf>
#include <ios>
#include <string>

using std::streambuf;
using std::string;
using std::ostream;

class InfoStreamBuf : public streambuf {
public:
	InfoStreamBuf() {}
	virtual ~InfoStreamBuf() {}
        virtual void setGui(SoXtGui *aGui) {theGui = aGui;}//set the output Gui
protected:
	virtual int_type overflow(int_type ch = traits_type::eof());
	virtual int underflow() {
	  return traits_type::eof(); }
	virtual int uflow() {
	  return traits_type::eof(); }
private:
  string my_buffer;
  SoXtGui *theGui;
};
class InfoStream : public ostream 
{
public:
	InfoStream(SoXtGui *aGui) : ostream(&mybuf) {mybuf.setGui(aGui);}
	virtual ~InfoStream(){}
	const InfoStreamBuf *rdbuf() const {return static_cast<const InfoStreamBuf *>(&mybuf);}
private:
	InfoStreamBuf mybuf;
};
#endif // __KCC
#endif


