#ifndef HEPVis_SoGL2PSAction_h
#define HEPVis_SoGL2PSAction_h 

#include <Inventor/actions/SoGLRenderAction.h>

/**
 *  SoGL2PSAction inherits Inventor/SoGLRenderAction.
 * It uses the gl2ps software to produce vector PostScript of a scene graph.
 *  See applications/DetectorTreeKit, Polyhedron for examples.
 */

class SoGL2PSAction : public SoGLRenderAction {
  SO_ACTION_HEADER(SoGL2PSAction);
public:
  SoGL2PSAction(const SbViewportRegion&);
  // gl2ps file formats (mapping of GL2PS_PS, etc...)
  enum Format {
    PS = 1,
    EPS = 2,
    TEX = 3,
    PDF = 4
  };
  void setFileFormat(Format);
  void setFileName(const char*);
  void enableFileWriting();
  void disableFileWriting();
  SbBool fileWritingEnabled() const;
  SbBool addBitmap(int,int,float=0,float=0,float=0,float=0);
  void beginViewport();
  void endViewport();
public: /*SoINTERNAL*/
  static void initClass();
protected:
  virtual void beginTraversal(SoNode*);
private:
  void gl2psBegin();
  static void separatorAction(SoAction*,SoNode*);
  static void drawStyleAction(SoAction*,SoNode*);
#ifdef __COIN__
#else //SGI
  static void GLRenderS(SoAction*,SoNode*);
#endif
private:
  Format fFileFormat;
  SbString fFileName;
  FILE* fFile;
};

#endif

