/*-----------------------------HEPVis----------------------------------------*/
/*                                                                           */
/* Node:             SoLegendRegion                                          */
/* Description:      A nodekit to handle a legend box for the plotter.       */
/* Author            Guy Barrand                                             */
/*                                                                           */
/*---------------------------------------------------------------------------*/

#ifndef HEPVis_SoLegendRegion_h
#define HEPVis_SoLegendRegion_h

/*
 * SoLegendRegion is a basekit to handle text within a viewport region.
 *  See the applications/Plotter example.
 */

#include <HEPVis/nodekits/SoRegion.h>

#include <Inventor/fields/SoMFString.h>
#include <Inventor/fields/SoSFEnum.h>

#include <vector>

class SoStyle;
class SoMFString;

class SoLegendRegion : public SoRegion {
  SO_KIT_HEADER(SoLegendRegion);
  SO_KIT_CATALOG_ENTRY_HEADER(topSeparator);
  SO_KIT_CATALOG_ENTRY_HEADER(camera);
  SO_KIT_CATALOG_ENTRY_HEADER(material);
  SO_KIT_CATALOG_ENTRY_HEADER(lines);
  SO_KIT_CATALOG_ENTRY_HEADER(styles);
public:
  SoMFString text;
public: /*SoINTERNAL*/
  static void initClass();
public:
  SoLegendRegion();
  void reset();
  SoStyle* getStyle(int);
public: //SoRegion
  virtual SoSeparator* getTopSeparator() const;
  virtual SoCamera* getCamera() const;
  virtual void resetCamera();
public: /*SoEXTENDER*/
  virtual void doAction(SoAction*);
  virtual void callback(SoCallbackAction*);
  virtual void GLRender(SoGLRenderAction*);
  virtual void getBoundingBox(SoGetBoundingBoxAction*);
  virtual void getMatrix(SoGetMatrixAction*);
  virtual void handleEvent(SoHandleEventAction*);
  virtual void rayPick(SoRayPickAction*);
  virtual void search(SoSearchAction*);
  virtual void write(SoWriteAction*);
  virtual void pick(SoPickAction*);
protected:
  virtual ~SoLegendRegion();
private:
  void matchViewport(short,short);
  void createChildren();
  void setTextPointSize(int);
  void getInsideSize(short&,short&,int&,int&,int&,std::vector<short>&,short&);
private:
  short fOldWidth;
  short fOldHeight;
  SoStyle* fDefaultStyle;
  SoMFString* fOldText;
};


#endif
