/*-----------------------------HEPVis----------------------------------------*/
/*                                                                           */
/* Node:             SoPlotterRegion                                         */
/* Description:      A nodekit to handle a plotter within a viewport region. */
/* Author            Guy Barrand                                             */
/*                                                                           */
/*---------------------------------------------------------------------------*/

#ifndef HEPVis_SoPlotterRegion_h
#define HEPVis_SoPlotterRegion_h

/*
 *  See the applications/Plotter example.
 */

// Inheritance :
#include <HEPVis/nodekits/SoRegion.h>

#include <Inventor/fields/SoSFVec2f.h>
#include <Inventor/fields/SoSFBool.h>
#include <Inventor/fields/SoSFString.h>
#include <Inventor/fields/SoSFEnum.h>

#include <vector>

class SbPlottableObject;
class SoMFString;

//#define SOPLOTTERREGION_WITH_SENSOR

#ifdef SOPLOTTERREGION_WITH_SENSOR
class SoFieldSensor;
class SoSensor;
#endif

class SbPlotterDoIt : public virtual SbDoIt {
public: //SbDoIt
  virtual void* cast(const SbString&) const;
public:
  SbPlotterDoIt(SbPlottableObject*);
  virtual ~SbPlotterDoIt();
  SbPlottableObject* getPlottableObject() const;
  void reset();
private:
  SbPlottableObject* fPlottableObject;
};

class SoPlotter;
class SoViewportRegion;
class SoOrthographicCamera;
class SoTextRegion;
class SoInfosRegion;
class SoLegendRegion;
class SoSensor;
class SoFieldSensor;
class SoNodeSensor;
class SoMFString;

class SoPlotterRegion : public SoRegion {
  SO_KIT_HEADER(SoPlotterRegion);
  SO_KIT_CATALOG_ENTRY_HEADER(topSeparator);
  SO_KIT_CATALOG_ENTRY_HEADER(plotterSeparator);
  SO_KIT_CATALOG_ENTRY_HEADER(light);
  SO_KIT_CATALOG_ENTRY_HEADER(lightModel);
  SO_KIT_CATALOG_ENTRY_HEADER(camera);
  SO_KIT_CATALOG_ENTRY_HEADER(plotter);
  SO_KIT_CATALOG_ENTRY_HEADER(titleRegion);
  SO_KIT_CATALOG_ENTRY_HEADER(titleDummy);
  SO_KIT_CATALOG_ENTRY_HEADER(infosRegion);
  SO_KIT_CATALOG_ENTRY_HEADER(infosDummy);
  SO_KIT_CATALOG_ENTRY_HEADER(legendRegion);
  SO_KIT_CATALOG_ENTRY_HEADER(legendDummy);
  SO_KIT_CATALOG_ENTRY_HEADER(etcSeparator);
public:
  SoSFBool titleRegionVisible;
  SoSFBool infosRegionVisible;
  SoSFBool legendRegionVisible;
  SoSFVec2f titleRegionOrigin;
  SoSFVec2f titleRegionSize;
  SoSFBool infosRegionAttachedToDataFrame;
  SoSFVec2f infosRegionOrigin;
  SoSFVec2f infosRegionSize;
  SoSFVec2f legendRegionOrigin;
  SoSFVec2f legendRegionSize;
  SoSFString infosRegionStyle;
  enum ViewportMapping {
    ADJUST_VIEWPORT,
    LEAVE_ALONE
  };
  SoSFEnum plotterViewportMapping;
  enum ResizePolicy {
    RESIZE_HEIGHT,
    ASPECT_RATIO
  };
  SoSFEnum infosRegionResizePolicy;
public: /*SoINTERNAL*/
  static void initClass();
#ifdef SOPLOTTERREGION_WITH_SENSOR
#else
public: /*SoEXTENDER*/
  virtual void doAction(SoAction*);
  virtual void callback(SoCallbackAction*);
  virtual void GLRender(SoGLRenderAction*);
  virtual void getBoundingBox(SoGetBoundingBoxAction*);
  virtual void getMatrix(SoGetMatrixAction*);
  virtual void handleEvent(SoHandleEventAction*);
  virtual void rayPick(SoRayPickAction*);
  virtual void search(SoSearchAction*);
  virtual void write(SoWriteAction*);
  virtual void pick(SoPickAction*);
#endif
public: //SoRegion
  virtual SoSeparator* getTopSeparator() const;
  virtual SoCamera* getCamera() const;
  virtual void clear(const SbString& = "");
  virtual void resetCamera();
  virtual void refresh();
  virtual SbBool doIt(const SbDoIt&);
  virtual void undo();
  virtual void resetUndo();
  virtual SoNode* getWriteNode(const SbString& = "") const;
public:
  void setStyle(const SbString&);
  void setStyleLab_Default();
  void setStyleHippo_Default();
  void setStylePAW_Default();
  void setStyleROOT_Default();
  void setStyleROOT_Plain();
  void setStyleROOT_Bold();
  void setStyleROOT_Video();
  void setStyleROOT_Pub();
  void setStyleROOT_LHCb();
public:
  SoPlotterRegion();
  SoPlotter* getPlotter() const;
  SoTextRegion* getTitleRegion() const;
  SoInfosRegion* getInfosRegion() const;
  SoLegendRegion* getLegendRegion() const;
  SoSeparator* getEtcSeparator() const;
  void addPlottableObject(SbPlottableObject*);
protected:
  virtual ~SoPlotterRegion();
private:
  void updateChildren();
  void updateTitleRegionContent();
  void updateInfosContent(); 
  void updateLegendContent(); 
  void updateTitleRegionGeometry(); 
  void updateInfosRegionGeometry(const SbVec2f&); 
  void updateLegendRegionGeometry(const SbVec2f&); 
private:
  std::vector<SbPlottableObject*> fDoList;
  int fOldWidth;
  int fOldHeight;
  // SoPlotterRegion fields :
  SbBool fOld_titleRegionVisible;
  SbBool fOld_infosRegionVisible;
  SbBool fOld_legendRegionVisible;
  SbVec2f fOld_titleRegionOrigin;
  SbVec2f fOld_titleRegionSize;
  SbVec2f fOld_infosRegionOrigin;
  SbVec2f fOld_infosRegionSize;
  SbVec2f fOld_legendRegionOrigin;
  SbVec2f fOld_legendRegionSize;
  SbString fOld_infosRegionStyle;
  int fOld_plotterViewportMapping;
  int fOld_infosRegionResizePolicy;
  // SoPlotter fieds :
  float fOldPlotter_height;
  float fOldPlotter_depth;
  int fOldPlotter_shape;
  SbString fOldPlotter_title;
  SbVec3f fOldPlotter_screenDataFrameUpperRightCorner;
  SoMFString* fOldPlotter_infos;
  SoMFString* fOldPlotter_legend;

  SbBool fUpdate;
#ifdef SOPLOTTERREGION_WITH_SENSOR
  SoFieldSensor* f_update_Sensor;
  static void updateCB(void*,SoSensor*); 
  SoSFBool update;
#endif
};

#endif
