#ifndef HEPVis_SoRegion_h
#define HEPVis_SoRegion_h

#include <Inventor/nodekits/SoBaseKit.h>
#include <Inventor/fields/SoSFVec2f.h>
#include <Inventor/fields/SoSFBool.h>
#include <Inventor/fields/SoSFColor.h>
#include <Inventor/fields/SoSFShort.h>
#include <Inventor/SbLinear.h>

#include<list>

class SoViewportRegion;
class SoSeparator;
class SoCamera;
class SoGLRenderAction;
class SoStyleCache;

class RegionCamera {
public:
  RegionCamera(const SoCamera&);
  RegionCamera(const RegionCamera&);
  RegionCamera& operator=(const RegionCamera&);
  virtual ~RegionCamera();
  void restore(SoCamera&);
private:
  int fViewportMapping;
  SbVec3f fPosition;
  SbRotation fOrientation;
  float fAspectRatio;
  float fNearDistance;
  float fFarDistance;
  float fFocalDistance;
  float fHeight;
};

class SbDoIt {
public:
  virtual ~SbDoIt() {}
public:
  virtual void* cast(const SbString&) const = 0;
};

class SbAddNode : public virtual SbDoIt {
public: //SbDoIt
  virtual void* cast(const SbString& aClass) const;
public:
  SbAddNode(SoNode* aNode,const SbString& aPlacement);
  virtual ~SbAddNode();
  SoNode* getNode() const;
  const SbString& getPlacement() const;
  void reset();
private:
  SoNode* fNode;
  SbString fPlacement;
};

class SoRegion : public SoBaseKit {
  SO_KIT_HEADER(SoRegion);
  SO_KIT_CATALOG_ENTRY_HEADER(viewportRegion);
public:
  SoSFBool visible;
  SoSFBool verbose;
  SoSFColor color;
  SoSFShort horizontalMargin; //In pixels
  SoSFShort verticalMargin; //In pixels
public:
  SoRegion();
  SoViewportRegion* getViewportRegion() const;
  void pushCamera(); 
  void popCamera(); 
  void clearCameraStack(); 
  void viewRectangle(const SbVec2s&,const SbVec2s&,const SbVec2s&); 
  SoNode* findPart(const SbName&);
  SoNode* findNode(const SbString&,const SbString&);
  SoStyleCache* styleCache() const;
  void setPAW_Placement(int,int,int);
  void getPAW_Placement(int&,int&,int&) const;
public:
  virtual SoSeparator* getTopSeparator() const;
  virtual SoCamera* getCamera() const;
  virtual void clear(const SbString& = "");
  virtual void refresh();
  virtual void resetCamera();
  virtual SbBool doIt(const SbDoIt&);
  virtual void undo();
  virtual void resetUndo();
  virtual void connectFrom(SoRegion*);
  virtual SoNode* getWriteNode(const SbString& = "") const;
public:
  static void initClass();
public: /*SoEXTENDER*/
  virtual void GLRender(SoGLRenderAction*);
protected:
  virtual ~SoRegion();
private:
  void center(const SbVec2f&,float); 
protected:
  SoStyleCache* fStyleCache;
private:
  std::list<RegionCamera> fCameraStack;
  int fPAW_Rows,fPAW_Cols,fPAW_Index;
};

#endif
