#ifndef Lab_Callbacks_AIDA_h
#define Lab_Callbacks_AIDA_h

//
//  This file contains static helper methods.
// It is intended to be included in OnX Callbacks file
// to ease callbacks programming.
//  It relies on interfaces and c++ libraries only.
//

#include <AIDA/ITree.h>
#include <AIDA/ITreeFactory.h>
#include <AIDA/IManagedObject.h>
#include <AIDA/IHistogram1D.h>
#include <AIDA/IHistogram2D.h>
#include <AIDA/IHistogram3D.h>
#include <AIDA/IProfile1D.h>
#include <AIDA/IAnalysisFactory.h>
#include <AIDA/ICloud.h>
#include <AIDA/ICloud2D.h>
#include <AIDA/ICloud3D.h>
#include <AIDA/IFunction.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IFunctionFactory.h>
#include <AIDA/ITupleFactory.h>
#include <AIDA/ITuple.h>
#include <AIDA/IEvaluator.h>
#include <AIDA/IFilter.h>
#include <AIDA/IAxis.h>
#include <AIDA/IAnnotation.h>
#include <AIDA/IDataPointSetFactory.h>
#include <AIDA/IDataPointSet.h>
#include <AIDA/IFitFactory.h>
#include <AIDA/IFitter.h>
#include <AIDA/IPlotterFactory.h>
#include <AIDA/IPlotter.h>
#include <AIDA/IPlotterRegion.h>

#include <Lib/Manager.h> // Does not involve to link with Lib.
#include <Lib/Cast.h> // Does not involve to link with Lib.
#include <Lib/smanip.h>

#include <math.h> //sqrt

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
/// Not AIDA /////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static AIDA::IAnalysisFactory* find_aida(
 ISession& aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return  Lib_findManager(aSession,"AnalysisFactory",AIDA::IAnalysisFactory);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
/// Over AIDA only ///////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static AIDA::ITree* find_memoryTree(
 AIDA::IAnalysisFactory& aAIDA
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITreeFactory* treeFactory = aAIDA.createTreeFactory();
  if(!treeFactory) return 0;
  AIDA::ITree* tree = treeFactory->create();
  delete treeFactory;
  return tree;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IBaseHistogram* find_baseHistogram(
 AIDA::ITree& aTree
,const std::string& aName
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IManagedObject* object = aTree.find(aName);
  if(!object) return 0;
  return Lib_Cast(*object,AIDA::IBaseHistogram);
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IHistogram1D* find_histogram1D(
 AIDA::ITree& aTree
,const std::string& aName
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IManagedObject* object = aTree.find(aName);
  if(!object) return 0;
  return Lib_Cast(*object,AIDA::IHistogram1D);
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IHistogram2D* find_histogram2D(
 AIDA::ITree& aTree
,const std::string& aName
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IManagedObject* object = aTree.find(aName);
  if(!object) return 0;
  return Lib_Cast(*object,AIDA::IHistogram2D);
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IFunction* find_function(
 AIDA::ITree& aTree
,const std::string& aName
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IManagedObject* object = aTree.find(aName);
  if(!object) return 0;
  return Lib_Cast(*object,AIDA::IFunction);
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::ITuple* find_tuple(
 AIDA::ITree& aTree
,const std::string& aName
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IManagedObject* object = aTree.find(aName);
  if(!object) return 0;
  return Lib_Cast(*object,AIDA::ITuple);
}
//////////////////////////////////////////////////////////////////////////////
static std::string tuple_name(
 const AIDA::ITuple& aTuple
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IManagedObject* mobject = Lib_Cast(aTuple,AIDA::IManagedObject);
  if(!mobject) return "";
  return mobject->name();
}
//////////////////////////////////////////////////////////////////////////////
static std::string histogram_name(
 const AIDA::IBaseHistogram& aHistogram
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IManagedObject* mobject = Lib_Cast(aHistogram,AIDA::IManagedObject);
  if(!mobject) return "";
  return mobject->name();
}
//////////////////////////////////////////////////////////////////////////////
/// Creators //////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static AIDA::ITree* create_tree(
 AIDA::IAnalysisFactory& aAIDA
,const std::string& aStoreName
,const std::string& aStoreType
,bool aReadOnly
,bool aCreateNew
,const std::string& aOptions = ""
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITreeFactory* treeFactory = aAIDA.createTreeFactory();
  if(!treeFactory) return 0;
  AIDA::ITree* tree = 
    treeFactory->create(aStoreName,aStoreType,aReadOnly,aCreateNew,aOptions);
  delete treeFactory;
  return tree;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IHistogram1D* create_histogram1D(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aName
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogramFactory* factory = aAIDA.createHistogramFactory(aTree);
  if(!factory) return 0;
  AIDA::IHistogram1D* histo = 
      factory->createHistogram1D(aName,aTitle,aXnumber,aXmin,aXmax);
  delete factory;
  return histo;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IHistogram1D* copy_histogram1D(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,AIDA::IHistogram1D& aHisto
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogramFactory* factory = aAIDA.createHistogramFactory(aTree);
  if(!factory) return 0;
  AIDA::IHistogram1D* histo = factory->createCopy(aPath,aHisto);
  delete factory;
  return histo;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IHistogram2D* copy_histogram2D(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,AIDA::IHistogram2D& aHisto
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogramFactory* factory = aAIDA.createHistogramFactory(aTree);
  if(!factory) return 0;
  AIDA::IHistogram2D* histo = factory->createCopy(aPath,aHisto);
  delete factory;
  return histo;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IBaseHistogram* copy_baseHistogram(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,AIDA::IBaseHistogram& aHisto
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogramFactory* factory = aAIDA.createHistogramFactory(aTree);
  if(!factory) return 0;
  if(AIDA::IHistogram1D* object = Lib_Cast(aHisto,AIDA::IHistogram1D)) {
    AIDA::IHistogram1D* cobject = factory->createCopy(aPath,*object);
    delete factory;
    return cobject;
  } else if(AIDA::IHistogram2D* object = Lib_Cast(aHisto,AIDA::IHistogram2D)) {
    AIDA::IHistogram2D* cobject = factory->createCopy(aPath,*object);
    delete factory;
    return cobject;
  } else if(AIDA::IHistogram3D* object = Lib_Cast(aHisto,AIDA::IHistogram3D)) {
    AIDA::IHistogram3D* cobject = factory->createCopy(aPath,*object);
    delete factory;
    return cobject;
  } else {
    delete factory;
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IHistogram2D* create_histogram2D(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aName
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
,int aYnumber
,double aYmin
,double aYmax
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogramFactory* factory = aAIDA.createHistogramFactory(aTree);
  if(!factory) return 0;
  AIDA::IHistogram2D* histo = factory->createHistogram2D(aName,aTitle,
    aXnumber,aXmin,aXmax,aYnumber,aYmin,aYmax);
  delete factory;
  return histo;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IHistogram3D* create_histogram3D(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aName
,const std::string& aTitle
,int aXnumber
,double aXmin
,double aXmax
,int aYnumber
,double aYmin
,double aYmax
,int aZnumber
,double aZmin
,double aZmax
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogramFactory* factory = aAIDA.createHistogramFactory(aTree);
  if(!factory) return 0;
  AIDA::IHistogram3D* histo = factory->createHistogram3D(aName,aTitle,
    aXnumber,aXmin,aXmax,aYnumber,aYmin,aYmax,aZnumber,aZmin,aZmax);
  delete factory;
  return histo;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::ICloud2D* create_cloud2D(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aName
,const std::string& aTitle
,int aLimit = -1
,const std::string& aOptions = ""
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogramFactory* factory = aAIDA.createHistogramFactory(aTree);
  if(!factory) return 0;
  AIDA::ICloud2D* cloud = factory->createCloud2D(aName,aTitle,aLimit,aOptions);
  delete factory;
  return cloud;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IDataPointSet* create_dataPointSet(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aName
,const std::string& aTitle
,int aDimension
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IDataPointSetFactory* factory = aAIDA.createDataPointSetFactory(aTree);
  if(!factory) return 0;
  AIDA::IDataPointSet* dps = factory->create(aName,aTitle,aDimension);
  delete factory;
  return dps;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IFitter* create_fitter(
 AIDA::IAnalysisFactory& aAIDA
,const std::string& aEngine = ""
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IFitFactory* factory = aAIDA.createFitFactory(); 
  if(!factory) return 0;
  AIDA::IFitter* fitter = factory->createFitter("",aEngine,"");
  delete factory;
  return fitter;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IFunction* create_function(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,const std::string& aModel
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IFunctionFactory* factory = aAIDA.createFunctionFactory(aTree); 
  if(!factory) return 0;
  AIDA::IFunction* function = factory->createFunctionByName(aPath,aModel);
  delete factory;
  return function;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IFunction* create_scripted_function(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aName
,int aDimension
,const std::string& aScript
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IFunctionFactory* factory = aAIDA.createFunctionFactory(aTree); 
  if(!factory) return 0;
  AIDA::IFunction* function = 
    factory->createFunctionFromScript(aName,aDimension,aScript,"","","");
  delete factory;
  return function;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IFunctionCatalog* find_function_catalog(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IFunctionFactory* factory = aAIDA.createFunctionFactory(aTree); 
  if(!factory) return 0;
  AIDA::IFunctionCatalog* catalog = factory->catalog();
  delete factory;
  return catalog;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::ITuple* create_tuple(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,const std::string& aTitle
,const std::string& aColumnNames
,const std::string& aOptions = ""
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITupleFactory* factory = aAIDA.createTupleFactory(aTree);
  if(!factory) return 0;
  AIDA::ITuple* tuple = factory->create(aPath,aTitle,aColumnNames,aOptions);
  delete factory;
  return tuple;
}
/*
//////////////////////////////////////////////////////////////////////////////
static AIDA::ITuple* copy_tuple(
 AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,const std::string& aPath
,AIDA::ITuple& aTuple
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITupleFactory* factory = aAIDA.createTupleFactory(aTree);
  if(!factory) return 0;
  AIDA::ITuple* tuple = factory->createCopy(aPath,aTuple); //Not yet in AIDA::ITuple
  delete factory;
  return tuple;
}
*/
//////////////////////////////////////////////////////////////////////////////
static AIDA::IPlotter* create_plotter(
 AIDA::IAnalysisFactory& aAIDA
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IPlotterFactory* factory = aAIDA.createPlotterFactory(); 
  if(!factory) return 0;
  AIDA::IPlotter* plotter = factory->create();
  delete factory;
  return plotter;
}
//////////////////////////////////////////////////////////////////////////////
static bool region_parameterValue(
 AIDA::IPlotterRegion& aRegion
,const std::string& aWhat
,double& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = 0;
  std::string s = aRegion.parameterValue(aWhat);
  double value;
  if(!Lib::smanip::todouble(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static bool region_parameterValue(
 AIDA::IPlotterRegion& aRegion
,const std::string& aWhat
,bool& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = false;
  std::string s = aRegion.parameterValue(aWhat);
  bool value;
  if(!Lib::smanip::tobool(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
/// Converters ///////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static AIDA::ICloud* convert_functionToCloud(
 AIDA::IHistogramFactory& aFactory
,const std::string& aName
,const std::string& aTitle
,const AIDA::IFunction& aFunction
,IPrinter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::vector<double>& aparams = aFunction.parameters();

  // Analyse params to see if we have a Array2DFunction :
  bool status = false;
  if(aparams.size()>=3) {
    unsigned int dim = (unsigned int)aparams[0];
    if(dim==2) {
      unsigned int nx = (unsigned int)aparams[1];
      unsigned int ny = (unsigned int)aparams[2];
      if(1+1+1+nx+ny+nx*ny==aparams.size()) {
        status = true;
      }
    }
  }

  if(!status) {
    aPrinter.write("convert_functionToCloud : input function not a Lab::Array2DFunction\n");
    return 0;
  }

  AIDA::ICloud3D* cloud = aFactory.createCloud3D(aName,aTitle); 
  if(!cloud) return 0;

  unsigned int nx = (unsigned int)aparams[1];
  unsigned int ny = (unsigned int)aparams[2];
  
  unsigned int index = 0;
  for(unsigned int j=0;j<ny;j++) {
    double y = aparams[3+nx+j];
    for(unsigned int i=0;i<nx;i++) {
      double x = aparams[3+i];
      double v = aparams[3+nx+ny+index];
      cloud->fill(x,y,v);
      index++;
    }
  }

  return cloud;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IFunction* convert_complexToRealFunction(
 AIDA::IFunctionFactory& aFactory
,const std::string& aName
,const std::string& aTitle
,const AIDA::IFunction& aFunction
,const std::string& aWhat
,IPrinter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::vector<double>& aparams = aFunction.parameters();

  // Analyse params to see if we have a ComplexArrayFunction :
  bool status = false;
  if(aparams.size()>=2) {
    unsigned int dim = (unsigned int)aparams[0];
    if(dim==1) {
      unsigned int nx = (unsigned int)aparams[1];
      if(1+1+nx+2*nx==aparams.size()) {
        status = true;
      }
    }
  }

  if(!status) {
    aPrinter.write("convert_complexToRealFunction : input function not a Lab::ComplexArrayFunction.\n");
    return 0;
  }

  unsigned int number = (unsigned int)aparams[1];
  std::vector<double> params;
  params.push_back(1); //Dimension.
  params.push_back(number); //Number of entries for X axis.

  // Xs :
  unsigned int aparami = 2;
  unsigned int index;
  for(index=0;index<number;index++) {
    params.push_back(aparams[aparami]);
    aparami++;
  }
  // Values :
  if(aWhat=="imag") {
    for(unsigned int index=0;index<number;index++) {
      params.push_back(aparams[aparami+1]);
      aparami += 2;
    }
  } else if(aWhat=="norm") {
    for(unsigned int index=0;index<number;index++) {
      double real = aparams[aparami];
      double imag = aparams[aparami+1];
      params.push_back(real * real + imag * imag);
      aparami += 2;
    }
  } else if(aWhat=="abs") {
    for(unsigned int index=0;index<number;index++) {
      double real = aparams[aparami];
      double imag = aparams[aparami+1];
      params.push_back(::sqrt(real * real + imag * imag));
      aparami += 2;
    }
  } else { // real part :
    for(unsigned int index=0;index<number;index++) {
      params.push_back(aparams[aparami]);
      aparami += 2;
    }
  }

  AIDA::IFunction* function = aFactory.createFunctionByName(aName,"Grid1D");
  if(function) {
    function->setTitle(aTitle);
    function->setParameters(params);
  }

  return function;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IFunction* convert_realToComplexFunction(
 AIDA::IFunctionFactory& aFactory
,const std::string& aName
,const std::string& aTitle
,const AIDA::IFunction& aFunction
,IPrinter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::vector<double>& aparams = aFunction.parameters();

  // Analyse params to see if we have a ArrayFunction :
  bool status = false;
  if(aparams.size()>=2) {
    unsigned int dim = (unsigned int)aparams[0];
    if(dim==1) {
      unsigned int nx = (unsigned int)aparams[1];
      if(1+1+nx+nx==aparams.size()) {
        status = true;
      }
    }
  }

  if(!status) {
    aPrinter.write("convert_realToComplexFunction : input function not a Lab::ArrayFunction.\n");
    return 0;
  }

  unsigned int nx = (unsigned int)aparams[1];

  // ComplexGrid 1D function :
  std::vector<double> params;
  params.push_back(1); //Dimension.
  params.push_back(nx); //Number of entries for X axis.

  unsigned int aparami = 2;

  unsigned int index;
  for(index=0;index<nx;index++) {
    params.push_back(aparams[aparami]);
    aparami++;
  }
  for(index=0;index<nx;index++) {
    params.push_back(aparams[aparami]);
    aparami++;
    params.push_back(0);
  }

  AIDA::IFunction* function = 
    aFactory.createFunctionByName(aName,"ComplexGrid1D");
  if(function) {
    function->setTitle(aTitle);
    function->setParameters(params);
  }

  return function;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::ICloud* convert_tupleToCloud(
 AIDA::IHistogramFactory& aFactory
,const std::string& aName
,const std::string& aTitle
,AIDA::ITuple& aTuple
,AIDA::IEvaluator& aEvaluatorX
,AIDA::IEvaluator& aEvaluatorY
,AIDA::IFilter& aFilter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ICloud2D* cloud = aFactory.createCloud2D(aName,aTitle); 
  if(!cloud) return 0;

  if(!aEvaluatorX.initialize(aTuple)) return 0;
  if(!aEvaluatorY.initialize(aTuple)) return 0;
  if(!aFilter.initialize(aTuple)) return 0;

  aTuple.start();
  while(aTuple.next()) {
    if(!aFilter.accept()) continue;
    cloud->fill(aEvaluatorX.evaluateDouble(),aEvaluatorY.evaluateDouble());
  }

  return cloud;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IFunction* convert_tupleToFunction(
 AIDA::IFunctionFactory& aFactory
,const std::string& aName
,const std::string& aTitle
,AIDA::ITuple& aTuple
,const std::string& aX
,AIDA::IEvaluator& aEvaluator
,AIDA::IFilter& aFilter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aEvaluator.initialize(aTuple)) return 0;
  if(!aFilter.initialize(aTuple)) return 0;

  int arrayX = aTuple.findColumn(aX);

  std::vector<double> Xs;
  std::vector<double> Vs;

  int NotFound = -1;

  // Loop on entries :
  aTuple.start();
  int row = 0;
  while(aTuple.next()) {

    if(arrayX!=NotFound) {
      Xs.push_back(aTuple.getDouble(arrayX));
    } else {
      Xs.push_back(row);
    }

    if(aFilter.accept()) {
      Vs.push_back(aEvaluator.evaluateDouble());
    } else {
      // Not taken.
      Vs.push_back(0);
    }

    row++;
  }

  // Grid 1D function :
  std::vector<double> params;
  params.push_back(1); //Dimension.
  params.push_back(row); //Number of entries for X axis.
  // Xs :
  int index;
  for(index=0;index<row;index++) params.push_back(Xs[index]);
  // Vs :
  for(index=0;index<row;index++) params.push_back(Vs[index]);

  AIDA::IFunction* function = aFactory.createFunctionByName(aName,"Grid1D");
  if(function) {
    function->setTitle(aTitle);
    function->setParameters(params);
  }

  return function;
}
//////////////////////////////////////////////////////////////////////////////
static void Lab_AIDA_quiet_cxx()
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  IPrinter* printer = 0;
  // AIDA :
  AIDA::ITree* tree = 0;
  find_baseHistogram(*tree,"");
  find_histogram1D(*tree,"");
  find_histogram2D(*tree,"");
  find_function(*tree,"");
  find_tuple(*tree,"");

  AIDA::IHistogramFactory* factory = 0;
  AIDA::IFunction* function = 0;
  convert_functionToCloud(*factory,"","",*function,*printer);
  AIDA::IFunctionFactory* ffactory = 0;
  convert_complexToRealFunction(*ffactory,"","",*function,"",*printer);

  AIDA::IAnalysisFactory* aida = 0;
  find_memoryTree(*aida);
  create_tree(*aida,"","",false,false,"");
  create_histogram1D(*aida,*tree,"","",0,0,0);
  create_histogram2D(*aida,*tree,"","",0,0,0,0,0,0);
  create_histogram3D(*aida,*tree,"","",0,0,0,0,0,0,0,0,0);
  create_dataPointSet(*aida,*tree,"","",0);
  create_tuple(*aida,*tree,"","","","");
  create_fitter(*aida);
  create_cloud2D(*aida,*tree,"","");
  create_function(*aida,*tree,"","");
  create_scripted_function(*aida,*tree,"",0,"");
  create_plotter(*aida);

  find_function_catalog(*aida,*tree);

  AIDA::IHistogram1D* h1d = 0;
  copy_histogram1D(*aida,*tree,"",*h1d);
  AIDA::IHistogram2D* h2d = 0;
  copy_histogram2D(*aida,*tree,"",*h2d);
  copy_baseHistogram(*aida,*tree,"",*h1d);
  histogram_name(*h1d);

  convert_realToComplexFunction(*ffactory,"","",*function,*printer);

  AIDA::ITuple* tuple = 0;
  AIDA::IEvaluator* evaluator = 0;
  AIDA::IEvaluator* evaluatorY = 0;
  AIDA::IFilter* filter = 0;
  convert_tupleToCloud(*factory,"","",*tuple,*evaluator,*evaluatorY,*filter);
  convert_tupleToFunction(*ffactory,"","",*tuple,"",*evaluator,*filter);

  tuple_name(*tuple);

  AIDA::IPlotterRegion* region = 0;
  double d;
  region_parameterValue(*region,"",d);
  bool b;
  region_parameterValue(*region,"",b);
}

#endif
