#ifndef Lab_Callbacks_Lab_h
#define Lab_Callbacks_Lab_h

//
//  This file contains static helper methods.
// It is intended to be included in OnX Callbacks file
// to ease callbacks programming.
//FIXME  It relies on interfaces and basic libraries (Lib, stdcxx) only.
//

#include <OnX/Interfaces/IUI.h>

#include <Lib/Interfaces/ISession.h>
#include <Lib/smanip.h>
#include <Lib/Out.h>
#include <Lib/sout.h>

#include <AIDA/AIDA.h>

//////////////////////////////////////////////////////////////////////////////
static AIDA::IManagedObject* mainTree_selection(
 IUI& aUI
,IManager*& aManager
,std::string& aPath
,std::string& aName
) 
//////////////////////////////////////////////////////////////////////////////
// FIXME : We do not return the AIAD::ITree, since we need manager->name()
//   in Callbacks.cxx. We need a name() over the AIDA::ITree.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aManager = 0;
  aPath = "";
  aName = "";

  std::string selection;
  if(!aUI.parameterValue("mainTree.selection",selection)) {
    Lib::Out out(aUI.session().printer());
    out << "mainTree_selection :"
        << " mainTree.selection parameter not found." 
        << Lib::endl;
    return 0;
  }
  if(selection=="") return 0;

  ISession& session = aUI.session();

  if(session.verboseLevel()) {
    Lib::Out out(aUI.session().printer());
    out << "mainTree_selection :"
        << " selection is " << Lib::sout(selection) << "." 
        << Lib::endl;
  }

  std::vector<std::string> items;
  //WARNING : Do not use Lib::smanip::lines. Some Windows file name
  // may contain a \n which is not a delimiter ; 
  // like in ..\examples\data\ntuples.hbook
  Lib::smanip::words(selection,"\n",items,true);
  if(items.size()<2) {
    Lib::Out out(aUI.session().printer());
    out << "mainTree_selection :"
        << " two items expected in " << Lib::sout(selection)
        << Lib::endl;
    return 0;
  }

  // First item is the tree name :
  std::string name = items[0];
  IManager* manager = session.findManager(name);
  if(!manager) {
    Lib::Out out(aUI.session().printer());
    out << "mainTree_selection :"
        << " manager " << Lib::sout(name) << " not found."
        << Lib::endl;
    return 0;
  }
  AIDA::ITree* tree = (AIDA::ITree*)manager->cast("AIDA::ITree");
  if(!tree) {
    Lib::Out out(aUI.session().printer());
    out << "mainTree_selection :"
        << " manager " << Lib::sout(name) << " is not an AIDA::ITree."
        << Lib::endl;
    return 0;
  }

  // Get dirname, basename of the item :

  // Get dirname :
  std::string dirname = "/";
  for(unsigned int index=1;index<items.size()-1;index++) {
    dirname += items[index] + "/";
  }

  // Get basenmae. The last item is of the form :
  //     <basename> class=[AIDA::<class> title="<title>"]
  // Note that <basename> may contain spaces.
  std::string basename = items[items.size()-1];
  std::string::size_type pos = basename.find("class=");
  if(pos!=std::string::npos) {
    basename = basename.substr(0,pos);
    Lib::smanip::strip(basename);
  }

  aManager = manager;
  aPath = dirname+basename;
  aName = basename;

  AIDA::IManagedObject* object = tree->find(aPath);
  if(!object) {
    Lib::Out out(aUI.session().printer());
    out << "mainTree_selection :"
        << " object " << Lib::sout(aPath) << " not found in AIDA::ITree "
        << Lib::sout(name)
        << Lib::endl;
    return 0;
  }
  return object;
}

#endif
