#ifndef Lab_Callbacks_Vector_h
#define Lab_Callbacks_Vector_h

//
//  This file contains static helper methods.
//

#include <Lib/Interfaces/IPrinter.h>
#include <Lib/smanip.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/Cast.h>

#include <AIDA/ITree.h>
#include <AIDA/IManagedObject.h>

#include <Lab/Interfaces/IVector.h>

//////////////////////////////////////////////////////////////////////////////
static void print_vector(
 const Lab::IVector& aVector
,IPrinter& aPrinter
,const std::string& aTitle
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aPrinter);
  if(aTitle!="") out << aTitle << Lib::endl;
  if(aVector.dimension()==1) {
    for(int i=0;i<aVector.orders()[0];i++) {
      std::vector<int> is(1);
      is[0] = i;
      double value;
      aVector.value(is,value);
      out << value << Lib::endl;
    }
  } else if(aVector.dimension()==2) {
    for(int j=0;j<aVector.orders()[1];j++) {
      for(int i=0;i<aVector.orders()[0];i++) {
        std::vector<int> is(2);
        is[0] = i;
        is[1] = j;
        double value;
        aVector.value(is,value);
        std::string s;
        Lib::smanip::printf(s,32,"%g",value);
        Lib::smanip::justify(s,5);
        out << s;
      }
      out << Lib::endl;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
static Lab::IVector* find_vector(
 AIDA::ITree& aTree
,const std::string& aName
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IManagedObject* object = aTree.find(aName);
  if(!object) return 0;
  return Lib_Cast(*object,Lab::IVector);
}
//////////////////////////////////////////////////////////////////////////////
static bool vector_set_dims(
 const Lab::IVector& aVector
,std::vector< std::pair<int,int> >& aDims
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int dim = aVector.dimension();
  if(aDims.size()) {
    if(aDims.size()!=dim) return false;
  } else {
    for(unsigned int index = 0;index<dim;index++) {
      int mn = 0;
      int mx = aVector.orders()[index] -1;
      aDims.push_back( std::pair<int,int>(mn,mx));
    }
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static void Lab_Vector_quiet_cxx()
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  IPrinter* printer = 0;
  Lab::IVector* vector = 0;
  print_vector(*vector,*printer,"");
  AIDA::ITree* tree = 0;
  find_vector(*tree,"");
  std::vector< std::pair<int,int> > dims;
  vector_set_dims(*vector,dims);
}

#endif
