#ifndef Lab_Callbacks_hprint_h
#define Lab_Callbacks_hprint_h

//
//  This file contains static helper methods.
// It is intended to be included in OnX Callbacks file
// to ease callbacks programming.
//  It relies on interfaces and basic libraries (Lib, c++) only.
//

#include <AIDA/IManagedObject.h>
#include <AIDA/IHistogram1D.h>
#include <AIDA/IHistogram2D.h>
#include <AIDA/IProfile1D.h>
#include <AIDA/IAxis.h>
#include <AIDA/ITuple.h>
#include <AIDA/IFunction.h>
#include <AIDA/ICloud1D.h>
#include <AIDA/ICloud2D.h>
#include <AIDA/ICloud3D.h>
#include <AIDA/IDataPointSet.h>

// Have to link with Lib.
#include <Lib/Out.h>
#include <Lib/smanip.h>
#include <Lib/Cast.h>

//////////////////////////////////////////////////////////////////////////////
static void hprint(
 AIDA::IHistogram1D& aH1D
,const std::string& aName
,IPrinter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
// A la HPRINT.
//
// From cern/src/packlib/hbook/code/hprst.F :
// 3000 FORMAT(' * ENTRIES =',I11,6X,'* ALL CHANNELS =',E11.4,6X,
//     +'* UNDERFLOW =',E11.4,6X,'* OVERFLOW =',E11.4,/,
//     +' * BIN WID =',E11.4,6X,'* MEAN VALUE   =',E11.4,6X,
//    +'* R . M . S =',E11.4)
// 4000 FORMAT(' * ENTRIES =',I11,6X,'* ALL CHANNELS =',E11.4,6X,
//     +'* UNDERFLOW =',E11.4,6X,'* OVERFLOW =',E11.4,/,
//     +' * BIN WID =',E11.4,6X,'* MEAN VALUE   =',E11.4,6X,
//     +'* R . M . S =',E11.4,6X,'* NEQUIVAL =',E11.4)
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aPrinter);

  int field = 13; //HBOOK had 17.

  if(!aH1D.allEntries()) {

    out << "****** AIDA   NO ENTRIES FOR HISTOGRAM = " << aName
        << Lib::endl;

  } else {

    out << aH1D.dimension() << "D " << aH1D.title() << Lib::endl;
    out << " * NAME = " << aName << Lib::endl;

    out << " * ENTRIES = " << Lib::sjust(aH1D.allEntries(),field)
        << " * ALL CHANNELS = " << Lib::sjust(aH1D.sumBinHeights(),field)
        << " * UNDERFLOW = " << Lib::sjust(aH1D.binHeight(AIDA::IAxis::UNDERFLOW_BIN),field)
        << " * OVERFLOW = " << Lib::sjust(aH1D.binHeight(AIDA::IAxis::OVERFLOW_BIN),field)
        << Lib::endl;
    out << " * BIN WID = " << Lib::sjust(aH1D.axis().binWidth(0),field)
        << " * MEAN VALUE   = " << Lib::sjust(aH1D.mean(),field)
        << " * R . M . S = " << Lib::sjust(aH1D.rms(),field)
        << Lib::endl;
    out << " * MIN = " << Lib::sjust(aH1D.minBinHeight(),field+4)
        << " * MAX = " << Lib::sjust(aH1D.maxBinHeight(),field)
        << Lib::endl;
      
  }
}

//////////////////////////////////////////////////////////////////////////////
static void hprint(
 AIDA::IManagedObject& aObject
,IPrinter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
// A la HPRINT.
//
// From cern/src/packlib/hbook/code/hprst.F :
// 3000 FORMAT(' * ENTRIES =',I11,6X,'* ALL CHANNELS =',E11.4,6X,
//     +'* UNDERFLOW =',E11.4,6X,'* OVERFLOW =',E11.4,/,
//     +' * BIN WID =',E11.4,6X,'* MEAN VALUE   =',E11.4,6X,
//    +'* R . M . S =',E11.4)
// 4000 FORMAT(' * ENTRIES =',I11,6X,'* ALL CHANNELS =',E11.4,6X,
//     +'* UNDERFLOW =',E11.4,6X,'* OVERFLOW =',E11.4,/,
//     +' * BIN WID =',E11.4,6X,'* MEAN VALUE   =',E11.4,6X,
//     +'* R . M . S =',E11.4,6X,'* NEQUIVAL =',E11.4)
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aPrinter);

  int field = 13; //HBOOK had 17.

  if(AIDA::IHistogram1D* h1 = Lib_Cast(aObject,AIDA::IHistogram1D)) {

    hprint(*h1,aObject.name(),aPrinter);

  } else if(AIDA::IProfile1D* h1 = Lib_Cast(aObject,AIDA::IProfile1D)) {

    if(!h1->allEntries()) {

      out << "****** AIDA   NO ENTRIES FOR PROFILE = " << aObject.name()
          << Lib::endl;

    } else {

      out << h1->dimension() << "D " << h1->title() << Lib::endl;
      out << " * NAME = " << aObject.name() << Lib::endl;

      out << " * ENTRIES = " << Lib::sjust(h1->allEntries(),field)
          << " * ALL CHANNELS = " << Lib::sjust(h1->sumBinHeights(),field)
          << " * UNDERFLOW = " << Lib::sjust(h1->binHeight(AIDA::IAxis::UNDERFLOW_BIN),field)
          << " * OVERFLOW = " << Lib::sjust(h1->binHeight(AIDA::IAxis::OVERFLOW_BIN),field)
          << Lib::endl;
      out << " * BIN WID = " << Lib::sjust(h1->axis().binWidth(0),field)
          << " * MEAN VALUE   = " << Lib::sjust(h1->mean(),field)
          << " * R . M . S = " << Lib::sjust(h1->rms(),field)
          << Lib::endl;
      out << " * MIN = " << Lib::sjust(h1->minBinHeight(),field+4)
          << " * MAX = " << Lib::sjust(h1->maxBinHeight(),field)
          << Lib::endl;
      
    }

   } else if(AIDA::IHistogram2D* h2 = Lib_Cast(aObject,AIDA::IHistogram2D)) {

    if(!h2->allEntries()) {

      out << "****** AIDA   NO ENTRIES FOR HISTOGRAM = " << aObject.name()
          << Lib::endl;

    } else {

      out << h2->dimension() << "D " << h2->title() << Lib::endl;
      out << " * NAME = " << aObject.name() << Lib::endl;

      out << " * ENTRIES = " << h2->allEntries() << Lib::endl;

      //   6 | 7 | 8
      //  -----------
      //   3 | 4 | 5
      //  -----------
      //   0 | 1 | 2

      double height_0 = h2->binHeight(AIDA::IAxis::UNDERFLOW_BIN,
                                     AIDA::IAxis::UNDERFLOW_BIN);
      double height_2 = h2->binHeight(AIDA::IAxis::OVERFLOW_BIN,
                                     AIDA::IAxis::UNDERFLOW_BIN);
      double height_6 = h2->binHeight(AIDA::IAxis::UNDERFLOW_BIN,
                                     AIDA::IAxis::OVERFLOW_BIN);
      double height_8 = h2->binHeight(AIDA::IAxis::OVERFLOW_BIN,
                                     AIDA::IAxis::OVERFLOW_BIN);
      
      int i,j;
      
      double height_1 = 0;
      double height_7 = 0;
      for(i=0;i<h2->xAxis().bins();i++){
        height_1 += h2->binHeight(i,AIDA::IAxis::UNDERFLOW_BIN);
        height_7 += h2->binHeight(i,AIDA::IAxis::OVERFLOW_BIN);
      }
      
      double height_3 = 0;
      double height_5 = 0;
      for(j=0;j<h2->yAxis().bins();j++){
        height_3 += h2->binHeight(AIDA::IAxis::UNDERFLOW_BIN,j);
        height_5 += h2->binHeight(AIDA::IAxis::OVERFLOW_BIN,j);
      }

      double height_4 = 0;
      for(i=0;i<h2->xAxis().bins();i++){
        for(j=0;j<h2->yAxis().bins();j++){
          height_4 += h2->binHeight(i,j);
        }
      }
      
      std::string bar(32,'-');
      out << Lib::sjust(height_6,10,Lib::smanip::middle)
          << "|" << Lib::sjust(height_7,10,Lib::smanip::middle) 
          << "|" << Lib::sjust(height_8,10,Lib::smanip::middle) 
          << Lib::endl;
      out << bar << Lib::endl;
      out << Lib::sjust(height_3,10,Lib::smanip::middle)
          << "|" << Lib::sjust(height_4,10,Lib::smanip::middle)
          << "|" << Lib::sjust(height_5,10,Lib::smanip::middle)
          << Lib::endl;
      out << bar << Lib::endl;
      out << Lib::sjust(height_0,10,Lib::smanip::middle)
          << "|" << Lib::sjust(height_1,10,Lib::smanip::middle)
          << "|" << Lib::sjust(height_2,10,Lib::smanip::middle)
          << Lib::endl;

      out << " * MIN = " << Lib::sjust(h2->minBinHeight(),field)
          << " * MAX = " << Lib::sjust(h2->maxBinHeight(),field)
          << Lib::endl;
    }


  } else if(AIDA::IFunction* f = Lib_Cast(aObject,AIDA::IFunction)) {

    std::vector<std::string> names = f->parameterNames();
    std::vector<double> vals = f->parameters();

    out << f->dimension() << "D " << f->title() << Lib::endl;
    out << " * NAME = " << aObject.name() << Lib::endl;

    unsigned int number = names.size();
    for(unsigned int index=0;index<number;index++){
      out << " * PARAM   = " << Lib::sjust(names[index],field)
          << " * VALUE = " << Lib::sjust(vals[index],field)
          << Lib::endl;
    }
    out << " * CODELET = " << f->codeletString() << Lib::endl;

  } else if(AIDA::ITuple* tuple = Lib_Cast(aObject,AIDA::ITuple)) {

    out << " ********************************************************" 
        << Lib::endl;

    out << " * NTUPLE NAME= " << aObject.name() 
        << " ENTRIES= " << tuple->rows()
        << " TITLE= " << tuple->title()
        << Lib::endl;

    out << " ********************************************************" 
        << Lib::endl;
    out << " *  Var numb  *   Name    *    Lower     *    Upper     *" 
        << Lib::endl;
    out << " ********************************************************" 
        << Lib::endl;

    
    int coln = tuple->columns();
    for(int coli=0;coli<coln;coli++) {
      out << " *" 
          << Lib::sjust(Lib::smanip::tostring(coli+1),12,Lib::smanip::middle) 
          << "* " 
          << Lib::sjust(tuple->columnName(coli),10) 
          << "* " 
          << Lib::sjust(Lib::smanip::tostring(tuple->columnMin(coli)),13) 
          << "* " 
          << Lib::sjust(Lib::smanip::tostring(tuple->columnMax(coli)),13) 
          << "*" 
          << Lib::endl;
    }

    out << " ********************************************************" 
        << Lib::endl;

/*

 ********************************************************
 * NTUPLE ID=   10  ENTRIES=   3354   CERN Population 
 ********************************************************
 *  Var numb  *   Name    *    Lower     *    Upper     *
 ********************************************************
 *      1     * CATEGORY  * 0.102000E+03 * 0.567000E+03 *
 *      2     * DIVISION  * 0.100000E+01 * 0.130000E+02 *
 *      3     * FLAG      * 0.000000E+00 * 0.310000E+02 *
 *      4     * AGE       * 0.210000E+02 * 0.640000E+02 *
 *      5     * SERVICE   * 0.000000E+00 * 0.350000E+02 *
 *      6     * CHILDREN  * 0.000000E+00 * 0.600000E+01 *
 *      7     * GRADE     * 0.300000E+01 * 0.140000E+02 *
 *      8     * STEP      * 0.000000E+00 * 0.150000E+02 *
 *      9     * NATION    * 0.100000E+01 * 0.150000E+02 *
 *     10     * HRWEEK    * 0.200000E+01 * 0.440000E+02 *
 *     11     * COST      * 0.686000E+03 * 0.188530E+05 *
 ********************************************************

*/

  } else if(AIDA::ICloud* cl = Lib_Cast(aObject,AIDA::ICloud)) {

    if(!cl->entries()) {

      out << "****** AIDA   NO ENTRIES FOR CLOUD = " << aObject.name()
          << Lib::endl;

    } else {

      out << cl->dimension() << "D " << cl->title() << Lib::endl;
      out << " * NAME = " << aObject.name() << Lib::endl;

      out << " * ENTRIES = " << Lib::sjust(cl->entries(),field)
          << Lib::endl;
      
      out << " * SUM OF WEIGHTS = " << Lib::sjust(cl->sumOfWeights(),field)
          << Lib::endl;

      out << " * IS CONVERTED TO HISTOGRAM = " 
          << Lib::smanip::tostring(cl->isConverted())
          << Lib::endl;

      if(AIDA::ICloud1D* cl1D = Lib_Cast(aObject,AIDA::ICloud1D)) {

        out << " * MEAN VALUE = " << Lib::sjust(cl1D->mean(),field)
            << " * R . M . S = " << Lib::sjust(cl1D->rms(),field)
            << Lib::endl;

        out << " * MIN = " << Lib::sjust(cl1D->lowerEdge(),field+7)
            << " * MAX = " << Lib::sjust(cl1D->upperEdge(),field)
            << Lib::endl;

      } else if(AIDA::ICloud2D* cl2D = Lib_Cast(aObject,AIDA::ICloud2D)) {

        out << " * X MEAN VALUE = " << Lib::sjust(cl2D->meanX(),field)
            << " * X R . M . S = " << Lib::sjust(cl2D->rmsX(),field)
            << Lib::endl;

        out << " * X MIN = " << Lib::sjust(cl2D->lowerEdgeX(),field+7)
            << " * X MAX = " << Lib::sjust(cl2D->upperEdgeX(),field)
            << Lib::endl;

        out << " * Y MEAN VALUE = " << Lib::sjust(cl2D->meanY(),field)
            << " * Y R . M . S = " << Lib::sjust(cl2D->rmsY(),field)
            << Lib::endl;

        out << " * Y MIN = " << Lib::sjust(cl2D->lowerEdgeY(),field+7)
            << " * Y MAX = " << Lib::sjust(cl2D->upperEdgeY(),field)
            << Lib::endl;

      } else if(AIDA::ICloud3D* cl3D = Lib_Cast(aObject,AIDA::ICloud3D)) {

        out << " * X MEAN VALUE = " << Lib::sjust(cl3D->meanX(),field)
            << " * X R . M . S = " << Lib::sjust(cl3D->rmsX(),field)
            << Lib::endl;

        out << " * X MIN = " << Lib::sjust(cl3D->lowerEdgeX(),field+7)
            << " * X MAX = " << Lib::sjust(cl3D->upperEdgeX(),field)
            << Lib::endl;

        out << " * Y MEAN VALUE = " << Lib::sjust(cl3D->meanY(),field)
            << " * Y R . M . S = " << Lib::sjust(cl3D->rmsY(),field)
            << Lib::endl;

        out << " * Y MIN = " << Lib::sjust(cl3D->lowerEdgeY(),field+7)
            << " * Y MAX = " << Lib::sjust(cl3D->upperEdgeY(),field)
            << Lib::endl;

        out << " * Z MEAN VALUE = " << Lib::sjust(cl3D->meanZ(),field)
            << " * Z R . M . S = " << Lib::sjust(cl3D->rmsZ(),field)
            << Lib::endl;

        out << " * Z MIN = " << Lib::sjust(cl3D->lowerEdgeZ(),field+7)
            << " * Z MAX = " << Lib::sjust(cl3D->upperEdgeZ(),field)
            << Lib::endl;
      }
    }

  } else if(AIDA::IDataPointSet* dps = Lib_Cast(aObject,AIDA::IDataPointSet)) {

    if(!dps->size()) {

      out << "****** AIDA   NO ENTRIES FOR DATA POINT SET = " << aObject.name()
          << Lib::endl;

    } else {

      out << dps->dimension() << "D " << dps->title() << Lib::endl;
      out << " * NAME = " << aObject.name() << Lib::endl;

      out << " * ENTRIES = " << Lib::sjust(dps->size(),field)
          << Lib::endl;
      
    }

  } else {

    out << "****** AIDA NO INFOS FOR OBJECT = " << aObject.name()
        << Lib::endl;

  }
}
//////////////////////////////////////////////////////////////////////////////
static void hprint_quiet_cxx()
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IHistogram1D* h = 0;
  IPrinter* p = 0;
  AIDA::IManagedObject* mo = 0;
  hprint(*h,"",*p);
  hprint(*mo,*p);
} 

#endif
