#ifndef Lab_AnalysisFactory_h
#define Lab_AnalysisFactory_h

#include <AIDA/IAnalysisFactory.h>

class ISession;
class IPrinter;
#include <Lib/Interfaces/IManager.h>

namespace Lab {

// A factory manager.

class AnalysisFactory 
:public virtual IManager
,public virtual AIDA::IAnalysisFactory {
public: //AIDA/IAnalysisFactory
  virtual AIDA::ITreeFactory* createTreeFactory();
  virtual AIDA::IHistogramFactory* createHistogramFactory(AIDA::ITree&);
  virtual AIDA::ITupleFactory* createTupleFactory(AIDA::ITree&);
  virtual AIDA::IFunctionFactory* createFunctionFactory(AIDA::ITree&);
  virtual AIDA::IPlotterFactory* createPlotterFactory(int,char**,
                                                      const std::string&,
                                                      const std::string&);
  virtual AIDA::IDataPointSetFactory* createDataPointSetFactory(AIDA::ITree&);
  virtual AIDA::IFitFactory* createFitFactory();
public: // IManager :
  virtual const std::string& name() const;
  virtual void* cast(const std::string&) const;
public:
  AnalysisFactory(ISession&);
  virtual ~AnalysisFactory();
private:
  ISession& fSession;
  IPrinter& fPrinter;
  std::string fName;
  AIDA::IFunctionFactory* fFunctionFactory;
};

}

#endif
