#ifndef Lab_BaseFitter_h
#define Lab_BaseFitter_h

// Inheritance :
#include <AIDA/IFitter.h>

namespace AIDA {
  class IFunctionFactory;
}

class IPrinter;

#include <Lab/Core/FitParameterSettings.h>

namespace Lab {

class BaseFitter : public virtual AIDA::IFitter {
public: //AIDA::IFitter
  virtual AIDA::IFitResult* fit(AIDA::IFitData&,AIDA::IFunction&);
  virtual AIDA::IFitResult* fit(AIDA::IBaseHistogram&,AIDA::IFunction&);
  virtual AIDA::IFitResult* fit(AIDA::IBaseHistogram&,const std::string&);
  virtual AIDA::IFitResult* fit(AIDA::IBaseHistogram&,const std::string&,
                                const std::vector<double>&);
  virtual AIDA::IFitResult* fit(AIDA::IDataPointSet&,AIDA::IFunction&);
  virtual AIDA::IFitResult* fit(AIDA::IDataPointSet&,const std::string&);
  virtual AIDA::IFitResult* fit(AIDA::IDataPointSet&,const std::string&,
                                const std::vector<double>&);
  virtual AIDA::IFitResult* fit(AIDA::IFitData&,const std::string&);
  virtual AIDA::IFitResult* fit(AIDA::IFitData&,const std::string&,
                                const std::vector<double>&);
  virtual bool setEngine(const std::string&);
  virtual std::string engineName();
  virtual AIDA::IFitParameterSettings& fitParameterSettings(const std::string&);
  virtual std::vector<std::string> listParameterSettings();
  virtual void resetParameterSettings();
  virtual bool setConstraint(const std::string&);
  virtual std::vector<std::string> constraints();
  virtual void resetConstraints();
  virtual bool setFitMethod(const std::string&);
  virtual std::string fitMethodName();
  virtual AIDA::IDataPointSet* createScan1D(AIDA::IFitData&,
                                            AIDA::IFunction&,
                                            const std::string&,
                                            int,double,double);
  virtual AIDA::IDataPointSet* createContour(AIDA::IFitData&,
                                             AIDA::IFitResult&,
                                             const std::string&,
                                             const std::string&,int,double);
  virtual void setUseFunctionGradient(bool);
  virtual bool useFunctionGradient();
public: //Lab::BaseFitter :
  virtual bool baseFit(AIDA::IFunction&,
                       const std::vector<double>&,
                       const std::vector<double>&,
                       const std::vector<double>&,
                       double&,std::vector<double>&) = 0;
public:
  BaseFitter(IPrinter&,AIDA::IFunctionFactory&,const std::string&);
  virtual ~BaseFitter();
  virtual void* cast(const std::string&) const;
private:
  static double userFunction(double,const std::vector<double>&,void*);
protected:
  IPrinter& fPrinter;
private:
  AIDA::IFunctionFactory& fFunctionFactory;
  std::string fEngineName;
  std::vector<std::string> fParameterSettings;
  std::vector<std::string> fConstraints;
  std::string fFitMethodName;
  int fCounter;
  std::vector<FitParameterSettings> fFitParameterSettings;
};

}

#endif
