#ifndef Lab_BaseTuple_h
#define Lab_BaseTuple_h

// Inheritance :
#include <Lab/Core/BaseObject.h>
#include <AIDA/ITuple.h>

#include <Lib/Variable.h>
#include <Lab/Core/Annotation.h>

namespace Lab {

class BaseColumn {
public:
  virtual int size() const = 0;
  virtual void reset() = 0;
  virtual void add() = 0;
  virtual BaseColumn* copy() const = 0;
  virtual void resetFilled() = 0;
  virtual void* cast(const std::string&) const;
  virtual std::string defaultValue() const = 0;
public:
  BaseColumn(IPrinter&,const std::string&,const std::string&);
  BaseColumn(const BaseColumn&);
  virtual ~BaseColumn();
  const std::string& label() const;
  const std::string& type() const;
protected:
  IPrinter& fPrinter;
private:
  std::string fLabel;
  std::string fType;
};

class BaseTuple : public BaseObject, public virtual AIDA::ITuple {
public: //AIDA/ITuple
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation & annotation();
  virtual const AIDA::IAnnotation & annotation() const;
  virtual int findColumn(const std::string&) const;
  virtual void resetRow();
  virtual void reset();
  virtual void start();
  virtual bool skip(int);
  virtual bool next();
  virtual bool setRow(int);
  virtual int columns() const;
  virtual std::string columnName(int) const;
  virtual std::vector<std::string>  columnNames() const;
  virtual std::string columnType(int) const;
  virtual std::vector<std::string>  columnTypes() const;
  virtual bool project(AIDA::IHistogram1D&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram1D&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::IHistogram1D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram1D&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::IHistogram2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::IHistogram3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IHistogram3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud1D&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud1D&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::ICloud1D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud1D&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::ICloud2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::ICloud3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::ICloud3D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);

  virtual bool project(AIDA::IProfile1D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IProfile1D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::IProfile1D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IProfile1D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IProfile2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IProfile2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&);
  virtual bool project(AIDA::IProfile2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&);
  virtual bool project(AIDA::IProfile2D&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IEvaluator&,
                       AIDA::IFilter&,
                       AIDA::IEvaluator&);
public:
  virtual std::string columnDefault(int) const;
  virtual unsigned char getByte(int) const = 0;
  //FIXME : AIDA-33. Put a 33 in order to avoid "hidden" warnings.
  virtual bool fill33(int column, unsigned char value) = 0;
public:
  BaseTuple(IPrinter&,const std::string&,const std::string&);
  BaseTuple(const std::string&,const BaseTuple&);
  virtual ~BaseTuple();
protected:
  static bool booking(IPrinter&,const std::string&,std::list<Lib::Variable>&);
protected:
  void clear();
  std::string convertType(const std::string&) const; 
  BaseColumn* findColumn1(int) const;
private:
  std::string fTitle;
protected:
  int fRow;
  Annotation fAnnotation;
  std::vector<BaseColumn*> fColumns;
};

}

#endif
