#ifndef Lab_DataPointSet_h
#define Lab_DataPointSet_h

// Inheritance :
#include <Lab/Core/BaseObject.h>
#include <AIDA/IDataPointSet.h>

#include <Lab/Core/Annotation.h>
#include <Lab/Core/DataPoint.h>

#include <list>

namespace Lab {

class DataPointSet : public BaseObject, public virtual AIDA::IDataPointSet {
public: //AIDA/IManagedObject
  virtual void* cast(const std::string&) const;
public: //AIDA/IDataPointSet
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation& annotation();
  virtual const AIDA::IAnnotation& annotation() const;
  virtual int dimension() const;
  virtual void clear();
  virtual int size() const;
  virtual AIDA::IDataPoint* point(int);
  virtual const AIDA::IDataPoint* point(int) const;
  virtual AIDA::IDataPoint* addPoint();
  virtual bool addPoint(const AIDA::IDataPoint&);
  virtual bool removePoint(int);
  virtual double lowerExtent(int) const;
  virtual double upperExtent(int) const;
  virtual bool scale(double);
  virtual bool scaleValues(double);
  virtual bool scaleErrors(double);
  virtual bool setCoordinate(int coord, 
                             const std::vector<double>&,
                             const std::vector<double>&);
  virtual bool setCoordinate(int coord, 
                             const std::vector<double>&,
                             const std::vector<double>&,
                             const std::vector<double>&);
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public:
  DataPointSet(IPrinter&,
               const std::string&,
               const std::string&,
               int);
  DataPointSet(const std::string&,const DataPointSet&);
  virtual ~DataPointSet();
private:
  std::string fTitle;
  Annotation fAnnotation;
  int fDimension;
  std::list<DataPoint> fPoints;
};

}

#endif
