#ifndef Lab_Function_h
#define Lab_Function_h

// Inheritance :
#include <Lab/Core/BaseObject.h>
#include <AIDA/IFunction.h>

#include <Lib/Functions.h>

#include <Lab/Core/Annotation.h>

namespace Lib {
  class IFunction;
}

namespace Lab {

class BaseFunction : public BaseObject, public virtual AIDA::IFunction {
public: //AIDA/IManagedObject
  virtual void* cast(const std::string&) const;
public: //AIDA/IFunction
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation& annotation();
  virtual const AIDA::IAnnotation& annotation() const;
  virtual bool isEqual(const IFunction&) const;
  virtual const std::vector<double>& gradient(const std::vector<double>&) const;  virtual bool providesGradient() const;
  virtual std::string variableName(int) const;
  virtual std::vector<std::string> variableNames() const;
  virtual std::vector<std::string> parameterNames() const;
  virtual const std::vector<double>& parameters() const;
  virtual int numberOfParameters() const;
  virtual int indexOfParameter(const std::string&) const;
  virtual std::string codeletString() const;
  virtual bool setParameter(const std::string&,double);
protected:
  BaseFunction(IPrinter&,const std::string&,const std::string&);
  BaseFunction(const std::string&,const BaseFunction&);
public:
  virtual ~BaseFunction();
private:
  std::string fTitle;
protected:
  std::vector<double> fParams;
  std::vector<std::string> fParamNames;
  std::vector<std::string> fVariableNames;
  std::vector<double> fGradient;
  Annotation fAnnotation;
  std::string fCodelet;
};

class Gaussian : public BaseFunction, public Lib::Gaussian {
public: //AIDA/IFunction
  virtual double value(const std::vector<double>&) const;
  virtual int dimension() const;
  virtual bool setParameters(const std::vector<double>&);
  virtual double parameter(const std::string&) const;
  virtual bool setParameter(const std::string&,double);
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public:
  Gaussian(IPrinter&,
           const std::string&,
           const std::string&,
           double = 1,double = 0,double = 1);
  Gaussian(const std::string&,const Gaussian&);
  virtual ~Gaussian();
};


class BreitWigner : public BaseFunction, public Lib::BreitWigner {
public: //AIDA/IFunction
  virtual double value(const std::vector<double>&) const;
  virtual int dimension() const;
  virtual bool setParameters(const std::vector<double>&);
  virtual double parameter(const std::string&) const;
  virtual bool setParameter(const std::string&,double);
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public:
  BreitWigner(IPrinter&,
              const std::string&,
              const std::string&,
              double = 1,double = 0,double = 1);
  BreitWigner(const std::string&,const BreitWigner&);
  virtual ~BreitWigner();
};


class ExpHBOOK : public BaseFunction, public Lib::Exponential {
public: //AIDA/IFunction
  virtual double value(const std::vector<double>&) const;
  virtual int dimension() const;
  virtual bool setParameters(const std::vector<double>&);
  virtual double parameter(const std::string&) const;
  virtual bool setParameter(const std::string&,double);
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public:
  ExpHBOOK(IPrinter&,
              const std::string&,
              const std::string&,
              double = 0,double = 1);
  ExpHBOOK(const std::string&,const ExpHBOOK&);
  virtual ~ExpHBOOK();
};


class ExpAIDA : public BaseFunction, public Lib::Exponential {
public: //AIDA/IFunction
  virtual double value(const std::vector<double>&) const;
  virtual int dimension() const;
  virtual bool setParameters(const std::vector<double>&);
  virtual double parameter(const std::string&) const;
  virtual bool setParameter(const std::string&,double);
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public:
  ExpAIDA(IPrinter&,
               const std::string&,
               const std::string&,
               double = 1,double = 1);
  ExpAIDA(const std::string&,const ExpAIDA&);
  virtual ~ExpAIDA();
};


class Polynomial : public BaseFunction, public Lib::Polynomial {
public: //AIDA/IFunction
  virtual double value(const std::vector<double>&) const;
  virtual int dimension() const;
  virtual bool setParameters(const std::vector<double>&);
  virtual double parameter(const std::string&) const;
  virtual bool setParameter(const std::string&,double);
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public:
  Polynomial(IPrinter&,
             const std::string&,
             const std::string&,
             const std::vector<double>&);
  Polynomial(const std::string&,const Polynomial&);
  virtual ~Polynomial();
};


class ArrayFunction : public BaseFunction, public Lib::ArrayFunction {
public: //AIDA/IFunction
  virtual double value(const std::vector<double>&) const;
  virtual int dimension() const;
  virtual const std::vector<double>& parameters() const;
  virtual bool setParameters(const std::vector<double>&);
  virtual double parameter(const std::string&) const;
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public:
  ArrayFunction(IPrinter&,
                const std::string&,
                const std::string&);
  ArrayFunction(IPrinter&,
                const std::string&,
                const std::string&,
                const std::vector<double>&,
                const std::vector<double>&);
  ArrayFunction(const std::string&,const ArrayFunction&);
  virtual ~ArrayFunction();
};

class Array2DFunction : public BaseFunction, public Lib::Array2DFunction {
public: //AIDA/IFunction
  virtual double value(const std::vector<double>&) const;
  virtual int dimension() const;
  virtual const std::vector<double>& parameters() const;
  virtual bool setParameters(const std::vector<double>&);
  virtual double parameter(const std::string&) const;
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public:
  Array2DFunction(IPrinter&,
                  const std::string&,
                  const std::string&);
  Array2DFunction(IPrinter&,
                  const std::string&,
                  const std::string&,
                  const std::vector<double>&,
                  const std::vector<double>&,
                  const std::vector<double>&);
  Array2DFunction(const std::string&,const Array2DFunction&);
  virtual ~Array2DFunction();
};

class ComplexArrayFunction 
: public BaseFunction, public Lib::ComplexArrayFunction {
public: //AIDA/IFunction
  virtual double value(const std::vector<double>&) const;
  virtual int dimension() const;
  virtual const std::vector<double>& parameters() const;
  virtual bool setParameters(const std::vector<double>&);
  virtual double parameter(const std::string&) const;
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public:
  ComplexArrayFunction(IPrinter&,
                       const std::string&,
                       const std::string&);
  ComplexArrayFunction(IPrinter&,
                       const std::string&,
                       const std::string&,
                       const std::vector<double>&,
                       const std::vector< std::complex<double> >&);
  ComplexArrayFunction(IPrinter&,
                       const std::string&,
                       const std::string&,
                       const std::vector<double>&,
                       const std::vector<double>&,
                       const std::vector<double>&);
  ComplexArrayFunction(const std::string&,const ComplexArrayFunction&);
  virtual ~ComplexArrayFunction();
};

class ScriptedFunction : public BaseFunction, public Lib::ScriptedFunction {
public: //AIDA/IFunction
  virtual double value(const std::vector<double>&) const;
  virtual int dimension() const;
  virtual bool setParameters(const std::vector<double>&);
  virtual double parameter(const std::string&) const;
  virtual bool setParameter(const std::string&,double);
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public:
  ScriptedFunction(IPrinter&,
                   const std::string&,
                   const std::string&,
                   const std::string&);
  ScriptedFunction(const std::string&,const ScriptedFunction&);
  virtual ~ScriptedFunction();
};

class ScriptedFunction2D : public BaseFunction, public Lib::ScriptedFunction2D {
public: //AIDA/IFunction
  virtual double value(const std::vector<double>&) const;
  virtual int dimension() const;
  virtual bool setParameters(const std::vector<double>&);
  virtual double parameter(const std::string&) const;
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public:
  ScriptedFunction2D(IPrinter&,
                     const std::string&,
                     const std::string&,
                     const std::string&);
  ScriptedFunction2D(const std::string&,const ScriptedFunction2D&);
  virtual ~ScriptedFunction2D();
};

class CompiledFunction : public BaseFunction {
public: //AIDA/IFunction
  virtual double value(const std::vector<double>&) const;
  virtual int dimension() const;
  virtual bool setParameters(const std::vector<double>&);
  virtual double parameter(const std::string&) const;
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public:
  CompiledFunction(IPrinter&,
                   const std::string&,
                   const std::string&,
                   const std::string&,
                   Lib::IFunction&);
  CompiledFunction(const std::string&,const CompiledFunction&);
  virtual ~CompiledFunction();
private:
  Lib::IFunction& fFunction;
};

}

#endif
