#ifndef Lab_Histogram_h
#define Lab_Histogram_h

// Inheritance :
#include <Lab/Core/BaseObject.h>
#include <AIDA/IAxis.h>
#include <AIDA/IHistogram1D.h>
#include <AIDA/IHistogram2D.h>
#include <AIDA/IHistogram3D.h>
#include <AIDA/IProfile1D.h>
#include <AIDA/IProfile2D.h>

#include <HCL/Histogram.h>

#include <Lab/Core/Annotation.h>

namespace Lab {

class Axis : public virtual AIDA::IAxis {
public: //AIDA::IAxis
  virtual bool isFixedBinning() const;
  virtual double lowerEdge() const;
  virtual double upperEdge() const;
  virtual int bins() const;
  virtual double binLowerEdge(int) const;
  virtual double binUpperEdge(int) const;
  virtual double binWidth(int) const;
  virtual int coordToIndex(double) const;
public:
  Axis();
  Axis(const Axis&);
  virtual ~Axis();
  void copy(const HCL::Axis&);
private:
  HCL::Axis fAxis;
};

class BaseHistogram : public BaseObject {
public:
  virtual const HCL::Histogram& bareHistogram() const = 0;
  virtual bool isValid() const = 0;
protected:
  BaseHistogram(IPrinter&,const std::string&);
  BaseHistogram(IPrinter&,const std::string&,const BaseHistogram&);
public:
  virtual ~BaseHistogram();
  Annotation& annotation();
  void setAnnotation(const Annotation&);
protected:
  Annotation fAnnotation;
};

class Histogram1D 
:public BaseHistogram
,public virtual AIDA::IHistogram1D
{
public: //AIDA/IBaseHistogram
  virtual void* cast(const std::string&) const;
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation& annotation();
  virtual const AIDA::IAnnotation& annotation() const;
  virtual int dimension() const;
  virtual int entries() const;
  virtual bool reset();
public: //AIDA/IHistogram
  virtual int allEntries() const;
  virtual int extraEntries() const;
  virtual double equivalentBinEntries() const;
  virtual double sumBinHeights() const;
  virtual double sumAllBinHeights() const;
  virtual double sumExtraBinHeights() const;
  virtual double minBinHeight() const;
  virtual double maxBinHeight() const;
  virtual bool scale(double);
public: //AIDA/IHistogram1D
  virtual bool fill(double,double);
  virtual double binMean(int) const;
  virtual double binStdDev(int) const; //Not AIDA
  virtual int binEntries(int) const;
  virtual double binHeight(int) const;
  virtual double binError(int) const;
  virtual double mean() const;
  virtual double rms() const;
  virtual const AIDA::IAxis& axis() const;
  virtual int coordToIndex(double) const;
  virtual bool add(const AIDA::IHistogram1D&);
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public: //Lab::BaseHistogram
  virtual const HCL::Histogram& bareHistogram() const;
  virtual bool isValid() const;
public:
  Histogram1D(IPrinter&,const std::string&,HCL::Histogram&);
  Histogram1D(IPrinter&,
              const std::string&,
              const std::string&,
              int,double,double);
  Histogram1D(IPrinter&,
              const std::string&,
              const std::string&,
              const std::vector<double>&);
  Histogram1D(const std::string&,const Histogram1D&);
  virtual ~Histogram1D();
  const HCL::Histogram1D& hclHistogram() const;
private:
  HCL::Histogram1D fHistogram; 
  Axis fAxis;
};

class Histogram2D 
:public BaseHistogram
,public virtual AIDA::IHistogram2D
{
public: //AIDA/IBaseHistogram
  virtual void* cast(const std::string&) const;
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation& annotation();
  virtual const AIDA::IAnnotation& annotation() const;
  virtual int dimension() const;
  virtual int entries() const;
  virtual bool reset();
public: //AIDA/IHistogram
  virtual int allEntries() const;
  virtual int extraEntries() const;
  virtual double equivalentBinEntries() const;
  virtual double sumBinHeights() const;
  virtual double sumAllBinHeights() const;
  virtual double sumExtraBinHeights() const;
  virtual double minBinHeight() const;
  virtual double maxBinHeight() const;
  virtual bool scale(double);
public: //AIDA/IHistogram2D
  virtual bool fill(double,double,double);
  virtual double binMeanX(int,int) const;
  virtual double binMeanY(int,int) const;
  virtual double binStdDevX(int,int) const;
  virtual double binStdDevY(int,int) const;
  virtual int binEntries(int,int) const;
  virtual int binEntriesX(int) const;
  virtual int binEntriesY(int) const;
  virtual double binHeight(int,int) const;
  virtual double binHeightX(int) const;
  virtual double binHeightY(int) const;
  virtual double binError(int,int) const;
  virtual double meanX() const;
  virtual double meanY() const;
  virtual double rmsX() const;
  virtual double rmsY() const;
  virtual const AIDA::IAxis& xAxis() const;
  virtual const AIDA::IAxis& yAxis() const;
  virtual int coordToIndexX(double) const;
  virtual int coordToIndexY(double) const;
  virtual bool add(const IHistogram2D&);
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public: //Lab::BaseHistogram
  virtual const HCL::Histogram& bareHistogram() const;
  virtual bool isValid() const;
public:
  Histogram2D(IPrinter&,const std::string&,HCL::Histogram&);
  Histogram2D(IPrinter&,
              const std::string&,
              const std::string&,
              int,double,double,
              int,double,double);
  Histogram2D(IPrinter&,
              const std::string&,
              const std::string&,
              const std::vector<double>&,
              const std::vector<double>&);
  Histogram2D(const std::string&,const Histogram2D&);
  virtual ~Histogram2D();
  const HCL::Histogram2D& hclHistogram() const;
private:
  HCL::Histogram2D fHistogram; 
  Axis fAxisX;
  Axis fAxisY;
};

class Histogram3D 
:public BaseHistogram
,public virtual AIDA::IHistogram3D
{
public: //AIDA/IBaseHistogram
  virtual void* cast(const std::string&) const;
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation& annotation();
  virtual const AIDA::IAnnotation& annotation() const;
  virtual int dimension() const;
  virtual int entries() const;
  virtual bool reset();
public: //AIDA/IHistogram
  virtual int allEntries() const;
  virtual int extraEntries() const;
  virtual double equivalentBinEntries() const;
  virtual double sumBinHeights() const;
  virtual double sumAllBinHeights() const;
  virtual double sumExtraBinHeights() const;
  virtual double minBinHeight() const;
  virtual double maxBinHeight() const;
  virtual bool scale(double);
public: //AIDA/IHistogram3D
  virtual bool fill(double,double,double,double);
  virtual double binMeanX(int,int,int) const;
  virtual double binMeanY(int,int,int) const;
  virtual double binMeanZ(int,int,int) const;
  virtual double binStdDevX(int,int,int) const;
  virtual double binStdDevY(int,int,int) const;
  virtual double binStdDevZ(int,int,int) const;
  virtual int binEntries(int,int,int) const;
  virtual int binEntriesX(int) const;
  virtual int binEntriesY(int) const;
  virtual int binEntriesZ(int) const;
  virtual double binHeight(int,int,int) const;
  virtual double binHeightX(int) const;
  virtual double binHeightY(int) const;
  virtual double binHeightZ(int) const;
  virtual double binError(int,int,int) const;
  virtual double meanX() const;
  virtual double meanY() const;
  virtual double meanZ() const;
  virtual double rmsX() const;
  virtual double rmsY() const;
  virtual double rmsZ() const;
  virtual const AIDA::IAxis& xAxis() const;
  virtual const AIDA::IAxis& yAxis() const;
  virtual const AIDA::IAxis& zAxis() const;
  virtual int coordToIndexX(double) const;
  virtual int coordToIndexY(double) const;
  virtual int coordToIndexZ(double) const;
  virtual bool add(const IHistogram3D&);
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public: //Lab::BaseHistogram
  virtual const HCL::Histogram& bareHistogram() const;
  virtual bool isValid() const;
public:
  Histogram3D(IPrinter&,const std::string&,HCL::Histogram&);
  Histogram3D(IPrinter&,
              const std::string&,
              const std::string&,
              int,double,double,
              int,double,double,
              int,double,double);
  Histogram3D(IPrinter&,
              const std::string&,
              const std::string&,
              const std::vector<double>&,
              const std::vector<double>&,
              const std::vector<double>&);
  Histogram3D(const std::string&,const Histogram3D&);
  virtual ~Histogram3D();
  const HCL::Histogram3D& hclHistogram() const;
private:
  HCL::Histogram3D fHistogram; 
  Axis fAxisX;
  Axis fAxisY;
  Axis fAxisZ;
};

class Profile1D 
:public BaseHistogram
,public virtual AIDA::IProfile1D
{
public: //AIDA/IBaseHistogram
  virtual void* cast(const std::string&) const;
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation& annotation();
  virtual const AIDA::IAnnotation& annotation() const;
  virtual int dimension() const;
  virtual int entries() const;
  virtual bool reset();
public: //AIDA/IHistogram
  virtual int allEntries() const;
  virtual int extraEntries() const;
  virtual double equivalentBinEntries() const;
  virtual double sumBinHeights() const;
  virtual double sumAllBinHeights() const;
  virtual double sumExtraBinHeights() const;
  virtual double minBinHeight() const;
  virtual double maxBinHeight() const;
  virtual bool scale(double);
public: //AIDA/IProfile1D
  virtual bool fill(double,double,double);
  virtual double binMean(int) const;
  virtual double binStdDev(int) const; //Not AIDA
  virtual int binEntries(int) const;
  virtual double binHeight(int) const;
  virtual double binError(int) const;
  virtual double binRms(int) const;
  virtual double mean() const;
  virtual double rms() const;
  virtual const AIDA::IAxis& axis() const;
  virtual int coordToIndex(double) const;
  virtual bool add(const AIDA::IProfile1D&);
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public: //Lab::BaseHistogram
  virtual const HCL::Histogram& bareHistogram() const;
  virtual bool isValid() const;
public:
  Profile1D(IPrinter&,const std::string&,HCL::Histogram&);
  Profile1D(IPrinter&,
            const std::string&,
            const std::string&,
            int,double,double);
  Profile1D(IPrinter&,
            const std::string&,
            const std::string&,
            int,double,double,
            double,double);
  Profile1D(IPrinter&,
            const std::string&,
            const std::string&,
            const std::vector<double>&);
  Profile1D(IPrinter&,
            const std::string&,
            const std::string&,
            const std::vector<double>&,
            double,double);
  Profile1D(const std::string&,const Profile1D&);
  virtual ~Profile1D();
private:
  HCL::Profile1D fHistogram; 
  Axis fAxis;
};

class Profile2D 
:public BaseHistogram
,public virtual AIDA::IProfile2D
{
public: //AIDA/IBaseHistogram
  virtual void* cast(const std::string&) const;
  virtual std::string title() const;
  virtual bool setTitle(const std::string&);
  virtual AIDA::IAnnotation& annotation();
  virtual const AIDA::IAnnotation& annotation() const;
  virtual int dimension() const;
  virtual int entries() const;
  virtual bool reset();
public: //AIDA/IHistogram
  virtual int allEntries() const;
  virtual int extraEntries() const;
  virtual double equivalentBinEntries() const;
  virtual double sumBinHeights() const;
  virtual double sumAllBinHeights() const;
  virtual double sumExtraBinHeights() const;
  virtual double minBinHeight() const;
  virtual double maxBinHeight() const;
  virtual bool scale(double);
public: //AIDA/IProfile2D
  virtual bool fill(double,double,double,double);
  virtual double binMeanX(int,int) const;
  virtual double binMeanY(int,int) const;
  virtual double binStdDevX(int,int) const; //Not AIDA
  virtual double binStdDevY(int,int) const; //Not AIDA
  virtual int binEntries(int,int) const;
  virtual int binEntriesX(int) const;
  virtual int binEntriesY(int) const;
  virtual double binRms(int,int) const;
  virtual double binHeight(int,int) const;
  virtual double binHeightX(int) const;
  virtual double binHeightY(int) const;
  virtual double binError(int,int) const;
  virtual double meanX() const;
  virtual double meanY() const;
  virtual double rmsX() const;
  virtual double rmsY() const;
  virtual const AIDA::IAxis& xAxis() const;
  virtual const AIDA::IAxis& yAxis() const;
  virtual int coordToIndexX(double) const;
  virtual int coordToIndexY(double) const;
  virtual bool add(const IProfile2D&);
public: //Lab::BaseObject
  virtual BaseObject* copy(const std::string&) const;
public: //Lab::BaseHistogram
  virtual const HCL::Histogram& bareHistogram() const;
  virtual bool isValid() const;
public:
  Profile2D(IPrinter&,const std::string&,HCL::Histogram&);
  Profile2D(IPrinter&,
              const std::string&,
              const std::string&,
              int,double,double,
              int,double,double);
  Profile2D(IPrinter&,
              const std::string&,
              const std::string&,
              int,double,double,
              int,double,double,
              double,double);
  Profile2D(IPrinter&,
              const std::string&,
              const std::string&,
              const std::vector<double>&,
              const std::vector<double>&);
  Profile2D(IPrinter&,
              const std::string&,
              const std::string&,
              const std::vector<double>&,
              const std::vector<double>&,
              double,double);
  Profile2D(const std::string&,const Profile2D&);
  virtual ~Profile2D();
private:
  HCL::Profile2D fHistogram; 
  Axis fAxisX;
  Axis fAxisY;
};

}

#endif
