#ifndef Lab_MemoryTree_h
#define Lab_MemoryTree_h

// Inheritance :
#include <Lab/Core/BaseTree.h>

#include <Lab/Core/Folder.h>

namespace Lab {

// ITree should come first due to Lab::castTree function.

class MemoryTree : public BaseTree {
public: //IManager
  virtual void* cast(const std::string&) const;
public: //AIDA/ITree
  virtual AIDA::IManagedObject* find(const std::string&);
  virtual bool cd(const std::string&);
  virtual bool mkdir(const std::string&);
  virtual bool mkdirs ( const std::string&);
  virtual bool commit();
  virtual std::string findPath(const AIDA::IManagedObject&) const;
  virtual std::string storeName() const;
  virtual bool rm(const std::string&);
  virtual std::string pwd() const;
  virtual bool ls(const std::string & = ".", bool = false,std::ostream& = std::cout) const;
  virtual bool rmdir ( const std::string&);
  virtual bool close();
  virtual std::vector<std::string> listObjectNames(const std::string& = ".", bool = false) const;
  virtual std::vector<std::string> listObjectTypes(const std::string& = ".", bool = false) const;
  virtual void setOverwrite(bool = true);
  virtual bool mv(const std::string&,const std::string&);
  virtual bool cp(const std::string&,const std::string&,bool = false );
  virtual bool symlink(const std::string&,const std::string&);
  virtual bool mount(const std::string&,AIDA::ITree&,const std::string&);
  virtual bool unmount(const std::string&);
  virtual AIDA::ITree* findTree(const std::string&);
public: //Lab::BaseTree.
  virtual bool addObject(BaseObject*,const std::string&);
  virtual bool isObjectValid(const AIDA::IManagedObject*) const;
  virtual std::string hierarchy() const; 
public: 
  virtual IFolder* currentFolder() const;
  virtual void setCurrentFolder(IFolder*);
  virtual IFolder& rootFolder();
  virtual std::vector<AIDA::IManagedObject*> allObjects() const;
  virtual bool isADirectory(const std::string&);
public:
  MemoryTree(ISession&,const std::string&);
  virtual ~MemoryTree();
protected:
  IFolder* findFolder(const std::string&,bool = false) const;
  IFolder* makeFolders(const std::string&); 
private:
  Folder fRoot;
  IFolder* fCurrentFolder;
};

}

#endif
