#ifndef Lab_IFolder_h
#define Lab_IFolder_h

#include <string>

class IIterator;
class IFolder;
namespace AIDA {
 class IManagedObject;
 class ITree;
}

namespace Lab {
 class BaseTree;
}

class IFolderIterator {
public:
  virtual ~IFolderIterator() {}
  virtual IFolder* folder() = 0;
  virtual void next() = 0;
};

class IFolderVisitor {
public:
  virtual ~IFolderVisitor() {}
  /*
   * @return False stop the traversal.
   */
  virtual bool visit(const IFolder&,const AIDA::IManagedObject&) = 0;
  virtual bool beginVisit(const IFolder&,const IFolder&) = 0;
  virtual bool endVisit(const IFolder&,const IFolder&) = 0;
  virtual bool isRecursive() const = 0;
};

class IFolder {
public:
  virtual ~IFolder() {}
  virtual void* cast(const std::string&) const = 0;
public:
  virtual IFolder* parent() const = 0;
  virtual const std::string& name() const = 0;
  virtual void setName(const std::string&) = 0;
  virtual void add(AIDA::IManagedObject*) = 0;
  virtual void remove(AIDA::IManagedObject*) = 0;
  virtual void add(IFolder*) = 0;
  virtual void remove(IFolder*) = 0;
  virtual IIterator* objects() const = 0;
  virtual IFolderIterator* folders() const = 0;
  virtual AIDA::IManagedObject* findInTree(const std::string&,IFolder*&) const = 0;
  virtual AIDA::IManagedObject* findObject(const std::string&) const = 0;
  virtual bool isValid(const AIDA::IManagedObject*) const = 0;
  virtual void clear() = 0;
  virtual IFolder* mkdir(const std::string&) = 0;
  virtual IFolder* findFolder(const std::string&) const = 0;
  virtual void ls() const = 0;
  virtual std::string path() const = 0;
  virtual bool visit(IFolderVisitor&) const = 0;
  virtual void setParent(IFolder*) = 0;
  virtual Lab::BaseTree& tree() const = 0; //FIXME
  // Link methods :
  virtual bool link(IFolder*,const std::string&) = 0;
  virtual bool unlink(IFolder*) = 0;
  virtual IFolder* linkingFolder() const = 0;
  virtual void setLinkingFolder(IFolder*) = 0;
  virtual std::string linkingPath() const = 0;
};

#endif
