#ifndef Lab_IVector_h
#define Lab_IVector_h

#include <string>
#include <vector>

namespace AIDA {
  class IAnnotation;
}

namespace Lab {

class IVector {

public: 
    typedef std::vector< std::pair<int,int> > Cut;


    /// Destructor.
    virtual ~IVector() { /* nop */; }

    /**
     * Get the IAnnotation of the IVector.
     * @return The IAnnotation.
     *
     */
    virtual AIDA::IAnnotation & annotation() = 0;

    virtual const AIDA::IAnnotation & annotation() const = 0;

    /**
     * Get the title of the IVector.
     * @return The title.
     *
     */
    virtual std::string title() const = 0;

    /**
     * Set the title of the IVector.
     * @param title The new title.
     * @return false If the title cannot be set.
     *
     */
    virtual bool setTitle(const std::string & title) = 0;

    /**
     * Get the Vector dimension.
     * @return The Vector dimension.
     *
     */ 
    virtual int dimension() const = 0;
    virtual int size() const = 0;

    virtual const std::vector<int>& orders() const = 0;

    virtual bool setValue(const std::vector<int>&,double) = 0;
    virtual bool value(const std::vector<int>&,double&) const = 0;

    virtual bool fill(const std::vector<double>&,Cut* = 0) = 0;

    virtual bool copy(const IVector& aFrom) = 0;
    virtual bool add(const IVector& aVector,Cut* = 0) = 0;
    virtual bool subtract(const IVector& aVector) = 0;
    virtual bool multiply(const IVector& aVector) = 0;
    virtual bool divide(const IVector& aVector) = 0;
    virtual bool add(double,Cut* = 0) = 0;
    virtual bool multiply(double) = 0;
    virtual bool invert() = 0;
    virtual bool indices(int,std::vector<int>&) const = 0;
    virtual const std::vector<double>& vector() const = 0;
    virtual std::vector<double>& vector() = 0;
    virtual bool accept(int,const Cut&) const = 0;

    /**
     *  See IManagedObject for a description.
     * @param className The name of the class to cast on.
     * @return The right pointer. Return 0 if failure.
     */ 
    virtual void * cast(const std::string & className) const = 0;

};

}

#endif
