#ifndef Lab_Plotter_h
#define Lab_Plotter_h

// Inheritance :
#include <AIDA/IPlotterRegion.h>
#include <AIDA/IPlotter.h>

#include <AIDA/IPlotterStyle.h>
#include <Lab/Inventor/Styles.h>
#include <list>

class IPrinter;
namespace Lib {
  class IRelationManager;
}

class IUI;
class SoPage;
class SoRegion;

namespace Lab {

class PlotterRegion 
:public virtual AIDA::IPlotterRegion
{
public: //AIDA/IPlotter
  virtual bool plot(const AIDA::IBaseHistogram&,const std::string& = "");
  virtual bool plot(const AIDA::IBaseHistogram&,
                    const AIDA::IPlotterStyle&,const std::string& = "");
  virtual bool plot(const AIDA::IFunction&,const std::string& = "");
  virtual bool plot(const AIDA::IFunction&,
                    const AIDA::IPlotterStyle&,const std::string& = "");
  virtual bool plot(const AIDA::IDataPointSet&,const std::string& = "");
  virtual bool plot(const AIDA::IDataPointSet&,
                    const AIDA::IPlotterStyle&,const std::string& = "");
  virtual bool remove(const AIDA::IBaseHistogram&);
  virtual bool remove(const AIDA::IFunction&);
  virtual bool remove(const AIDA::IDataPointSet&);
  virtual void clear();
  virtual bool setParameter(const std::string&,const std::string&);
  virtual std::string parameterValue(const std::string&) const;
  virtual std::vector<std::string> availableParameterOptions(const std::string&) const;  
  virtual std::vector<std::string> availableParameters() const;

  virtual AIDA::IPlotterStyle & style();
  virtual bool setStyle(const AIDA::IPlotterStyle&);
  virtual bool applyStyle(const AIDA::IPlotterStyle&);
  virtual void setTitle(const std::string&);
  virtual bool setXLimits(double,double);
  virtual bool setYLimits(double,double);
  virtual bool setZLimits(double,double);
  virtual AIDA::IPlotterLayout& layout();
  virtual bool setLayout(const AIDA::IPlotterLayout&);
  virtual AIDA::IInfo& info();
public:
  PlotterRegion(IPrinter&,Lib::IRelationManager&,int,SoRegion*,IUI&);
  PlotterRegion(const PlotterRegion&);
  PlotterRegion& operator=(const PlotterRegion&);
  virtual ~PlotterRegion();
  SoRegion* soRegion() const;
private:
  IPrinter& fPrinter;
  Lib::IRelationManager& fRelationManager;
  int fVerboseLevel;
  SoRegion* fSoRegion;
  IUI& fUI;
  RegionSetParameter fSetParameter;
  PlotterStyle fPlotterStyle;
  Info fInfo;
  PlotterLayout fLayout;
  bool fRetain;
};

class Plotter : public virtual AIDA::IPlotter {
public: //AIDA/IPlotter
public:
  virtual AIDA::IPlotterRegion* createRegion(double,double,double,double);
  virtual bool createRegions(int,int,int);
  virtual AIDA::IPlotterRegion& currentRegion() const;
  virtual int currentRegionNumber() const;
  virtual int numberOfRegions() const;
  virtual bool setCurrentRegionNumber(int);
  virtual AIDA::IPlotterRegion& next();
  virtual AIDA::IPlotterRegion* region(int) const;
  virtual void destroyRegions();
  virtual void clearRegions();
  virtual bool setParameter(const std::string&,const std::string&);
  virtual std::string parameterValue(const std::string&) const;
  virtual std::vector<std::string> availableParameterOptions(const std::string&) const;  
  virtual std::vector<std::string> availableParameters() const;
  virtual bool show();
  virtual bool hide();
  virtual bool refresh();
  virtual bool interact();
  virtual bool writeToFile(const std::string&,const std::string&);
  virtual void setTitle(const std::string&);
  virtual AIDA::ITitleStyle& titleStyle();
  virtual void setTitleStyle(const AIDA::ITitleStyle&);
public:
  Plotter(IPrinter&,IUI&,int,SoPage&,const std::string&);
  virtual ~Plotter();
  bool isValid() const;
private:
  AIDA::IPlotterRegion* plotterRegion(SoRegion*,IUI&) const;
  SoRegion* currentSoRegion() const;
private:
  IPrinter& fPrinter;
  IUI& fUI;
  int fVerboseLevel;
  SoPage& fSoPage;
  std::string fName;
  std::list<PlotterRegion> fRegions;
  PageSetParameter fSetParameter;
  TitleStyle fTitleStyle;
  Lib::IRelationManager* fRelationManager;
};

}

#endif
