#ifndef Lab_Styles_h
#define Lab_Styles_h

#include <AIDA/IBaseStyle.h>
#include <AIDA/IBrushStyle.h>
#include <AIDA/IFillStyle.h>
#include <AIDA/ILineStyle.h>
#include <AIDA/IMarkerStyle.h>
#include <AIDA/ITextStyle.h>
#include <AIDA/ITitleStyle.h>
#include <AIDA/IInfoStyle.h>
#include <AIDA/IDataStyle.h>
#include <AIDA/IAxisStyle.h>
#include <AIDA/IPlotterStyle.h>
#include <AIDA/IPlotterLayout.h>
#include <AIDA/IInfo.h>

class SoPage;
class SoRegion;

namespace Lab {

class ISetParameter {
public:
  virtual ~ISetParameter() {}
  virtual bool setParameter(const std::string&,const std::string&) = 0;
  virtual bool parameterValue(const std::string&,std::string&) = 0;
};

class RegionSetParameter : public virtual ISetParameter {
public: //ISetParameter
  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool parameterValue(const std::string&,std::string&);
public:
  RegionSetParameter(SoRegion*);
  RegionSetParameter(const RegionSetParameter&);
  RegionSetParameter& operator=(const RegionSetParameter&);
  virtual ~RegionSetParameter();
private:
  SoRegion* fSoRegion;
};

class PageSetParameter : public virtual ISetParameter {
public: //ISetParameter
  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool parameterValue(const std::string&,std::string&);
public:
  PageSetParameter(SoPage*);
  PageSetParameter(const PageSetParameter&);
  PageSetParameter& operator=(const PageSetParameter&);
  virtual ~PageSetParameter();
private:
  SoPage* fSoPage;
};

// Basic styles (not over other styles) :

class FillStyle : public virtual AIDA::IFillStyle {
public: //AIDA::IBaseStyle
  virtual void reset();
  virtual bool setParameter(const std::string & paramName, const std::string & options = "");
  virtual std::string parameterValue(const std::string & parameter) const;
  virtual std::vector<std::string>  availableParameters() const;
  virtual std::vector<std::string>  availableParameterOptions(const std::string & paramName) const;
public: //AIDA::IBrushStyle
  virtual std::vector<std::string>  availableColors() const;
  virtual std::string color() const;
  virtual double opacity() const;
  virtual bool setColor(const std::string & newColor);
  virtual bool setOpacity(double newOpacity);
public: //AIDA::IFillStyle
  virtual std::vector<std::string>  availablePatterns() const;
  virtual std::string pattern() const;
  virtual bool setPattern(const std::string & pattern);
public:
  FillStyle(ISetParameter&,const std::string&);
  virtual ~FillStyle();
private:
  ISetParameter& fSetParameter;
  std::string fField;
};

class LineStyle : public virtual AIDA::ILineStyle {
public: //AIDA::IBaseStyle
  virtual void reset();
  virtual bool setParameter(const std::string & paramName, const std::string & options = "");
  virtual std::string parameterValue(const std::string & parameter) const;
  virtual std::vector<std::string>  availableParameters() const;
  virtual std::vector<std::string>  availableParameterOptions(const std::string & paramName) const;
public: //AIDA::IBrushStyle
  virtual std::vector<std::string>  availableColors() const;
  virtual std::string color() const;
  virtual double opacity() const;
  virtual bool setColor(const std::string & newColor);
  virtual bool setOpacity(double newOpacity);
public: //AIDA::ILineStyle
  virtual std::vector<std::string>  availableLineTypes() const;
  virtual std::string lineType() const;
  virtual int thickness() const;
  virtual bool setLineType(const std::string & newLineType);
  virtual bool setThickness(int newThickness);
public:
  LineStyle(ISetParameter&,const std::string&);
  virtual ~LineStyle();
private:
  ISetParameter& fSetParameter;
  std::string fField;
};

class MarkerStyle : public virtual AIDA::IMarkerStyle {
public: //AIDA::IBaseStyle
  virtual void reset();
  virtual bool setParameter(const std::string & paramName, const std::string & options = "");
  virtual std::string parameterValue(const std::string & parameter) const;
  virtual std::vector<std::string>  availableParameters() const;
  virtual std::vector<std::string>  availableParameterOptions(const std::string & paramName) const;
public: //AIDA::IBrushStyle
  virtual std::vector<std::string>  availableColors() const;
  virtual std::string color() const;
  virtual double opacity() const;
  virtual bool setColor(const std::string & newColor);
  virtual bool setOpacity(double newOpacity);
public: //AIDA::IMarkerStyle
  virtual std::vector<std::string>  availableShapes() const;
  virtual std::string shape() const;
  virtual bool setShape(const std::string & shape);
public:
  MarkerStyle(ISetParameter&,const std::string&);
  virtual ~MarkerStyle();
private:
  ISetParameter& fSetParameter;
  std::string fField;
};

class TextStyle : public virtual AIDA::ITextStyle {
public: //AIDA::IBaseStyle
  virtual void reset();
  virtual bool setParameter(const std::string & paramName, const std::string & options = "");
  virtual std::string parameterValue(const std::string & parameter) const;
  virtual std::vector<std::string>  availableParameters() const;
  virtual std::vector<std::string>  availableParameterOptions(const std::string & paramName) const;
public: //AIDA::IBrushStyle
  virtual std::vector<std::string>  availableColors() const;
  virtual std::string color() const;
  virtual double opacity() const;
  virtual bool setColor(const std::string & newColor);
  virtual bool setOpacity(double newOpacity);
public: //AIDA::ITextStyle
  virtual std::vector<std::string>  availableFonts() const;
  virtual double fontSize() const;
  virtual bool setFontSize(double size);
  virtual std::string font() const;
  virtual bool setFont(const std::string & font);
  virtual bool isBold() const;
  virtual bool isItalic() const;
  virtual bool isUnderlined() const;
  virtual bool setBold(bool bold = true);
  virtual bool setItalic(bool italic = true);
  virtual bool setUnderlined(bool underlined = true);
public:
  TextStyle(ISetParameter&,const std::string&);
  virtual ~TextStyle();
private:
  ISetParameter& fSetParameter;
  std::string fField;
};

// Plotter scene part styles (over basic styles) :

class DataStyle : public virtual AIDA::IDataStyle {
public: //AIDA::IBaseStyle
  virtual void reset();
  virtual bool setParameter(const std::string & paramName, const std::string & options = "");
  virtual std::string parameterValue(const std::string & parameter) const;
  virtual std::vector<std::string>  availableParameters() const;
  virtual std::vector<std::string>  availableParameterOptions(const std::string & paramName) const;
public: //AIDA::IDataStyle
  virtual AIDA::ILineStyle & lineStyle();
  virtual AIDA::IMarkerStyle & markerStyle();
  virtual AIDA::IFillStyle & fillStyle();
  virtual bool setLineStyle(const AIDA::ILineStyle & lineStyle);
  virtual bool setMarkerStyle(const AIDA::IMarkerStyle & markerStyle);
  virtual bool setFillStyle(const AIDA::IFillStyle & fillStyle);
public:
  DataStyle(ISetParameter&,const std::string&);
  virtual ~DataStyle();
private:
  ISetParameter& fSetParameter;
  std::string fField;
  LineStyle fLineStyle;
  MarkerStyle fMarkerStyle;
  FillStyle fFillStyle;
};

class InfoStyle : public virtual AIDA::IInfoStyle {
public: //AIDA::IBaseStyle
  virtual void reset();
  virtual bool setParameter(const std::string & paramName, const std::string & options = "");
  virtual std::string parameterValue(const std::string & parameter) const;
  virtual std::vector<std::string>  availableParameters() const;
  virtual std::vector<std::string>  availableParameterOptions(const std::string & paramName) const;
public: //AIDA::IInfoStyle
  virtual AIDA::ITextStyle & textStyle();
  virtual bool setTextStyle(const AIDA::ITextStyle & textStyle);
public:
  InfoStyle(ISetParameter&,const std::string&);
  virtual ~InfoStyle();
private:
  ISetParameter& fSetParameter;
  std::string fField;
  TextStyle fTextStyle;
};

class AxisStyle : public virtual AIDA::IAxisStyle {
public: //AIDA::IBaseStyle
  virtual void reset();
  virtual bool setParameter(const std::string & paramName, const std::string & options = "");
  virtual std::string parameterValue(const std::string & parameter) const;
  virtual std::vector<std::string>  availableParameters() const;
  virtual std::vector<std::string>  availableParameterOptions(const std::string & paramName) const;
public: //AIDA::IAxisStyle
  virtual AIDA::ILineStyle& lineStyle();
  virtual AIDA::ITextStyle& tickLabelStyle();
  virtual AIDA::ITextStyle& labelStyle();
  virtual bool setlineStyle(const AIDA::ILineStyle& lineStyle);
  virtual bool setTickLabelStyle(const AIDA::ITextStyle& tickLabelStyle);
  virtual bool setLabelStyle(const AIDA::ITextStyle& labelStyle);
  virtual void setLabel(const std::string& label);
public: 
  AxisStyle(ISetParameter&,const std::string&);
  virtual ~AxisStyle();
private:
  ISetParameter& fSetParameter;
  std::string fField;
  LineStyle fLineStyle;
  LineStyle fTicksStyle;
  TextStyle fLabelsStyle;
  TextStyle fTitleStyle;
};

class TitleStyle : public virtual AIDA::ITitleStyle {
public: //AIDA::IBaseStyle
  virtual void reset();
  virtual bool setParameter(const std::string & paramName, const std::string & options = "");
  virtual std::string parameterValue(const std::string & parameter) const;
  virtual std::vector<std::string>  availableParameters() const;
  virtual std::vector<std::string>  availableParameterOptions(const std::string & paramName) const;
public: //AIDA::ITitleStyle
  virtual AIDA::ITextStyle& textStyle();
  virtual bool setTextStyle(const AIDA::ITextStyle& textStyle);
public:
  TitleStyle(ISetParameter&,const std::string&);
  virtual ~TitleStyle();
private:
  ISetParameter& fSetParameter;
  std::string fField;
  TextStyle fTextStyle;
};

// Global plotter style :

class PlotterStyle : public virtual AIDA::IPlotterStyle {
public: //AIDA::IBaseStyle
  virtual void reset();
  virtual bool setParameter(const std::string & paramName, const std::string & options = "");
  virtual std::string parameterValue(const std::string & parameter) const;
  virtual std::vector<std::string>  availableParameters() const;
  virtual std::vector<std::string>  availableParameterOptions(const std::string & paramName) const;
public: //AIDA::IPlotterStyle
  virtual AIDA::IDataStyle& dataStyle();
  virtual AIDA::IAxisStyle& xAxisStyle();
  virtual AIDA::IAxisStyle& yAxisStyle();
  virtual AIDA::IAxisStyle& zAxisStyle();
  virtual AIDA::ITitleStyle& titleStyle();
  virtual AIDA::IInfoStyle& infoStyle();
  virtual bool setDataStyle(const AIDA::IDataStyle& dataStyle);
  virtual bool setAxisStyleX(const AIDA::IAxisStyle& xAxisStyle);
  virtual bool setAxisStyleY(const AIDA::IAxisStyle& yAxisStyle);
  virtual bool setAxisStyleZ(const AIDA::IAxisStyle& zAxisStyle);
  virtual bool setTitleStyle(const AIDA::ITitleStyle& textStyle);
  virtual bool setInfoStyle(const AIDA::IInfoStyle& infoStyle);
public:
  PlotterStyle(ISetParameter&,const std::string&);
  virtual ~PlotterStyle();
private:
  ISetParameter& fSetParameter;
  std::string fField;
  DataStyle fDataStyle;
  AxisStyle f_xAxisStyle;
  AxisStyle f_yAxisStyle;
  AxisStyle f_zAxisStyle;
  TitleStyle fTitleStyle;
  InfoStyle fInfoStyle;
};

class PlotterLayout : public virtual AIDA::IPlotterLayout {
public: 
  virtual void reset();
  virtual bool setParameter(const std::string & paramName, double paramValue);
  virtual double parameterValue(const std::string & paramName);
  virtual std::vector<std::string>  availableParameters() const;
public:
  PlotterLayout(ISetParameter&,const std::string&);
  virtual ~PlotterLayout();
private:
  ISetParameter& fSetParameter;
  std::string fField;
};

class Info : public virtual AIDA::IInfo {
public: 
  virtual void clear();
  virtual void addText(const std::string & text);
  virtual void addLegend(const AIDA::IMarkerStyle & style, const std::string & description);
  virtual void addLegend(const AIDA::ILineStyle & style, const std::string & description);
  virtual void addLegend(const AIDA::IFillStyle & style, const std::string & description);
public:
  virtual ~Info();
};

}

#endif
