#ifndef Lab_XML_Tree_h
#define Lab_XML_Tree_h

#include <Lab/Core/MemoryTree.h>

namespace Lib {
  class Out;
  class ItemML;
}

namespace HCL {
  class Histogram;
}

namespace AIDA {
  class ITuple;
  class IHistogram1D;
  class IHistogram2D;
  class IHistogram3D;
  class IProfile1D;
  class IProfile2D;
  class IAxis;
  class ICloud1D;
  class ICloud2D;
  class ICloud3D;
  class IDataPointSet;
  class IFunction;
}

namespace Lab {

class FunctionBooking {
public:
  std::string fArgument;
  std::vector< std::pair<double,double> > fRanges;
  std::vector< std::pair<std::string,double> > fParameters;
};

class BaseHistogram;
class Cloud1D;
class Cloud2D;
class Cloud3D;

class XML_Manager;

class XML_Tree : public MemoryTree {
public: //IManager
  virtual void* cast(const std::string&) const;
public: //AIDA/ITree
  virtual bool commit();
  virtual std::string storeName() const;
public:
  XML_Tree(ISession&,const std::string&,bool,bool,const std::string&,
           XML_Manager&,int);
  virtual ~XML_Tree();
  bool isValid() const;
public:
  void writeHistogram(Lib::Out&,
                      const std::string&,const std::string&,
                      const AIDA::IProfile1D&);
  void writeHistogram(Lib::Out&,
                      const std::string&,const std::string&,
                      const AIDA::IProfile2D&);
  void writeTuple(Lib::Out&,
                  const std::string&,const std::string&,
                  AIDA::ITuple&);
  void writeCloud(Lib::Out&,
                  const std::string&,const std::string&,
                  AIDA::ICloud1D&);
  void writeCloud(Lib::Out&,
                  const std::string&,const std::string&,
                  AIDA::ICloud2D&);
  void writeCloud(Lib::Out&,
                  const std::string&,const std::string&,
                  AIDA::ICloud3D&);
  void writeDataPointSet(Lib::Out&,
                         const std::string&,const std::string&,
                         AIDA::IDataPointSet&);
  void writeFunction(Lib::Out&,
                     const std::string&,const std::string&,
                     AIDA::IFunction&);
private:
  std::string buildName(Lib::ItemML&);
  bool createObject(Lib::ItemML&);

  bool readHistogram(Lib::ItemML&,int,bool,bool,BaseHistogram*&);
  bool readHistogramAxis(Lib::ItemML&,int,int&,int&,double&,double&,std::vector<double>&,bool&);
  bool readHistogramData(Lib::ItemML&,HCL::Histogram&);
  bool readTuple(Lib::ItemML&);
  bool readTupleColumns(Lib::ItemML&,bool&,std::string&);
  bool readTupleRows(Lib::ItemML&,AIDA::ITuple&,bool&);

  bool readCloud(Lib::ItemML&,int);
  bool readCloudData(Lib::ItemML&,Cloud1D&);
  bool readCloudData(Lib::ItemML&,Cloud2D&);
  bool readCloudData(Lib::ItemML&,Cloud3D&);

  bool readDataPointSet(Lib::ItemML&);
  bool readDataPointSetData(Lib::ItemML&,AIDA::IDataPointSet&);

  bool readFunction(Lib::ItemML&);
  bool readFunctionBooking(Lib::ItemML&,FunctionBooking&);

  bool axisIndex(int,const std::string&,int&);
  bool codeletToScript(const std::string&,std::string&);
  void writeTupleRows(Lib::Out&,AIDA::ITuple&,const std::string&);
private:
  std::string fStoreName;
  int fNameCounter;
  bool fReadOnly;
  bool fCreateNew;
  bool fCompress;
  bool fIsValid;
};

}

#endif
