
source $env(ONXROOT)/scripts/Tcl/init.tcl

loadLibrary [dllName LabTclSWIG] Lab_SWIG

#///////////////////////////////////////////////////////
#//// Tree /////////////////////////////////////////////
#///////////////////////////////////////////////////////
delete aida
IAnalysisFactory aida -this [IManager_to_IAnalysisFactory [session findManager AnalysisFactory ]]

ITree memoryTree -this [IManager_to_ITree [session findManager MemoryTree ]]

proc RioTree { aHandle aFile aMode } {
 ITreeFactory tf -this [aida createTreeFactory]
 if { $aMode == "READ" } { 
   ITree $aHandle -this [tf create $aFile ROOT 1 0]
 } else {
   if { $aMode == "UPDATE" } {
     ITree $aHandle -this [tf create $aFile ROOT 0 0]
   } else { 
     ITree $aHandle -this [tf create $aFile ROOT 1 1]
   }
 }
 del_ITreeFactory [& tf]
 rename tf ""
}
proc AsciiTree { aHandle aFile } {
 ITree $aHandle -this [aida createAscii $aHandle $aFile]
}
proc DestroyTree { aHandle } {
 session destroyManager $aHandle
}

#///////////////////////////////////////////////////////
#//// Histogram ////////////////////////////////////////
#///////////////////////////////////////////////////////

proc H1D { aHandle aLabel aXnumber aXmin aXmax {aTree memoryTree} } {
 IHistogramFactory hf -this [aida createHistogramFactory [& $aTree]]
 IHistogram1D $aHandle -this [hf createHistogram1D $aHandle $aLabel $aXnumber $aXmin $aXmax]
 del_IHistogramFactory [& hf]
 rename hf ""   
}
proc H2D { aHandle aLabel aXnumber aXmin aXmax aYnumber aYmin aYmax {aTree memoryTree} } {
 IHistogramFactory hf -this [aida createHistogramFactory [& $aTree]]
 IHistogram2D $aHandle -this [hf createHistogram2D $aHandle $aLabel $aXnumber $aXmin $aXmax $aYnumber $aYmin $aYmax]
 del_IHistogramFactory [& hf]
 rename hf ""   
}
proc H1D_get { aHandle aTree aSID } {
 IHistogram1D $aHandle -this [ IManagedObject_to_IHistogram1D [ $aTree find $aSID ] ]
}
proc H2D_get { aHandle aTree aSID } {
 IHistogram2D $aHandle -this [ IManagedObject_to_IHistogram2D [ $aTree find $aSID ] ]
}

# To get a handle over an already booked histo.
# The handle has the same name that the name of the object.
proc HHook { aName { aTree memoryTree} } {
 IManagedObject object -this [ $aTree find $aName]
 IHistogram $aName -this [ IManagedObject_to_IHistogram [& object] ]
 delete object
}
proc H1DHook { aName { aTree memoryTree} } {
 IManagedObject object -this [ $aTree find $aName]
 IHistogram1D $aName -this [ IManagedObject_to_IHistogram1D [& object] ]
 delete object
}
proc H2DHook { aName { aTree memoryTree} } {
 IManagedObject object -this [ $aTree find $aName]
 IHistogram2D $aName -this [ IManagedObject_to_IHistogram2D [& object] ]
 delete object
}

# Used to delete a SWIG handle and associated object :
#proc HDelete {aObject} {
# if { [info commands $aObject] != "" } { 
#   histogramManager destroy [ $aObject cget -this]
#   rename $aObject ""
# }
#}

# When working with the type manager :
proc fill1D { aHisto aExpression } {
 set ss [new_string 3]
 set_string $ss 0 MemoryTree
 set_string $ss 1 $aHisto
 set_string $ss 2 $aExpression
 typeManager execute histogram 3 $ss
 delete_string $ss
}
proc fill2D { aHisto aExpressionX aExpressionY } {
 set ss [new_string 4]
 set_string $ss 0 MemoryTree
 set_string $ss 1 $aHisto
 set_string $ss 2 $aExpressionX
 set_string $ss 3 $aExpressionY
 typeManager execute histogram 4 $ss
 delete_string $ss
}

#///////////////////////////////////////////////////////
#//// Tuple ////////////////////////////////////////////
#///////////////////////////////////////////////////////

proc RioTuple { aHandle aName aLabel aTree } {
 ITupleFactory tf -this [aida createTupleFactory [& $aTree]]
 ITuple $aHandle -this [ tf create $aName $aLabel ""]
 del_ITupleFactory [& tf]
 rename tf ""
}
proc AsciiTuple { aHandle aName aLabel aTree aLabels} {
 ITupleFactory tf -this [aida createTupleFactory [& $aTree]]
 ITuple $aHandle -this [ tf create $aName $aLabel $aLabels ]
 del_ITupleFactory [& tf]
 rename tf ""
}

proc Evaluator { aHandle aScript aTree} {
 ITupleFactory tf -this [aida createTupleFactory [& $aTree]]
 IEvaluator $aHandle -this [tf createEvaluator $aScript ]
 del_ITupleFactory [& tf]
 rename tf ""   
}

proc Filter { aHandle aScript aTree} {
 ITupleFactory tf -this [aida createTupleFactory [& $aTree]]
 IFilter $aHandle -this [tf createFilter $aScript ]
 del_ITupleFactory [& tf]
 rename tf ""   
}

#///////////////////////////////////////////////////////
#//// Function /////////////////////////////////////////
#///////////////////////////////////////////////////////
proc Function { aHandle aName aLabel aType aVec {aTree memoryTree} } {
 IFunctionFactory ff -this [aida createFunctionFactory [& $aTree]]
 IFunction $aHandle -this [ff create $aName $aLabel $aType $aVec $aNames]
 del_IFunctionFactory [& ff]
 rename ff ""
}

proc FitHistogramGauss { aHandle aHistogram } {
 IFitFactory ff -this [aida createFitFactory]
 IFitter fitter -this [ff createFitter "" "" ""]
 IFitResult fitResult -this [fitter fit [& $aHistogram] "G"]
 IFunction $aHandle -this [fitResult fittedFunction]
 del_IFitResult [& fitResult]
 rename fitResult ""
 del_IFitter [& fitter]
 rename fitter ""
 del_IFitFactory [& ff]
 rename ff ""
}

#///////////////////////////////////////////////////////
#//// Cloud ////////////////////////////////////////////
#///////////////////////////////////////////////////////
proc Cloud1D { aHandle aLabel {aTree memoryTree} } {
 ICloudFactory cf -this [aida createCloudFactory [& $aTree]]
 ICloud1D $aHandle -this [cf create1D $aHandle $aLabel]
 del_ICloudFactory [& cf]
 rename cf ""
}
proc Cloud2D { aHandle aLabel {aTree memoryTree} } {
 ICloudFactory cf -this [aida createCloudFactory [& $aTree]]
 ICloud2D $aHandle -this [cf create2D $aHandle $aLabel]
 del_ICloudFactory [& cf]
 rename cf ""
}
proc Cloud3D { aHandle aLabel {aTree memoryTree} } {
 ICloudFactory cf -this [aida createCloudFactory [& $aTree]]
 ICloud3D $aHandle -this [cf create3D $aHandle $aLabel]
 del_ICloudFactory [& cf]
 rename cf ""
}
proc Cloud2DFromFunction { aHandle aFunction {aTree memoryTree} } {
 ICloudFactory cf -this [aida createCloudFactory [& $aTree]]
 ICloud2D $aHandle -this [cf createFromFunction $aFunction]
 del_ICloudFactory [& cf]
 rename cf ""
}
proc Cloud2DFromTuple { aHandle aTuple aX aY aCuts {aTree memoryTree} } {
 ICloudFactory cf -this [aida createCloudFactory [& $aTree]]
 ICloud2D $aHandle -this [cf createFromTuple $aTuple $aX $aY $aCuts]
 del_ICloudFactory [& cf]
 rename cf ""
}

#///////////////////////////////////////////////////////
#//// Plotter //////////////////////////////////////////
#///////////////////////////////////////////////////////
proc Plotter { aHandle { aName "" } } {
 IPlotterFactory pf -this [aida createPlotterFactory ]
 IPlotter $aHandle -this [pf create $aName]
 del_IPlotterFactory [& pf]
 rename pf ""   
}
#///////////////////////////////////////////////////////
#//// Else /////////////////////////////////////////////
#///////////////////////////////////////////////////////
proc echo_label { aHisto} {
 std_string ss -this [ $aHisto title]
 ui echo [ ss c_str ]
 delete ss
}

