#ifndef Lib_BaseML_h
#define Lib_BaseML_h 

#include <vector>
#include <string>

#include <expat.h>

class IPrinter;

namespace Lib {

class IFileGetter;

class ItemMLFactory;
class ItemML;

class BaseML {
public:
  typedef std::pair< std::string, std::pair< std::string, std::string > > Alias;
public:
  BaseML(ItemMLFactory&,int = 0);
  virtual ~BaseML();
  bool loadFile(const std::string&,bool,IPrinter&);
  bool loadString(const std::string&,IPrinter&);
  void setTags(const std::vector<std::string>&);
  void setAliases(const std::vector<Alias>&);
  void addAlias(const std::string&,const std::string&,const std::string&);
  ItemML* topItem();
  void empty();
  bool isTag(const std::string&);
  bool findAlias(const std::string&,Alias&);
  void setFileGetter(IFileGetter*);
private:
  void clear();
  bool readFile(const std::string&,
                 XML_StartElementHandler,XML_EndElementHandler,void*,
                bool,IPrinter&);
  bool readString(const std::string&,
                 XML_StartElementHandler,XML_EndElementHandler,void*,
                IPrinter&);
  bool solveReferences(Lib::ItemML&,bool,IPrinter&);
  bool parseFile(const std::string&,
                   XML_StartElementHandler,XML_EndElementHandler,void*,
                 bool,IPrinter&);
  bool parseString(const std::string&,
                    XML_StartElementHandler,XML_EndElementHandler,void*,
                   IPrinter&);
  static void characterDataHandler(void*,const XML_Char*,int);
  static void startElement(void*,const XML_Char*,const XML_Char**);
  static void endElement(void*,const XML_Char*);
private:
  ItemMLFactory& fItemMLFactory;
  std::vector<std::string> fTags;
  std::vector<Alias> fAliases;
  int fVerboseLevel;
  ItemML* fFirstCreatedItem;
  ItemML* fCurrent;
  std::string fItem;
  std::vector<std::string> fAtts;
  std::string fValue;
  IFileGetter* fFileGetter;
};

}

#endif
