#ifndef Lib_BigVector_h
#define Lib_BigVector_h 

// Compared to std::vector, BigVector gains 
// at fill time, but wastes time when clearing (then deleting).

#include <vector>

namespace Lib {

template <class T> class BigVector {
public:
  BigVector(int aChunck = 128)
  :fChunck(aChunck),fAllocated(0),fSize(0){
    if(fChunck<=0) fChunck = 128;
  }
  BigVector(const BigVector<T>& aFrom)
  :fChunck(aFrom.fChunck),fAllocated(0),fSize(0) {
    T v;
    int fromSize = aFrom.fSize;
    for(int index=0;index<fromSize;index++) {  
      aFrom.value(index,v);
      push_back(v);
    }
  }
  virtual ~BigVector() { clear();}
  BigVector<T>& operator=(const BigVector<T>& aFrom) {
    clear();
    fChunck = aFrom.fChunck;
    T v;
    int fromSize = aFrom.fSize;
    for(int index=0;index<fromSize;index++) {  
      aFrom.value(index,v);
      push_back(v);
    }
    return *this;
  }
public:
  int size() const { return fSize;}
  const T& operator[](int aIndex) const {
    int ichunck = aIndex / fChunck;
    int index = aIndex % fChunck;
    return (*(fData[ichunck]))[index];
  }
  T& operator[](int aIndex) {
    int ichunck = aIndex / fChunck;
    int index = aIndex % fChunck;
    return (*(fData[ichunck]))[index];
  }
  void push_back(const T& aValue) {
    if( fSize >= fAllocated) {
      fData.push_back(new std::vector<T>(fChunck));
      fAllocated += fChunck;
    }
    int ichunck = fSize / fChunck;
    int index = fSize % fChunck;
    (*(fData[ichunck]))[index] = aValue;
    fSize++;
  }
  void clear() {
    unsigned int number = fData.size();
    for(unsigned int index=0;index<number;index++) delete fData[index];
    fData.clear();
    fAllocated = 0;
    fSize = 0;    
  }
  bool value(int aIndex,T& aValue) const {
    if((aIndex<0) || (aIndex>=fSize)) {
      //FIXME aValue = fDefault;
      return false;
    } else {
      int ichunck = aIndex / fChunck;
      int index = aIndex % fChunck;
      aValue = (*(fData[ichunck]))[index];
      return true;
    }
  }
  void valueNoCheck(int aIndex,T& aValue) const {
    int ichunck = aIndex / fChunck;
    int index = aIndex % fChunck;
    aValue = (*(fData[ichunck]))[index];
  }
private:
  int fChunck;
  std::vector< std::vector<T>* > fData;
  int fAllocated;
  int fSize;
};

}

#endif
