#ifndef Lib_Expression_h
#define Lib_Expression_h

#include <string>
#include <vector>

class IPrinter;

#include <Lib/Variable.h>

namespace Lib {

class Expression {
public:
  enum eType {
    GT = 1,
    GE = 2,
    LT = 3,
    LE = 4,
    EQ = 5,
    NE = 6,
    ADD = 7,
    MUL = 8,
    SUB = 9,
    DIV = 10,
    AND = 11,
    OR = 12,
    ASSIGN = 13,
    MINUS = 14,
    UNSIGNED_LONG = 15,
    REAL = 16,
    NAME = 17,
    STRING = 18,
    PI = 19,
    FUNC = 20,
    BOOL_TRUE = 21,
    BOOL_FALSE = 22,
    NOT = 23
  };
  enum eFunction {
    COS = 1,
    SIN = 2,
    TAN = 3,
    ACOS = 4,
    ASIN = 5,
    ATAN = 6,
    SQRT = 7,
    POW = 8,
    LOG = 9,
    LOG10 = 11,
    EXP = 12,
    FABS = 13,
    COSH = 14,
    SINH = 15,
    TANH = 16,
    GAUSS = 17,
    BW = 18,
    EXPO = 19,
    POL1 = 20,
    POL2 = 21,
    POL3 = 22,
    POL4 = 23,
    TOINT = 24,
    MOD = 25,
    ARRAY = 26,
    NCO = 27,
    VSUM = 28,
    RNDM = 29
  };
  Expression(IPrinter&,eType);
  Expression(IPrinter&,eType,Expression*);
  Expression(IPrinter&,eType,Expression*,Expression*);
  Expression(IPrinter&,eType,eFunction,Expression*);
  Expression(IPrinter&,eType,eFunction,Expression*,Expression*);
  Expression(IPrinter&,eType,eFunction,Expression*,Expression*,Expression*);
  Expression(IPrinter&,eType,eFunction,Expression*,Expression*,Expression*,Expression*);
  Expression(IPrinter&,eType,eFunction,Expression*,Expression*,Expression*,Expression*,Expression*);
  Expression(IPrinter&,eType,eFunction,Expression*,Expression*,Expression*,Expression*,Expression*,Expression*);
  Expression(IPrinter&,eType,unsigned long);
  Expression(IPrinter&,eType,double);
  Expression(IPrinter&,eType,const std::string&);
  Expression(IPrinter&,eType,int);
  virtual ~Expression();
  void print(IPrinter&);
  Variable execute(const std::vector<Variable>&);
private:
  IPrinter& fPrinter;
  eType fType;
  eFunction fFunction;
  Variable fVariable;
  int fIndex;
  //Below, can't have objects of same class than the class.
  Expression* fA; 
  Expression* fB;
  Expression* fC;
  Expression* fD;
  Expression* fE;
  Expression* fF;
};

}

#endif



