#ifndef Lib_FunctionManager_h
#define Lib_FunctionManager_h

// Inheritance :
#include <Lib/Interfaces/IFunction.h>
#include <Lib/Interfaces/IManager.h>
#include <Lib/Interfaces/IFunctionManager.h>

#include <list>

class ISession;

namespace Lib {

class Function : public virtual IFunction {
public: // IFunction :
  virtual const std::string& file() const;
  virtual const std::string& name() const;
  virtual void* address() const;
  virtual const std::string& signature() const;
  virtual unsigned int dimension() const;
  virtual double value(const std::vector<double>&) const;
public:
  Function(const std::string&,
           const std::string&,
           void*,
           const std::string&);
  virtual ~Function();
private:
  std::string fFile;
  std::string fName;
  void* fAddress;
  std::string fSignature;
  std::vector<std::string> fTypes;
  std::vector<unsigned int> fDims;
};

class FunctionManager 
:public virtual IManager 
,public virtual IFunctionManager 
{
public: //IManager
  virtual const std::string& name() const;
  virtual void* cast(const std::string&) const;
public: //IFunctionManager
  virtual IFunction* addFunction(const std::string&,
                                 const std::string&,
                                 void*,
                                 const std::string&);
  virtual IFunction* findFunction(const std::string&);
  virtual bool load(const std::string&,const std::string& = "");
public:
  FunctionManager(ISession&);
  virtual ~FunctionManager();
private:
  bool loadF77(const std::string&,const std::string& = "");
  bool loadC(const std::string&,const std::string& = "");
private:
  ISession& fSession;
  std::string fName;
  std::list<IFunction*> fList;
};

}

#endif




