#ifndef Lib_ItemML_h
#define Lib_ItemML_h 

#include <string>
#include <vector>
#include <list>

class IPrinter;

namespace Lib {

class ItemML;

class ItemMLFactory {
public:
  ItemMLFactory();
  virtual ~ItemMLFactory();
public:
  virtual ItemML* create(const std::string&,ItemML*);
};

typedef bool (*ExecFunction)(ItemML&,void*);

class Element;

class ItemML {
public:
  ItemML(ItemMLFactory&,ItemML*);
  virtual ~ItemML();
  virtual void* cast(const std::string&) const;
public:
  void addProperty(const std::string&,
                   const std::vector<std::string>&,
                   const std::string&);
  void print(IPrinter&);
  bool getPropertyValue(const std::string&,std::string&) const;
  void addAttributes(const std::vector<std::string>&);
  bool getAttValue(const std::string&,const std::string&,std::string&) const;
  const std::list<Element*>& properties() const;
  void setSaveFlag(bool);
  bool saveFlag() const;
  void setFile(const std::string&);
  const std::string& file() const;
  bool setPropertyValue(const std::string&,const std::string&,int = 0);
  bool setAttValue(const std::string&,const std::string&,const std::string&,int = 0);
  bool removeAttribute(const std::string&);
  bool getPropertyValueBoolean(const std::string&,bool&) const;
  void deleteProperty(const std::string&);
  void deleteProperty(Lib::Element*);
  const std::vector<std::string>& attributes() const;
  void postExecute(ExecFunction,void*);

  ItemML* parent() const;
  const std::list<ItemML*>& children();
  ItemML* findItem(const std::string&);
  ItemML* findByAttribute(const std::string&,const std::string&);
  ItemML* findByPropertyInSameLevel(const std::string&,const std::string&);
  void addChild(ItemML*);
  void setParent(ItemML*);
  void addAfter(ItemML*,ItemML*);
  int numberOfChildren() const;
  bool isFirstChild(const ItemML&);
  bool isLastChild(const ItemML&);
  ItemML* previousChild(const ItemML&);
  ItemML* nextChild(const ItemML&);
  void removeChild(ItemML*);
  void updateTree(ItemML*);
  ItemML* createCopy(ItemML*);

  void setData(void*,void*,int);
  void* getData1() const;
  void* getData2() const;
  int getDataInt() const;
  void getData(void*&,void*&,int&) const;
protected:
  void clear();
private:
  ItemMLFactory& fItemMLFactory;
  ItemML* fParent;
  std::list<ItemML*> fChildren;
  std::list<Element*> fProperties;
  std::vector<std::string> fAtts;
  std::string fFile;
  bool fSaveFlag;
  void* fData1;
  void* fData2;
  int fDataInt;
};

}

#endif
