#ifndef Lib_LibraryManager_h
#define Lib_LibraryManager_h

// Inheritance :
#include <Lib/Interfaces/ILibrary.h>
#include <Lib/Interfaces/IManager.h>
#include <Lib/Interfaces/ILibraryManager.h>

#include <list>

class ISession;
class ILibrary;

namespace Lib {

class DLL;

class Library : public virtual ILibrary {
public: // ILibrary :
  virtual const std::string& name() const;
  virtual bool isInitialized() const;
  virtual ILibraryProcedure findProcedure(const std::string&) const;
public:
  Library(ISession&,
          const std::string&,
          const std::string&,
          const std::string&);
  Library(ISession&,const std::string&,DLL*,const std::string&);
  virtual ~Library();
  const std::string& package() const;
  bool execute(const std::string&) const;
private:
  ISession& fSession;
  std::string fName;
  std::string fLibrary;
  std::string fPackage;
  DLL* fDLL;
};

class LibraryManager
:public virtual IManager 
,public virtual ILibraryManager 
{
public: //IManager
  virtual const std::string& name() const;
  virtual void* cast(const std::string&) const;
public: //ILibraryManager
  virtual ILibrary* addLibrary(const std::string&,
                               const std::string&,
                               const std::string&);
  virtual ILibrary* addLibrary(const std::string&,
                               Lib::DLL*,
                               const std::string&);
  virtual ILibrary* findLibrary(const std::string&);
public:
  LibraryManager(ISession&);
  virtual ~LibraryManager();
private:
  ISession& fSession;
  std::string fName;
  std::list<ILibrary*> fList;
};

}

#endif




