#ifndef Lib_Message_h
#define Lib_Message_h

#include <Lib/Interfaces/IMessage.h>

namespace Lib {

class BaseMessage : public virtual Lib::IMessage {
public:
  BaseMessage();
  virtual ~BaseMessage();
};

class ExitMessage : public BaseMessage {
public:
  virtual void* cast(const std::string&) const;
  virtual std::string message() const;
public:
  ExitMessage(int = 0);
  virtual ~ExitMessage();
  int code() const;
private:
  int fCode;
};

class CreateMessage : public BaseMessage {
public:
  virtual void* cast(const std::string&) const;
  virtual std::string message() const;
public:
  CreateMessage(const std::string& = "");
  virtual ~CreateMessage();
  const std::string& what() const;
private:
  std::string fWhat;
};

class DeleteMessage : public BaseMessage {
public:
  virtual void* cast(const std::string&) const;
  virtual std::string message() const;
public:
  DeleteMessage(const std::string& = "");
  virtual ~DeleteMessage();
  const std::string& what() const;
private:
  std::string fWhat;
};

class UpdateMessage : public BaseMessage {
public:
  virtual void* cast(const std::string&) const;
  virtual std::string message() const;
public:
  UpdateMessage(const std::string& = "");
  virtual ~UpdateMessage();
  const std::string& what() const;
private:
  std::string fWhat;
};

class IAction;

class ExecuteMessage : public BaseMessage {
public:
  virtual void* cast(const std::string&) const;
  virtual std::string message() const;
public:
  ExecuteMessage(IAction&);
  virtual ~ExecuteMessage();
  IAction& action() const;
private:
  IAction& fAction;
};

class IRelation;

class RelationMessage : public BaseMessage {
public: //IMessage:
  virtual void* cast(const std::string&) const;
  virtual std::string message() const;
public:
  RelationMessage(IRelation&);
  virtual ~RelationMessage();
  IRelation& relation() const;
private:
  IRelation& fRelation;
};

}

#endif
