#ifndef Lib_Parameters_h
#define Lib_Parameters_h

#include <list>
#include <string>
#include <vector>

namespace Lib {

template <class K,class V>
class Parameter {
public:
  Parameter(
   const K& aKey
  ,const std::string& aLabel
  ,const V& aValue
  ,const V& aDefault
  ,const V& aAlternative
  )
  :fKey(aKey)
  ,fLabel(aLabel)
  ,fValue(aValue)
  ,fDefault(aDefault)
  ,fAlternative(aAlternative){
  }
  virtual ~Parameter(){}
  Parameter(const Parameter& aFrom)
  :fKey(aFrom.fKey)
  ,fLabel(aFrom.fLabel)
  ,fValue(aFrom.fValue)
  ,fDefault(aFrom.fDefault)
  ,fAlternative(aFrom.fAlternative){
  }
  Parameter& operator =(const Parameter& aFrom){
    fKey = aFrom.fKey;
    fLabel = aFrom.fLabel;
    fValue = aFrom.fValue;
    fDefault = aFrom.fDefault;
    fAlternative = aFrom.fAlternative;
    return *this;
  }
  //friend int operator==(const Parameter& a1,const Parameter& a2){
  //  if(a1.fKey!=a2.fKey) return 0;
  //  if(a1.fLabel!=a2.fLabel) return 0;
  //  if(a1.fValue!=a2.fValue) return 0;
  //  if(a1.fDefault!=a2.fDefault) return 0;
  //  if(a1.fAlternative!=a2.fAlternative) return 0;
  //  return 1;
  //}
  const K& key() const { return fKey;}
  const std::string& label() const { return fLabel;}
  const V& value() const { return fValue;}
  const V& defaultValue() const { return fDefault;}
  const V& alternative() const { return fAlternative;}
public:
  K fKey;
  std::string fLabel;
  V fValue;
  V fDefault;
  V fAlternative;
};


template <class K,class V>
class Parameters {
public:
  typedef std::list< Parameter<K,V> > List;
  typedef typename List::iterator Iterator;
  typedef typename List::const_iterator ConstIterator;

  void add(const K& aKey,const std::string& aLabel,const V& aValue,const V& aDefault,const V& aAlternative) {
    fList.push_back
      (Parameter<K,V>(aKey,aLabel,aValue,aDefault,aAlternative));
  }
  void remove(const K& aKey) {
    Iterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        //fList.remove(*it); //Need a operator==(Parameter,Parameter)
        fList.erase(it);
        return;
      }
    } 
  }
  void setValue(const K& aKey,V aValue) {
    Iterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        (*it).fValue = aValue;
        return;
      }
    } 
  }
  void setAllToDefault() {
    Iterator it;
    for(it=fList.begin();it!=fList.end();++it)
      (*it).fValue = (*it).fDefault;
  }
  void setToDefault(const K& aKey) {
    Iterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        (*it).fValue = (*it).fDefault;
      }
    }
  }
  std::string label(const K& aKey) const {
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        return (*it).fLabel;
      }
    }
    return "";
  }
  V value(const K& aKey) const {
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        return (*it).fValue;
      }
    }
    return fValueNotFound;
  }
  V defaultValue(const K& aKey) const {
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        return (*it).fDefault;
      }
    }
    return fValueNotFound;
  }
  const K& key(V aValue) const {
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aValue==(*it).fValue) { //Need operator==(V,V)
        return (*it).fKey;
      }
    }
    return fKeyNotFound;
  }
  const K& keyBoolean(V aValue) const {
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if( (aValue==(*it).fDefault) || 
          (aValue==(*it).fAlternative) ) { //Need operator==(V,V)
        return (*it).fKey;
      }
    }
    return fKeyNotFound;
  }
  bool isKey(const K& aKey) const {
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        return true;
      }
    }
    return false;
  }
  const List& list() const { return fList;}
  const K& keyNotFound() const { return fKeyNotFound;}
  V valueNotFound() const { return fValueNotFound;}
  std::vector<K> keys() const {
    std::vector<K> ks;
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      ks.push_back((*it).fKey);
    }
    return ks;
  }
public:
  Parameters(const K& aKeyNotFound,V aValueNotFound)
  :fKeyNotFound(aKeyNotFound),fValueNotFound(aValueNotFound){}
  virtual ~Parameters(){}
protected:
  K fKeyNotFound;
  V fValueNotFound;
  List fList;
};

}

#endif



