#ifndef Lib_Processor_h
#define Lib_Processor_h 

class IPrinter;
#include <string>
#include <vector>

#include <Lib/Variable.h>
#include <Lib/Expression.h>
#include <Lib/ProcessorLexer.h>

namespace Lib {

class Processor {
public:
  Processor(const std::string&,IPrinter&);
  virtual ~Processor();
  Processor& operator=(const Processor&);
  void setScript(const std::string&);
  void findVariables(std::vector<std::string>&);
  void compile();
  void compile(const std::vector<std::string>&);
  bool isSuccess() const;
  Variable execute(const std::vector<Lib::Variable>&);
  bool cut(const std::vector<Lib::Variable>&);
  double calc() const;
  double calc(const std::vector<double>&) const;
public:  // Internal :
  bool findNames() const;
  void addName(const std::string&);
  int findVariable(const std::string&);
  const ProcessorLexer& lexer() const;
  IPrinter& printer() const;
private:
  void clear();
public:
  std::string fString;
  IPrinter& fPrinter;
  bool fFindNames;
  std::vector<std::string> fNames;
  ProcessorLexer fLexer;
  Expression* fResult;
  bool fSuccess;
  std::vector<Expression*> fTmpExps;
};

}

#endif
