#ifndef Lib_SpaceAlgebra_h
#define Lib_SpaceAlgebra_h 

namespace Lib {

class SpaceVector {
public:
  SpaceVector();
  SpaceVector(const double[3]);
  SpaceVector(double,double,double);
  SpaceVector cross(const SpaceVector &) const;
  // Returns dot (inner) product of vector and another vector
  double dot(const SpaceVector &v) const;
  double normalize();
  SpaceVector& setValue(const double [3]);
  SpaceVector& setValue(double,double,double);
  void value(double&,double&,double&) const;
  double length() const;
  //Operators
  double& operator[](int i);
  const double& operator[](int i) const;
  SpaceVector& operator*=(double);
  SpaceVector& operator/=(double);
  SpaceVector& operator+=(SpaceVector);
  SpaceVector& operator-=(SpaceVector);
private:
  double fVec[3];
};

class SpaceLine {
public:
  SpaceLine();
  SpaceLine(const SpaceVector&,const SpaceVector&);
  void setValue(const SpaceVector&,const SpaceVector&);
  const SpaceVector& position() const;
  const SpaceVector& direction() const;
private:
  SpaceVector fPos;
  SpaceVector fDir;
};

class SpacePlane {
public:
  SpacePlane();
  SpacePlane(const SpaceVector&,const SpaceVector&,const SpaceVector&);
  SpacePlane(const SpaceVector&,double);
  SpacePlane(const SpaceVector&,const SpaceVector&);
  void offset(double);
  bool intersect(const SpaceLine&,SpaceVector&) const;
  bool isInHalfSpace(const SpaceVector&) const;
  const SpaceVector& normal() const;
  double distanceFromOrigin() const;
private:
  SpaceVector fNormal;
  double fDistance;
};

}

#endif

