#ifndef Lib_TypeManager_h
#define Lib_TypeManager_h

#include <Lib/Interfaces/IManager.h>
#include <Lib/Interfaces/ITypeManager.h>
#include <Lib/Interfaces/IAction.h>

#include <Lib/Handler.h>

#include <list>

namespace Lib {

class TypeManager 
:public virtual IManager
,public virtual ITypeManager {
public: //IManager
  virtual const std::string& name() const;
  virtual void* cast(const std::string&) const;
public: //ITypeManager
  virtual void addType(IType*);
  virtual void removeType(const std::string&);
  virtual void addAction(IAction*);
  virtual void eraseHandlers();
  virtual IType* findType(const std::string&);
  virtual IPrinter& printer() const;
  virtual void addHandler(IType&,Identifier,void*);
  virtual std::list<Lib::Handler>& handlers();
  virtual std::list<Lib::IType*> handlersTypes();
  virtual std::string execute(const std::string&,
                              const std::vector<std::string>&);
  virtual std::vector<std::string> typeNames() const;
  virtual const std::list<Lib::IType*>& types() const;
  virtual void setVerboseLevel(int);
  virtual int verboseLevel() const;
public:
  TypeManager(IPrinter&);
  virtual ~TypeManager();
public:
  void removeType(IType*);
  // Actions :
  IAction* findAction(const std::string&);
protected:
  std::list<Handler> fHandlers;
private:
  IPrinter& fPrinter;
  std::string fName;
  std::list<IType*> fTypes;
  std::list<IAction*> fActions;
  int fVerboseLevel;
};

class CollectAction : public virtual IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual const std::string& name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  CollectAction(ITypeManager&);
  virtual ~CollectAction();
private:
  ITypeManager& fTypeManager;
  std::string fName;
};

class FilterAction : public virtual IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual const std::string& name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  FilterAction(ITypeManager&);
  virtual ~FilterAction();
private:
  ITypeManager& fTypeManager;
  std::string fName;
};

class DumpAction : public virtual IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual const std::string& name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  DumpAction(ITypeManager&);
  virtual ~DumpAction();
private:
  ITypeManager& fTypeManager;
  std::string fName;
};

class NumberAction : public virtual IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual const std::string& name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  NumberAction(ITypeManager&);
  virtual ~NumberAction();
private:
  ITypeManager& fTypeManager;
  std::string fName;
};

class GetValuesAction : public virtual IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual const std::string& name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  GetValuesAction(ITypeManager&);
  virtual ~GetValuesAction();
private:
  ITypeManager& fTypeManager;
  std::string fName;
};

class DestroyAction : public virtual Lib::IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual const std::string& name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  DestroyAction(Lib::ITypeManager&);
  virtual ~DestroyAction();
private:
  Lib::ITypeManager& fTypeManager;
  std::string fName;
};

class SetAction : public virtual Lib::IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual const std::string& name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  SetAction(Lib::ITypeManager&);
  virtual ~SetAction();
private:
  Lib::ITypeManager& fTypeManager;
  std::string fName;
};

class CalculateAction : public virtual Lib::IAction {
public: //IAction
  virtual void* cast(const std::string&) const;
  virtual const std::string& name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  CalculateAction(Lib::ITypeManager&);
  virtual ~CalculateAction();
private:
  Lib::ITypeManager& fTypeManager;
  std::string fName;
};

}

#endif



