#ifndef Lib_Variable_h
#define Lib_Variable_h

class IPrinter;

#include <string>

#include <Lib/Vector.h>

namespace Lib {

class Variable {
public: // Internal :
  enum eType {
    NONE = 0,
    INTEGER = 1,
    DOUBLE = 2,
    STRING = 3,
    POINTER = 4,
    UNSIGNED_LONG = 5,
    BOOLEAN = 6,
    SHORT = 7,
    LONG = 8,
    FLOAT = 9,
    BYTE = 10,
    CHAR = 11,
    VECTOR_DOUBLE = 12,
    VECTOR_UNSIGNED_LONG = 13
  };
public:
  Variable(IPrinter&);
  Variable(const Variable&);
  virtual ~Variable();
  // By value :
  Variable(IPrinter&,int);
  Variable(IPrinter&,double);
  Variable(IPrinter&,const char*);
  Variable(IPrinter&,const std::string&);
  Variable(IPrinter&,void*);
  Variable(IPrinter&,unsigned long);
  Variable(IPrinter&,long);
  Variable(IPrinter&,short);
  Variable(IPrinter&,float);
  Variable(IPrinter&,bool);
  Variable(IPrinter&,const Vector<double>&);
  Variable(IPrinter&,const Vector<unsigned long>&);
  // Else
  Variable& operator =(const Variable&);
  void print(IPrinter&) const;
  std::string type() const;
  std::string toString() const;
  double toDouble() const;
  Variable minus() const;
  bool isNone() const;
  void setType(eType);
  void setName(const std::string&);
  const std::string& name() const;
  void setDimension(unsigned int);
  unsigned int dimension() const;
  bool assign( const Variable&);
  Variable doNot();
  IPrinter& printer() const;
#ifndef __CINT__ //Problems on Windows.
  Variable operator  >(const Variable&);
  Variable operator >=(const Variable&);
  Variable operator <=(const Variable&);
  Variable operator  <(const Variable&);
  Variable operator ==(const Variable&);
  Variable operator !=(const Variable&);
  Variable operator &&(const Variable&);
  Variable operator ||(const Variable&);
  Variable operator  +(const Variable&);
  Variable operator  -(const Variable&);
  Variable operator  *(const Variable&);
  Variable operator  /(const Variable&);
#endif
  bool equal(const Variable&) const;
private:
  void reset();
private:
  IPrinter& fPrinter;
  std::string fName;
  unsigned int fDimension;
public: // Internal :
  eType fType;
  int fInteger;
  double fDouble;
  std::string fString;
  void* fPointer;
  unsigned long fUnsignedLong;
  bool fBoolean;
  short fShort;
  long fLong;
  float fFloat;
  unsigned char fByte;
  char fChar;
  Vector<double> fVectorDouble;
  Vector<unsigned long> fVectorUnsignedLong;
};

//int operator==(const Lib::Variable&,const Lib::Variable&);
//int operator!=(const Lib::Variable&,const Lib::Variable&);

}

//int operator==(const Lib::Variable&,const Lib::Variable&);
//int operator!=(const Lib::Variable&,const Lib::Variable&);

#endif



