#ifndef Lib_Vectors_h
#define Lib_Vectors_h

// Inheritance :
#include <Lib/Vector.h>

class IPrinter;

namespace Lib {

class Vector1d : public Vector<double> {
public:
  Vector1d();
  Vector1d(int);
  Vector1d(const Vector1d&);
  virtual ~Vector1d();
  Vector1d& operator=(const Vector1d&);
};

class Vector2d : public Vector<double> {
public:
  Vector2d();
  Vector2d(int,int);
  Vector2d(const Vector2d&);
  virtual ~Vector2d();
  Vector2d& operator=(const Vector2d&);
};

namespace Vectors {

//////////////////////////////////////////////////////////////////////////////
template <class T>
bool contract(
 const Vector<T>& aA
,const Vector<T>& aB
,int aIndex
,Vector<T>& aR
)
//////////////////////////////////////////////////////////////////////////////
// Contract a tensor (aA) with a vector (aB) on indice aIndex of aA.
// aA.dimension must be > 1.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aA.dimension()<=1) return false; //FIXME : handle contract dim = 1.
  if(aB.dimension()!=1) return false;
  if((aIndex<0)||(aIndex>=aA.dimension())) return false;
  if(aA.orders()[aIndex]!=aB.orders()[0]) return false;

  int rdim = aA.dimension()-1;
  std::vector<int> rorders(rdim);
  int index;
  for(index=0;index<aIndex;index++) rorders[index] = aA.orders()[index];
  for(index=aIndex;index<rdim;index++) rorders[index] = aA.orders()[index+1];
  Vector<T> result(rorders);

  std::vector<int> ais(aA.dimension());
  std::vector<int> bis(aB.dimension());
  std::vector<int> ris(result.dimension());

  //FIXME : optimize the below.
  int order = aA.orders()[aIndex];
  unsigned int rsize = result.size();
  for(unsigned int roffset=0;roffset<rsize;roffset++) {
    result.indices(roffset,ris);
    for(index=0;index<aIndex;index++) ais[index] = ris[index];
    for(index=aIndex;index<rdim;index++) ais[index+1] = ris[index];
    double value = 0;
    for(index=0;index<order;index++) {
      ais[aIndex] = index;
      double av;
      aA.value(ais,av);

      bis[0] = index;
      double bv;
      aB.value(bis,bv);

      value += av * bv;
    }
    result.setValue(ris,value);
  }

  aR = result;
  return true;
}

}

}

#endif




