#ifndef Lib_mxmanip_h
#define Lib_mxmanip_h

#include <Lib/Matrix.h>

namespace Lib {

namespace mxmanip {

template <class T>
Matrix<T> commutator(const Matrix<T>& a1,const Matrix<T>& a2) {
  Matrix<T> m1 = a1;
  m1.mul(a2);
  Matrix<T> m2 = a2;
  m2.mul(a1);
  m1.sub(m2);
  return m1;
}

template <class T>
Matrix<T> antiCommutator(const Matrix<T>& a1,const Matrix<T>& a2) {
  Matrix<T> m1 = a1;
  m1.mul(a2);
  Matrix<T> m2 = a2;
  m2.mul(a1);
  m1.add(m2);
  return m1;
}

template <class T>
Matrix<T> identity(int aOrder) {
  int order = aOrder;
  if(order<=0) order = 1;
  int n = order * order;
  std::vector<T> v(n);
  int index;
  for(index=0;index<n;index++) v[index] = 0;
  for(index=0;index<order;index++) v[index * order + index] = 1;
  return Matrix<T>(order,v);
}

template <class T>
Matrix<T> exp(const Matrix<T>& aMatrix,int aNumber = 100) {
  Matrix<T> result = identity<T>(aMatrix.order());
  Matrix<T> m = identity<T>(aMatrix.order());     
  for(int i=1;i<aNumber;i++) {
    m.mul(aMatrix); 
    m.mul(1./double(i)); 
    result.add(m);
  }
  return result;
}

template <class T>
Matrix<double> abs(const Matrix<T>& aMatrix,double(*aFunc)(T)) {
  int order = aMatrix.order();
  int n = order * order;
  std::vector<double> v(n);
  for(int index=0;index<n;index++) v[index] = aFunc(aMatrix.value(index));
  return Matrix<double>(order,v);
}

template <class T>
bool isLogConvergent(const Matrix<T>& aMatrix,double(*aFunc)(T)) {
  int order = aMatrix.order();
  Matrix<T> s(aMatrix);
  s.sub(identity<T>(order));
  Matrix<double> a = abs(s,aFunc);
  double radius = 1./order;
  int n = order * order;
  for(int index=0;index<n;index++) {
    if(a.value(index)>=radius) {
      //::printf("debug : not convergent at : %d : %g rad %g\n",
        //index,a.value(index),radius);
      return false;
    }
  }
  return true;
}

template <class T>
Matrix<T> log(const Matrix<T>& aMatrix,int aNumber = 100) {
  // Warning : cannot converge...
  Matrix<T> result(aMatrix.order());
  Matrix<T> id = identity<T>(aMatrix.order());     
  Matrix<T> x(aMatrix);     
  x.sub(id);
  Matrix<T> m(id);
  Matrix<T> tmp(aMatrix.order());
  double fact = -1;
  for(int i=0;i<aNumber;i++) {
    m.mul(x); 
    fact *= -1;
    tmp = m;
    tmp.mul(fact/double(i+1)); 
    result.add(tmp);
  }
  return result;
}

}

}

#endif
