#ifndef Lib_smanip_h
#define Lib_smanip_h

#include <string>
#include <vector>

class IWriter;

namespace Lib {

class smanip {
public:
  enum what { leading, trailing, both };
  static void strip(std::string&,what = both,char = ' ');
  enum side { left, right, middle };
  static bool justify(std::string&,unsigned int,side = left);
  static bool match(const std::string&,const std::string&);
  static bool getenv(std::string&);
  static bool replace(std::string&,const std::string&,const std::string&);
  static bool printf(std::string&,int,const char*,...);
  //static bool scanf(const std::string&,const char*,...);
  static bool tolong(const std::string&,long&);
  static bool toint(const std::string&,int&);
  static bool toulong(const std::string&,unsigned long&);
  static bool tobool(const std::string&,bool&);
  static bool tofloat(const std::string&,float&);
  static bool todouble(const std::string&,double&);
  static bool tolongs(const std::string&,std::vector<long>&);
  static bool toints(const std::string&,std::vector<int>&);
  static bool todoubles(const std::string&,std::vector<double>&,const std::string& = " ");
  // On OSF1 tolower, toupper are CPP macros !
  static void tolowercase(std::string&);
  static void touppercase(std::string&);
  static bool togeometry(const std::string&,int&,int&,int&,int&);
  // Return value to be deleted with delete [].
  static char* tocstring(const std::string&);
  static bool polynomialDegree(const std::string&,int&);
  static int axtoi(const std::string&); 
  static bool pathNameSuffix(const std::string&,
                     std::string&,std::string&,std::string&);
  static std::string suffix(const std::string&);
  static std::string nosuffix(const std::string&);
  static void lines(const std::string&,std::vector<std::string>&);
  static std::vector<std::string> words(const std::string&,const std::string&,bool=false);
  static void words(const std::string&,const std::string&,std::vector<std::string>&,bool=false);
  static void dump(const std::string&,IWriter&);

  static void remove(std::vector<std::string>&,const std::string&);
  static void unique(std::vector<std::string>&);
  static int lineIndex(const std::vector<std::string>&,const std::string&);
  static void write(const std::vector<std::string>&,IWriter&);
  static bool torgb(const std::string&,double&,double&,double&);
  static bool torgbs(const std::string&,std::string&);
  static void justifylines(const std::vector<std::string>&,std::vector<std::string>&);
  static void strip(std::vector<std::string>&,what = both,char = ' ');
  static bool replace(std::vector<std::string>&,const std::string&,const std::string&);
  static void sort(std::vector<std::string>&,bool = true);
  static std::string tostring(const std::vector<std::string>&,const std::string&);
  static std::string tostring(char);
  static std::string tostring(short);
  static std::string tostring(int);
  static std::string tostring(unsigned int);
  static std::string tostring(long);
  static std::string tostring(unsigned long);
  static std::string tostring(float);
  static std::string tostring(double);
  static std::string tostring(bool);
  static std::string tostring(const char*,unsigned int);
  static bool options(const std::string&,
              std::vector< std::pair<std::string,std::string> >&);
  static bool option(const std::string&,const std::string&,std::string&);
  static bool version(const std::string&,std::vector<int>&);
  static bool binRoot(const std::string&,std::string&);
  static bool nameArgs(const std::string&,std::string&,std::vector<std::string>&);
  static bool parseArray(const std::string&,std::string&,int&);
  static std::string int2coord(int);
  static bool coord2int(const std::string&,int&);
  static std::string toxml(const std::string&);
};

class sjust : public std::string {
public:
  sjust(const std::string&,unsigned int,Lib::smanip::side = Lib::smanip::left);
  sjust(int,unsigned int,Lib::smanip::side = Lib::smanip::left);
  sjust(double,unsigned int,Lib::smanip::side = Lib::smanip::left);
  sjust(float,unsigned int,Lib::smanip::side = Lib::smanip::left);
  sjust(char,unsigned int,Lib::smanip::side = Lib::smanip::left);
  sjust(short,unsigned int,Lib::smanip::side = Lib::smanip::left);
  sjust(long,unsigned int,Lib::smanip::side = Lib::smanip::left);
  sjust(bool,unsigned int,Lib::smanip::side = Lib::smanip::left);
};
              
}

#endif
