#ifndef OnX_Callbacks_OnX_h
#define OnX_Callbacks_OnX_h

//
//  This file contains static helper methods.
// It is intended to be included in OnX Callbacks file
// to ease callbacks programming.
//  It relies on interfaces and basic libraries (Lib, stdcxx) only.
//

// This file contains no Inventor related helpers.
// See OnX/Callbacks/Inventor.h for Inventor related ones.

#include <Lib/Interfaces/ISession.h>
#include <Lib/Interfaces/ITypeManager.h>
#include <Lib/Manager.h>
#include <Lib/fmanip.h>
#include <Lib/smanip.h>

#include <OnX/Interfaces/IUI.h>
#include <OnX/Interfaces/IPage.h>
#include <OnX/Interfaces/IRegion.h>

//////////////////////////////////////////////////////////////////////////////
/// lib //////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static std::string lib_torgb(
 const std::string& aColorName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string value;
  Lib::smanip::torgbs(aColorName,value);
  return value;
}
//////////////////////////////////////////////////////////////////////////////
/// session //////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static void session_setColor(
 ISession& aSession
,const std::string& aColor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aSession.setParameter("modeling.color",aColor);
}
//////////////////////////////////////////////////////////////////////////////
static void session_modeling_none(
 ISession& aSession
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Must match list of SbStyle::setFromString.
  aSession.setParameter("modeling.modeling","none");
  aSession.setParameter("modeling.color","none");
  aSession.setParameter("modeling.highlightColor","none");
  aSession.setParameter("modeling.lineStyle","none");
  aSession.setParameter("modeling.linePattern","none");
  aSession.setParameter("modeling.markerStyle","none");
  aSession.setParameter("modeling.areaStyle","none");
  aSession.setParameter("modeling.cut","none");
  aSession.setParameter("modeling.painting","none");
  aSession.setParameter("modeling.projection","none");
  aSession.setParameter("modeling.fontName","none");
  aSession.setParameter("modeling.lineWidth","none");
  aSession.setParameter("modeling.markerSize","none");
  aSession.setParameter("modeling.fontSize","none");
  aSession.setParameter("modeling.transparency","none");
  aSession.setParameter("modeling.multiNodeLimit","none");
  aSession.setParameter("modeling.hatching","none");
  aSession.setParameter("modeling.angle","none");
  aSession.setParameter("modeling.offset","none");
  aSession.setParameter("modeling.stripWidth","none");
  aSession.setParameter("modeling.spacing","none");
  aSession.setParameter("modeling.visible","none");
  aSession.setParameter("modeling.barOffset","none");
  aSession.setParameter("modeling.barWidth","none");

  aSession.setParameter("modeling.localSetup","none");
  // Backward compatibility :
  aSession.setParameter("modeling.type","none");
}
//////////////////////////////////////////////////////////////////////////////
static bool session_setParameter(
 ISession& aSession
,const std::string& aWhat
,int aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  Lib::smanip::printf(s,32,"%d",aValue);
  return aSession.setParameter(aWhat,s);
}
//////////////////////////////////////////////////////////////////////////////
static bool session_setParameter(
 ISession& aSession
,const std::string& aWhat
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  Lib::smanip::printf(s,32,"%g",aValue);
  return aSession.setParameter(aWhat,s);
}
//////////////////////////////////////////////////////////////////////////////
static bool session_setParameter(
 ISession& aSession
,const std::string& aWhat
,bool aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return aSession.setParameter(aWhat,std::string(aValue?"true":"false"));
}
//////////////////////////////////////////////////////////////////////////////
static bool session_parameterValue(
 ISession& aSession
,const std::string& aWhat
,int& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = 0;
  std::string s;
  if(!aSession.parameterValue(aWhat,s)) return false;
  int value;
  if(!Lib::smanip::toint(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static bool session_parameterValue(
 ISession& aSession
,const std::string& aWhat
,double& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = 0;
  std::string s;
  if(!aSession.parameterValue(aWhat,s)) return false;
  double value;
  if(!Lib::smanip::todouble(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static bool session_parameterValue(
 ISession& aSession
,const std::string& aWhat
,bool& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = false;
  std::string s;
  if(!aSession.parameterValue(aWhat,s)) return false;
  bool value;
  if(!Lib::smanip::tobool(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
/// ui ///////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static void ui_help_text(
 IUI& aUI
,const std::string& aString
) 
//////////////////////////////////////////////////////////////////////////////
// aString : text to output in the help dialog (or terminal).
//           It is a list of strings separated by \n.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  ISession& session = aUI.session();
  std::string svalue;
  if(!session.parameterValue("session.helpOnDialog",svalue)) return;
  if(svalue=="yes") {
    // Output on dialog :
    aUI.load("main",Lib::fmanip::name("$ONXROOT/scripts/OnX/HelpDialog.onx"));
    aUI.setParameter("OnX_HelpDialog_text.value",aString);
    aUI.show("OnX_HelpDialog_dialog");
  } else {
    std::vector<std::string> text;
    Lib::smanip::words(aString,"\n",text,true);
    // Output on console :
    for(unsigned int index=0;index<text.size();index++) {
      session.printer().write(text[index]);
      session.printer().write("\n");
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
static void ui_execute_file(
 IUI& aUI
,const std::string& aInterp
,const std::string& aFile
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string file = Lib::fmanip::name(aFile);
  if(aInterp=="Python") {
    std::string path,name,suffix;
    Lib::smanip::pathNameSuffix(file,path,name,suffix);
    std::vector<std::string> words;
    Lib::smanip::words(name,".",words);
    aUI.executeScript
      ("Python","from OnX import sys_import;sys_import('"+words[0]+"')");
  } else if(aInterp=="Tcl") {
    aUI.executeScript("Tcl","source "+file);
  } else if(aInterp=="CINT") {
    aUI.executeScript("CINT","x "+file);
  } else if(aInterp=="KUIP") {
    aUI.executeScript("KUIP","exec "+file);
  }
}
//////////////////////////////////////////////////////////////////////////////
static bool ui_setParameter(
 IUI& aUI
,const std::string& aWhat
,int aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  Lib::smanip::printf(s,32,"%d",aValue);
  return aUI.setParameter(aWhat,s);
}
//////////////////////////////////////////////////////////////////////////////
static bool ui_setParameter(
 IUI& aUI
,const std::string& aWhat
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  Lib::smanip::printf(s,32,"%g",aValue);
  return aUI.setParameter(aWhat,s);
}
//////////////////////////////////////////////////////////////////////////////
static bool ui_setParameter(
 IUI& aUI
,const std::string& aWhat
,bool aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return aUI.setParameter(aWhat,std::string(aValue?"true":"false"));
}
//////////////////////////////////////////////////////////////////////////////
static bool ui_parameterValue(
 IUI& aUI
,const std::string& aWhat
,int& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = 0;
  std::string s;
  if(!aUI.parameterValue(aWhat,s)) return false;
  int value;
  if(!Lib::smanip::toint(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static bool ui_parameterValue(
 IUI& aUI
,const std::string& aWhat
,double& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = 0;
  std::string s;
  if(!aUI.parameterValue(aWhat,s)) return false;
  double value;
  if(!Lib::smanip::todouble(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static bool ui_parameterValue(
 IUI& aUI
,const std::string& aWhat
,bool& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = false;
  std::string s;
  if(!aUI.parameterValue(aWhat,s)) return false;
  bool value;
  if(!Lib::smanip::tobool(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static double ui_cbk_double(
 IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  double value;
  Lib::smanip::todouble(aUI.callbackValue(),value);
  return value;
}
//////////////////////////////////////////////////////////////////////////////
static int ui_cbk_int(
 IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int value;
  Lib::smanip::toint(aUI.callbackValue(),value);
  return value;
}
//////////////////////////////////////////////////////////////////////////////
static const std::string& ui_cbk(
 IUI& aUI
,const std::string& aArg
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArg=="ui_callbackValue") return aUI.callbackValue();
  else return aArg;
}
//////////////////////////////////////////////////////////////////////////////
static void ui_add_prompt(
 IUI& aUI
,const std::string& aArg
) 
//////////////////////////////////////////////////////////////////////////////
// Add a prompt in the consolePrompt list.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string value;
  if(!aUI.parameterValue("consolePrompt.items",value)) {
    aUI.echo("ui_add_prompt : widget consolePrompt not found.");
    return;
  }
  std::vector<std::string> items;
  Lib::smanip::lines(value,items);
  items.push_back(aArg);
  Lib::smanip::unique(items);
  aUI.setParameter("consolePrompt.items",Lib::smanip::tostring(items,"\n"));
}
//////////////////////////////////////////////////////////////////////////////
static IPage* ui_create_page(
 IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int n;
  if(!ui_parameterValue(aUI,"ViewerTabStack.itemCount",n)) return 0;

  std::string name;
  if(!Lib::smanip::printf(name,64,"Viewer_%d",n+1)) return 0;

  aUI.setParameter("Viewer_1.tabLabel",name);
  aUI.copyComponent(name,"Viewer_1","ViewerTabStack");
  aUI.setParameter("Viewer_1.tabLabel","Viewer_1");

  return aUI.currentPage();
}
//////////////////////////////////////////////////////////////////////////////
/// page /////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static bool page_setParameter(
 IPage& aPage
,const std::string& aWhat
,int aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  Lib::smanip::printf(s,32,"%d",aValue);
  return aPage.setParameter(aWhat,s);
}
//////////////////////////////////////////////////////////////////////////////
static bool page_setParameter(
 IPage& aPage
,const std::string& aWhat
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  Lib::smanip::printf(s,32,"%g",aValue);
  return aPage.setParameter(aWhat,s);
}
//////////////////////////////////////////////////////////////////////////////
static bool page_setParameter(
 IPage& aPage
,const std::string& aWhat
,bool aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return aPage.setParameter(aWhat,std::string(aValue?"TRUE":"FALSE"));
}
//////////////////////////////////////////////////////////////////////////////
static bool page_parameterValue(
 IPage& aPage
,const std::string& aWhat
,int& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = 0;
  std::string s;
  if(!aPage.parameterValue(aWhat,s)) return false;
  int value;
  if(!Lib::smanip::toint(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static bool page_parameterValue(
 IPage& aPage
,const std::string& aWhat
,double& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = 0;
  std::string s;
  if(!aPage.parameterValue(aWhat,s)) return false;
  double value;
  if(!Lib::smanip::todouble(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static bool page_parameterValue(
 IPage& aPage
,const std::string& aWhat
,bool& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = false;
  std::string s;
  if(!aPage.parameterValue(aWhat,s)) return false;
  bool value;
  if(!Lib::smanip::tobool(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static void page_connect_current_region(
 IPage& aPage
,int aIndex
)
//////////////////////////////////////////////////////////////////////////////
// Connect the current region to the "aIndex" region.
// The current region will then display the scene graph of the "aIndex" region.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  IRegion* region = aPage.currentRegion();
  if(!region) return;
  IRegion* fromRegion = aPage.region(aIndex);
  if(!fromRegion) return;
  region->connectFrom(*fromRegion);
}
//////////////////////////////////////////////////////////////////////////////
/// viewer ///////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
/// region ///////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static bool region_setParameter(
 IRegion& aRegion
,const std::string& aWhat
,int aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  Lib::smanip::printf(s,32,"%d",aValue);
  return aRegion.setParameter(aWhat,s);
}
//////////////////////////////////////////////////////////////////////////////
static bool region_setParameter(
 IRegion& aRegion
,const std::string& aWhat
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  Lib::smanip::printf(s,32,"%g",aValue);
  return aRegion.setParameter(aWhat,s);
}
//////////////////////////////////////////////////////////////////////////////
static bool region_setParameter(
 IRegion& aRegion
,const std::string& aWhat
,bool aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return aRegion.setParameter(aWhat,std::string(aValue?"TRUE":"FALSE"));
}
//////////////////////////////////////////////////////////////////////////////
static bool region_parameterValue(
 IRegion& aRegion
,const std::string& aWhat
,int& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = 0;
  std::string s;
  if(!aRegion.parameterValue(aWhat,s)) return false;
  int value;
  if(!Lib::smanip::toint(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static bool region_parameterValue(
 IRegion& aRegion
,const std::string& aWhat
,unsigned int& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = 0;
  std::string s;
  if(!aRegion.parameterValue(aWhat,s)) return false;
  unsigned long value;
  if(!Lib::smanip::toulong(s,value)) return false;
  aValue = (unsigned int)value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static bool region_parameterValue(
 IRegion& aRegion
,const std::string& aWhat
,double& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = 0;
  std::string s;
  if(!aRegion.parameterValue(aWhat,s)) return false;
  double value;
  if(!Lib::smanip::todouble(s,value)) return false;
  aValue = value;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static bool region_parameterValue(
 IRegion& aRegion
,const std::string& aWhat
,bool& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = false;
  std::string s;
  if(!aRegion.parameterValue(aWhat,s)) return false;
  bool value;
  if(!Lib::smanip::tobool(s,value)) return false;
  aValue = value;
  return true;
}
//
//  region_<xxx> related to Inventor are in OnX/Callbacks/Inventor.h
//
//////////////////////////////////////////////////////////////////////////////
/// data /////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static void data_collect(
 ISession& aSession
,const std::string& aWhat
,const std::string& aCuts = ""
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::ITypeManager* typeManager = 
    Lib_findManager(aSession,"TypeManager",Lib::ITypeManager);
  if(!typeManager) return;
  std::vector<std::string> ss;
  ss.push_back(aWhat);
  ss.push_back(aCuts);
  typeManager->execute("collect",ss);
}
//////////////////////////////////////////////////////////////////////////////
static void data_visualize(
 ISession& aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::ITypeManager* typeManager = 
    Lib_findManager(aSession,"TypeManager",Lib::ITypeManager);
  if(!typeManager) return;
  std::vector<std::string> ss;
  typeManager->execute("visualize",ss);
}
//////////////////////////////////////////////////////////////////////////////
static std::string data_number(
 ISession& aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::ITypeManager* typeManager = 
    Lib_findManager(aSession,"TypeManager",Lib::ITypeManager);
  if(!typeManager) return "0";
  std::vector<std::string> ss;
  return typeManager->execute("number",ss);
}
//////////////////////////////////////////////////////////////////////////////
static std::string data_values(
 ISession& aSession
,const std::string& aWhat
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::ITypeManager* typeManager = 
    Lib_findManager(aSession,"TypeManager",Lib::ITypeManager);
  if(!typeManager) return "0";
  std::vector<std::string> ss;
  ss.push_back(aWhat);
  return typeManager->execute("values",ss);
}
//////////////////////////////////////////////////////////////////////////////
static void data_dump(
 ISession& aSession
,const std::string& aMode = "table"
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::ITypeManager* typeManager = 
    Lib_findManager(aSession,"TypeManager",Lib::ITypeManager);
  if(!typeManager) return;
  std::vector<std::string> ss;
  ss.push_back(aMode);
  typeManager->execute("dump",ss);
}
//////////////////////////////////////////////////////////////////////////////
static void data_filter(
 ISession& aSession
,const std::string& aWhat
,const std::string& aFilter = ""
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::ITypeManager* typeManager = 
    Lib_findManager(aSession,"TypeManager",Lib::ITypeManager);
  if(!typeManager) return;
  std::vector<std::string> ss;
  ss.push_back(aWhat);
  ss.push_back(aFilter);
  typeManager->execute("filter",ss);
}
//////////////////////////////////////////////////////////////////////////////
static void data_set(
 ISession& aSession
,const std::string& aWhat
,const std::string& aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::ITypeManager* typeManager = 
    Lib_findManager(aSession,"TypeManager",Lib::ITypeManager);
  if(!typeManager) return;
  std::vector<std::string> ss;
  ss.push_back(aWhat);
  ss.push_back(aValue);
  typeManager->execute("set",ss);
}
//////////////////////////////////////////////////////////////////////////////
static void data_destroy(
 ISession& aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::ITypeManager* typeManager = 
    Lib_findManager(aSession,"TypeManager",Lib::ITypeManager);
  if(!typeManager) return;
  std::vector<std::string> ss;
  typeManager->execute("destroy",ss);
}
//////////////////////////////////////////////////////////////////////////////
static void data_toggle(
 ISession& aSession
,const std::string& aName
,const std::string& aColor
,const std::string& aStatus
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aStatus=="true") {
    session_setColor(aSession,aColor);
    data_collect(aSession,aName);
    data_number(aSession);
    data_visualize(aSession);
  } else {
    std::string filter = "name==\""+aName+"/*\"";
    data_collect(aSession,"SceneGraph",filter);
    data_destroy(aSession);
  }
}

//////////////////////////////////////////////////////////////////////////////
/// tree /////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static void tree_set_with_strings(
 IUI& aUI
,const std::string& aParameter
,const std::vector<std::string>& aList
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string sout = "<tree>";
  for(unsigned int index=0;index<aList.size();index++) {
    sout += "<treeItem><label>"+aList[index]+"</label></treeItem>";
  }
  sout += "</tree>";
  aUI.setParameter(aParameter,sout);
}
//////////////////////////////////////////////////////////////////////////////
static void tree_set(
 IUI& aUI
,const std::string& aParameter
,const std::string& aString
) 
//////////////////////////////////////////////////////////////////////////////
// Example : tree_set(aUI,"mainTree.items","aa\nbb")
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> items;
  Lib::smanip::lines(aString,items);
  tree_set_with_strings(aUI,aParameter,items);
}
//////////////////////////////////////////////////////////////////////////////
static void tree_set_with_file(
 IUI& aUI
,const std::string& aParameter
,const std::string& aFile
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> lines;
  if(!Lib::fmanip::read(aFile,lines)) {
    aUI.echo("Can't open \""+aFile+"\" file.");
  } else {
    aUI.setParameter(aParameter,Lib::smanip::tostring(lines,""));
  }
}
//////////////////////////////////////////////////////////////////////////////
static void OnX_quiet_cxx()
//////////////////////////////////////////////////////////////////////////////
// To quiet g++ warnings about unused macros.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  lib_torgb("");

  int i;
  unsigned int uint;
  double d;
  bool b;

  ISession* session = 0;
  session_modeling_none(*session);
  session_parameterValue(*session,"",i);
  session_parameterValue(*session,"",d);
  session_parameterValue(*session,"",b);
  session_setParameter(*session,"",i);
  session_setParameter(*session,"",d);
  session_setParameter(*session,"",b);
  session_setColor(*session,"");

  IUI* ui = 0;
  ui_parameterValue(*ui,"",i);
  ui_parameterValue(*ui,"",d);
  ui_parameterValue(*ui,"",b);
  ui_setParameter(*ui,"",i);
  ui_setParameter(*ui,"",d);
  ui_setParameter(*ui,"",b);
  ui_execute_file(*ui,"","");
  ui_cbk(*ui,"");
  ui_add_prompt(*ui,"");
  ui_create_page(*ui);
  ui_help_text(*ui,"");
  d = ui_cbk_double(*ui);
  i = ui_cbk_int(*ui);

  tree_set(*ui,"","");
  tree_set_with_file(*ui,"","");

  IPage* page = 0;
  page_parameterValue(*page,"",i);
  page_parameterValue(*page,"",d);
  page_parameterValue(*page,"",b);
  page_setParameter(*page,"",i);
  page_setParameter(*page,"",d);
  page_setParameter(*page,"",b);
  page_connect_current_region(*page,0);

  IRegion* region = 0;
  region_parameterValue(*region,"",i);
  region_parameterValue(*region,"",uint);
  region_parameterValue(*region,"",d);
  region_parameterValue(*region,"",b);
  region_setParameter(*region,"",i);
  region_setParameter(*region,"",d);
  region_setParameter(*region,"",b);

  data_collect(*session,"","");
  data_visualize(*session);
  data_number(*session);
  data_values(*session,"");
  data_dump(*session,"");
  data_filter(*session,"","");
  data_set(*session,"","");
  data_destroy(*session);
  data_toggle(*session,"","","");
}

#endif
