#ifndef OnX_BaseUI_h
#define OnX_BaseUI_h 

// Inheritance :
#include <Lib/Interfaces/IAction.h>

#include <OnX/Interfaces/IUI.h>
#include <OnX/Interfaces/IIB.h>
#include <OnX/Core/CallbackData.h>
#include <OnX/Core/Widget.h>

#include <list>

class IPrinter;
class IScriptManager;
class IWidgetFactory;

namespace Lib {
  class ItemML;
  class IRelationManager;
}

namespace OnX {

class Action : public virtual Lib::IAction {
public: //Lib::IAction
  virtual void* cast(const std::string&) const;
  virtual const std::string& name() const;
  virtual std::string execute(const std::vector<std::string>&);
public:
  Action(IUI&,IComponent,
         const std::string&,int,
         const std::string&,const std::string&);
  Action(IUI&,IComponent,
         const std::string&,int,
         const std::string&,const std::string&,
         const CallbackData&);
  Action(const Action&);
  Action& operator=(const Action&);
  bool equal(const Action&);
  virtual ~Action();
  bool isBackground();
  IComponent getComponent() const;
  void setIdentifier(int);
  int getIdentifier() const;
  IUI& ui() const;
  const CallbackData& callbackData() const;
  const std::string& script() const;
  void setScript(const std::string&);
public:
  IUI& fUI;
  IComponent fComponent;
  std::string fName;
  int fIdentifier;
  std::string fInterpreter;
  std::string fScript;
  CallbackData fCallbackData;
};

typedef int(*Dispatcher)(void*);

class BaseUI;

class Execute {
public:
  Execute(BaseUI&,const std::string&);
  const std::string& what() const;
  BaseUI& baseUI();
private:
  BaseUI& fUI;
  std::string fWhat;
};

class BaseUI : public virtual IUI, public virtual IIB {
public: //IManager
  virtual const std::string& name() const;
  virtual void* cast(const std::string&) const;
public: //IUI
  virtual void executeScript(const std::string&,const std::string&);
  virtual void addDispatcher(int(*)(void*));
  virtual bool setCallback(const std::string&,
                           const std::string&,
                           const std::string&,
                           const std::string&,
                           int = 0);
  virtual bool removeCallback(const std::string&,
                              const std::string&,
                              int = 0);
  virtual const std::string& callbackValue();
  virtual const ICallbackData& callbackData();
  virtual bool executeCallbacks(const std::string&,const std::string&);
  virtual bool load(const std::string&,const std::string&,bool = true);
  virtual bool save();
  virtual void deleteChildren(const std::string&);
  virtual void echo(const std::string&);
  virtual std::string ask(const std::string&);
  virtual IWidget* findWidget(const std::string&);
  virtual std::string currentViewer();
  virtual bool setCurrentViewer(const std::string&);
  virtual IPage* currentPage();
  virtual bool createComponent(const std::string&,
                               const std::string&,
                               const std::string&);
  virtual bool destroyComponent(const std::string&);
  virtual bool copyComponent(const std::string&,
                             const std::string&,
                             const std::string&);
  virtual std::vector<std::string> findChildren(const std::string&,
                                                const std::string&,
                                                const std::string&) const;
  virtual ISession& session() const;
  virtual ICyclic* findCyclic(const std::string&) const;
  virtual bool isComponent(const std::string&);
  virtual int steerLevel() const;
public: //IB
  virtual std::string tree() const;
  virtual bool loadReferenceFile(Lib::ItemML&, const std::string&);
  virtual Lib::ItemML* topItemML() const;
  virtual IWidgetClass* findWidgetClass(const std::string&) const;
  virtual std::vector< std::pair<bool,std::string> > parseDecorations(const std::string&) const;
  virtual const std::vector<std::string>& callbackLanguageList() const;
  virtual bool propertyType(const std::string&,std::string&) const;
  virtual std::string createWidgetName();
public:
  // Used in QtUI :
  void setCallbackData(const CallbackData&);
  bool notification(Lib::IMessage*);
  void emitDelete(const std::string&);
  // Used in Hippo :
  Lib::ItemMLFactory& itemMLFactory();
  void addFactory(IWidgetFactory*);
  IPrinter& printer() const;
  static bool executeItemMLCallbacks(Lib::ItemML&,void*);
protected:
  BaseUI(ISession&);
  virtual ~BaseUI();
protected:
  // REVIST : should have some IDevUI.h for the below.
  virtual bool createWidget(Lib::ItemML&) = 0;
  virtual bool destroyWidget(Lib::ItemML&) = 0;
  virtual Action* addCallback(Lib::ItemML&,
                              const std::string&,
                              const std::string&,
                              const std::string&) = 0;
  virtual void deleteCallback(Lib::ItemML&,
                              const std::string&,
                              const Action&) = 0;
  virtual std::string widgetName(Lib::ItemML&) = 0;
  virtual void showAll(Lib::ItemML&) = 0;
  //
  virtual Action* getAction(IComponent);
  virtual Action* getAction(const std::string&);
  virtual Action* getAction(int);
  virtual Action* getAction(IComponent,const std::string&,int = 0);
  bool beingDestroyed() const;
  void destroyViewer(const std::string&);
  void emitCreate(const std::string&);
  //
  bool startup(const std::string&);
  std::list<Action> fActions;
  Action& addAction(IUI&,IComponent,
                    const std::string&,
                    const std::string&,
                    const std::string& = "",int = 0);
  void removeActions(IComponent);
  void removeAction(IComponent,const std::string&,int = 0);
  void removeDispatcher(int(*)(void*));
  bool toWidgetAndResource(const std::string&,std::string&,std::string&);
  bool propertyValue(const std::string&,const std::string&,std::string&);
  virtual void* findNativeWidget(const std::string&);
  virtual Lib::ItemML* createComponent(const std::string&,
                                       const std::string&,
                                       Lib::ItemML&);
  virtual bool solveInputDialog(Lib::ItemML&);
  virtual bool createInputDialog(Lib::ItemML&);
  virtual std::string buildName(Lib::ItemML&);
  virtual Lib::ItemML* parentItemML(Lib::ItemML&);
  virtual bool searchChild(Lib::ItemML&,
                           const std::string&,
                           const std::string&,
                           std::vector<std::string>&) const;
  virtual void steerModal();
  bool executeCallbacks(Lib::ItemML&,const std::string&);
  bool createIf(Lib::ItemML&);
  Lib::ItemML* findFirstWidget(Lib::ItemML&);
  std::vector<std::string> disableCallbackScript(IComponent,const std::string&);
  bool enableCallbackScript(IComponent,const std::string&,const std::vector<std::string>&);
  bool createChildren(Lib::ItemML&);
private:
  std::string subTree(Lib::ItemML&,int) const;
  void createWidgetTable();
  void createLanguageList();
  void createPropertyList();
protected:
  ISession& fSession;
  std::string fName;
  CallbackData fCallbackData;
  std::list<Dispatcher> fDispatchers;
  Lib::ItemML* fTopItemML;
  IScriptManager* fScriptManager;
  std::list<ICyclic*> fCyclics;
  int fWidgetNameCounter;
  bool fBeingDestroyed;
  int fVerboseLevel;
  int fSteerLevel;
  Lib::IRelationManager* fRelationManager;
private: //IB
  std::vector<IWidgetClass*> fWidgetClasses;
  std::vector<std::string> fCallbackLanguageList;
  std::vector<std::string> fPropertyList;
  WidgetFactory fWidgetFactory;
  std::vector<IWidgetFactory*> fFactories;
};

}

#endif


