#ifndef OnX_BatchUI_h
#define OnX_BatchUI_h 

// Inheritance :
#include <OnX/Interfaces/IUI.h>

#include <OnX/Core/CallbackData.h>
class ISession;
class IScriptManager;

#include <Lib/Printer.h>

namespace OnX {

class BatchUI : public virtual IUI {
public: //IManager :
  virtual const std::string& name() const;
  virtual void* cast(const std::string&) const;
public: //IUI :
  virtual void executeScript(const std::string&,const std::string&);
  virtual void put(const std::string&);
  virtual int steer();
  virtual int steerLevel() const;
  virtual void synchronize();
  virtual void exit(int = 0);
  virtual bool lock();
  virtual bool unlock();
  virtual void echo(const std::string&);
  virtual void warn(const std::string&);
  virtual std::string ask(const std::string&);
  virtual void addDispatcher(int(*)(void*));
  virtual bool removeCallback(const std::string&,
                              const std::string&,
                              int = 0);
  virtual bool setCallback(const std::string&,
                           const std::string&,
                           const std::string&,
                           const std::string&,
                           int = 0);
  virtual void show(const std::string& = "");
  virtual void hide(const std::string& = "");
  virtual bool refresh(const std::string&);
  virtual bool size(const std::string&,unsigned int&,unsigned int&);
  virtual bool position(const std::string&,int&,int&);
  virtual const std::string& callbackValue();
  virtual const ICallbackData& callbackData();
  virtual const std::string& thisComponent();
  virtual bool parameterValue(const std::string&,std::string&);
  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool executeCallbacks(const std::string&,const std::string&);
  virtual bool load(const std::string&,const std::string&,bool = true);
  virtual bool save();
  virtual void deleteChildren(const std::string&);
  virtual bool write(const std::string&,
                     const std::string&,const std::string&);

  virtual IWidget* findWidget(const std::string&);
  virtual std::string currentViewer();
  virtual bool setCurrentViewer(const std::string&);
  virtual IPage* currentPage();
  virtual IPage* findPage(const std::string&);

  virtual ICyclic* createCyclic(const std::string&,
                                int,const std::string&,const std::string&);
  virtual ICyclic* findCyclic(const std::string&) const;
  virtual bool createComponent(const std::string& aName,
                               const std::string& aClass,
                               const std::string& aParent);
  virtual bool copyComponent(const std::string&,
                             const std::string&,
                             const std::string&);
  virtual std::vector<std::string> findChildren(const std::string&,
                                                const std::string&,
                                                const std::string&) const;
  virtual bool destroyComponent(const std::string&);
  virtual bool isComponent(const std::string&);
  virtual ISession& session() const;
public: //Lib::IAuditor
  virtual bool notify(Lib::IMessage*);
public:
  BatchUI(ISession&,const std::string& = "",int = 0,char** = 0);
  virtual ~BatchUI();
  IPrinter& printer() const;
private:
  ISession& fSession;
  std::string fName;
  std::string fValue;
  std::string fResource;
  Lib::Printer fPrinter;
  IScriptManager* fScriptManager;
  CallbackData fCallbackData;
};

}

#endif

