#ifndef OnX_ScriptManager_h
#define OnX_ScriptManager_h

#include <list>

#include <Lib/Interfaces/IManager.h>
#include <OnX/Interfaces/IScriptManager.h>

class ISession;

namespace OnX {

class Interpreter {
public:
  Interpreter(ISession&,
              const std::string&,
              const std::string&,
              const std::string&);
  virtual ~Interpreter();
  const std::string& name() const;
  const std::string& package() const;
  void setFunction(IInterpreterFunction,void*);
  IInterpreterFunction function(void*&) const;
  bool load(bool = false);
private:
  std::string fName;
  std::string fLibrary;
  std::string fPackage;
  bool fLoaded;
  IInterpreterFunction fFunction;
  void* fTag;
  ISession& fSession;
};

class ScriptManager
:public virtual IManager 
,public virtual IScriptManager 
{
public: //IManager
  virtual const std::string& name() const;
  virtual void* cast(const std::string&) const;
public: //IScriptManager
  virtual void addInterpreter(const std::string&,
                              const std::string&,const std::string&);
  virtual void setInterpreterFunction(const std::string&,
                                      IInterpreterFunction,
                                      void*);
  virtual bool findInterpreterFunction(const std::string&,
                                       IInterpreterFunction&,
                                       void*&);
  virtual void executeFile(const std::string&,const std::string&);
  virtual void executeScript(const std::string&,const std::string&);
  virtual void recordExecutedScripts(bool);
  virtual bool isOperational(const std::string&);
public:
  ScriptManager(ISession&);
  virtual ~ScriptManager();
  Interpreter* findInterpreter(const std::string&);
  void history(const std::string&);
private:
  void closeLog();
private:
  ISession& fSession;
  std::string fName;
  std::list<Interpreter*> fList;
  std::string fLogFile;
};

}

#endif




