#ifndef OnX_Session_h
#define OnX_Session_h

#include <list>

class IPrinter;

#include <Lib/Interfaces/ISession.h>
#include <Lib/StreamBuffer.h>
#include <Lib/Parameters.h>

namespace OnX {

class StreamDestination : public Lib::StreamDestination {
public:
  StreamDestination(IPrinter&);
  virtual ~StreamDestination();
  int receive(const std::string&);
private:
  IPrinter& fPrinter;
};


class Session : public virtual ISession {
public: //ISession
  virtual void addManager(IManager*);
  virtual void removeManager(IManager*);
  virtual IManager* findManager(const std::string&) const;
  virtual void destroyManager(const std::string&);
  virtual IPrinter& printer() const;
  virtual bool load(const std::string&,const std::string&);
  virtual void redirectOutput(bool);
  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool parameterValue(const std::string&,std::string&);
  virtual std::vector<std::string> availableParameters() const;
  virtual void flush();
  virtual void setVerboseLevel(int);
  virtual int verboseLevel() const;
  virtual std::vector<std::string> managerNames(const std::string&) const;
public:
  Session(const std::string&,int);
  virtual ~Session();
  void coutToOnX();
  void coutToDefault();
  void stdoutToOnX();
  void stdoutToDefault();
  //virtual void executeScript(const std::string&,const std::string&);
private:
  std::list<IManager*> fManagers;
  IPrinter* fPrinter;
  // cout redirection :
  streambuf* fOutOld;
  Lib::StreamBuffer* fOutNew;
  StreamDestination* fOutDestination;
  // cerr redirection :
  streambuf* fErrOld;
  Lib::StreamBuffer* fErrNew;
  StreamDestination* fErrDestination;
  // stdout redirection :
  std::string fOutDefault;
  std::string fOutName;
  void* fOutFile; //Do not expose FILE*
  // stderr redirection :
  std::string fErrDefault;
  std::string fErrName;
  void* fErrFile; //Do not expose FILE*
  Lib::Parameters<std::string,std::string> fParameters;
  int fVerboseLevel;
};

}

#endif


