#ifndef OnX_GtkUI_h
#define OnX_GtkUI_h 

// Inheritance :
#include <OnX/Core/BaseUI.h>

#include <gtk/gtk.h>
#include <OnX/Gtk/gtkpageviewer.h> //FIXME

#include <OnX/Inventor/Page.h> //FIXME

namespace OnX {

class GtkUI : public BaseUI {
public: //IUI
  virtual void* cast(const std::string&) const;
  virtual void put(const std::string&);
  virtual int steer();
  virtual void synchronize();
  virtual void exit(int = 0);
  virtual bool lock();
  virtual bool unlock();
  virtual void show(const std::string& = "");
  virtual void hide(const std::string& = "");
  virtual void warn(const std::string&);
  virtual bool refresh(const std::string&);
  virtual bool size(const std::string&,unsigned int&,unsigned int&);
  virtual bool position(const std::string&,int&,int&);
  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool parameterValue(const std::string&,std::string&);
  virtual void deleteChildren(const std::string&);
  virtual bool write(const std::string&,const std::string&,const std::string&);
  virtual IPage* findPage(const std::string&);
  virtual ICyclic* createCyclic(const std::string&,
                                int,const std::string&,const std::string&);
  virtual bool notify(Lib::IMessage*);
public: //OnX::BaseUI
  virtual bool createWidget(Lib::ItemML&);
  virtual bool destroyWidget(Lib::ItemML&);
  virtual Action* addCallback(Lib::ItemML&,
                           const std::string&,
                           const std::string&,
                           const std::string&);
  virtual void deleteCallback(Lib::ItemML&,
                              const std::string&,
                              const OnX::Action&);
  virtual std::string widgetName(Lib::ItemML&);
  virtual void showAll(Lib::ItemML&);
public:
  GtkUI(ISession&,const std::string&,int,char**);
  virtual ~GtkUI();
  bool createTree(GtkWidget*,Lib::ItemML*);
private:
  static void callback(GtkObject*,gpointer);
  static void pageCallback(GtkObject*,GtkCallbackStruct*,gpointer);
#ifdef HAS_GTK1
  static void realizeCallback(GtkObject*,gpointer);
#else
  static void treeViewActivatedCallback(GtkTreeView*,GtkTreePath*,GtkTreeViewColumn*,gpointer);
  static void treeViewOpenCloseCallback(GtkTreeView*,GtkTreeIter*,GtkTreePath*,gpointer);
#endif
  static void notebookCallback(GtkObject*,GtkNotebookPage*,guint,gpointer);
  static void destroyCallback(GtkObject*,gpointer);
  static gboolean avoidDeleteCallback(GtkObject*,gpointer);
  static void listCallback(GtkObject*,GdkEvent*,gpointer);
  static void exposeCallback(GtkObject*,GdkEventExpose*,gpointer);
  static void eventCallback(GtkObject*,GdkEvent*,gpointer);
  static void warningCallback(GtkWidget*,gpointer);
  static gint timer_message(gpointer);

#ifndef HAS_GTK1
  static bool fillModel(GtkWidget*,GtkTreeStore *,GtkTreeIter*,Lib::ItemML*);
#endif
private:
  int getCompound(const std::string&);
  GtkType getType(const std::string&);
  void addCallbacks(Lib::ItemML&,IComponent,const std::string&,GtkObject*,const std::string&,GtkSignalFunc = (GtkSignalFunc)OnX::GtkUI::callback);
  std::string getCallbackName(GtkType,const std::string&);
  IPage* getIPage(SoPage*);
  void setTreeSelection(GtkWidget*,const std::string&);
  std::string getTreeSelection(GtkWidget*);
private:
  GdkWindow* fTopWindow;
  GtkWidget* fConsoleWindow;
  GtkWidget* fConsoleText;
  GtkWidget* fWarningDialog;
  GdkFont* fFixedFont;
  std::list<GtkWidget*> fWidgets;
  std::list<OnX::Page> fPages;
  int fExitCode;
};

}

#endif

