#ifndef OnX_InventorType_h
#define OnX_InventorType_h

// Inheritance :
#include <OnX/Core/BaseType.h>

// Should be an embedded class.

#include <Lib/Interfaces/ISession.h>
#include <Lib/Manager.h>
#include <Lib/smanip.h>

#include <OnX/Interfaces/IUI.h>
#include <OnX/Interfaces/IPage.h>
#include <OnX/Interfaces/IRegion.h>
#include <OnX/Interfaces/IStyleManager.h>

#include <HEPVis/misc/SoGC.h>
#include <HEPVis/nodekits/SoRegion.h>

namespace OnX {

class InventorType : public OnX::BaseType {
public: //Lib::IType.
  virtual void* cast(const std::string& aTo) const {
    if(aTo=="OnX::InventorType") {
      return Lib_SCast(OnX::InventorType);
    } else {
      return OnX::BaseType::cast(aTo);
    }
  }
public: //OnX::IType.
  virtual void beginVisualize() {
    fSoGC.clear();
    fSoRegion = 0;
    IUI* ui = Lib_findManager(fSession,"UI_Manager",IUI);
    if(!ui) return;
    IPage* page = ui->currentPage();
    if(!page) return;
    IRegion* region = page->currentRegion();
    if(!region) return;
    fSoRegion = (SoRegion*)region->nativeRegion();  
    fSoGC.setStyleCache(fSoRegion->styleCache());
    fillSoGC(name()); //By default get a style with same name as the type.
  }
  virtual void endVisualize() {
  }
public:
  InventorType(ISession& aSession,const std::string& aStyleFile = "")
  :OnX::BaseType(aSession.printer())
  ,fSession(aSession)
  ,fStyleManager(0)
  ,fSoRegion(0){
    fStyleManager = Lib_findManager(fSession,"StyleManager",IStyleManager);
    if(fStyleManager && (aStyleFile!="") ) {
      fStyleManager->loadFile(aStyleFile);
    }
  }
  virtual ~InventorType() {}
  bool loadStyleFile(const std::string& aStyleFile) {
    if(!fStyleManager) return false;
    return fStyleManager->loadFile(aStyleFile);
  }
protected:
  void fillSoGC(const std::string& aStyle) {
    fSoGC.reset();
    std::string sstyle;
    if(fStyleManager && fStyleManager->findStyle(aStyle,sstyle)) { 
      fSoGC.setFromString(SbString(sstyle.c_str()));
    } 

    // If not "none", session modeling.[color,...] overrides file style.

    sstyle = "";
    std::string value;
    fSession.parameterValue("modeling.color",value);
    if((value!="")&&(value!="none")) sstyle += "\ncolor "+value;

    fSession.parameterValue("modeling.highlightColor",value);
    if((value!="")&&(value!="none")) sstyle += "\nhighlightColor "+value;

    fSession.parameterValue("modeling.transparency",value);
    if((value!="")&&(value!="none")) sstyle += "\ntransparency "+value;
  
    fSession.parameterValue("modeling.lineStyle",value); // Deprecated :
    if((value!="")&&(value!="none")) sstyle += "\nlineStyle "+value;

    fSession.parameterValue("modeling.linePattern",value);
    if((value!="")&&(value!="none")) sstyle += "\nlinePattern "+value;

    fSession.parameterValue("modeling.lineWidth",value);
    if((value!="")&&(value!="none")) sstyle += "\nlineWidth "+value;
  
    fSession.parameterValue("modeling.projection",value);
    if((value!="")&&(value!="none")) sstyle += "\nprojection "+value;

    fSession.parameterValue("modeling.cut",value);
    if((value!="")&&(value!="none")) sstyle += "\ncut "+value;

    fSession.parameterValue("modeling.modeling",value);
    if((value!="")&&(value!="none")) sstyle += "\nmodeling "+value;
  
    //Backward compatibility.
    fSession.parameterValue("modeling.type",value);
    if((value!="")&&(value!="none")) {
      if(value=="text") {
        sstyle += "\nmodeling texts";
      } else {
        sstyle += "\nmodeling "+value;
      }
    }
  
    fSoGC.setFromString(SbString(sstyle.c_str()));

    fSession.parameterValue("modeling.localSetup",value);
    if((value!="")&&(value!="none")) {
      bool b;
      if(Lib::smanip::tobool(value,b)) {
        fSoGC.setLocalSetup(b?TRUE:FALSE);
      }
    }


  }
protected:
  ISession& fSession;
  IStyleManager* fStyleManager;
  SoRegion* fSoRegion;
  SoGC fSoGC;
};

}

#endif
