#ifndef OnX_NextStepUI_h
#define OnX_NextStepUI_h 

// Inheritance :
#include <OnX/Core/BaseUI.h>

#import <Foundation/NSGeometry.h>

@class NSAutoreleasePool;
@class NSApplication;
@class NSObject;
@class NSText;
@class NSWindow;
@class NSAlert;

@class OnX_NSTreeItem;
@class OnX_NSApplicationDelegate;

#include <OnX/Inventor/Page.h> //FIXME

namespace OnX {

class NextStepUI : public BaseUI {
public: //IUI
  virtual void* cast(const std::string&) const;
  virtual void put(const std::string&);
  virtual int steer();
  virtual void synchronize();
  virtual void exit(int = 0);
  virtual bool lock();
  virtual bool unlock();

  virtual void show(const std::string& = "");
  virtual void hide(const std::string& = "");
  virtual bool refresh(const std::string&);
  virtual void warn(const std::string&);
  virtual bool size(const std::string&,unsigned int&,unsigned int&);
  virtual bool position(const std::string&,int&,int&);
  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool parameterValue(const std::string&,std::string&);
  virtual bool write(const std::string&,const std::string&,const std::string&);
  virtual IPage* findPage(const std::string&);
  virtual ICyclic* createCyclic(const std::string&,
                                int,const std::string&,const std::string&);
  virtual bool notify(Lib::IMessage*);
public: //OnX::BaseUI
  virtual bool createWidget(Lib::ItemML&);
  virtual bool destroyWidget(Lib::ItemML&);
  virtual Action* addCallback(Lib::ItemML&,
                           const std::string&,
                           const std::string&,
                           const std::string&);
  virtual void deleteCallback(Lib::ItemML&,
                              const std::string&,
                              const Action&);
  virtual std::string widgetName(Lib::ItemML&);
  virtual void showAll(Lib::ItemML&);
public:
  NextStepUI(ISession&,const std::string&,int,char**);
  virtual ~NextStepUI();
  bool createTree(OnX_NSTreeItem*,Lib::ItemML*);
  static void callback(Action&);
  bool findName(NSObject*,std::string&);
private:
  static NSRect getFrame(NSObject*);
private:
  void addInParent(Lib::ItemML&,NSObject*,NSObject*);
  bool destroyOneWidget(Lib::ItemML&);
  int getType(const std::string&);
  void addCallbacks(Lib::ItemML&,NSObject*,const std::string&);
  void removeInPool(NSObject*);
  IPage* getIPage(SoPage*);
  virtual void steerModal();
protected:
  NSAutoreleasePool* fAutoreleasePool;
  NSApplication* fApplication;
  NSObject* fTopWidget;
private:
  std::list< std::pair<NSObject*,std::string> > fPool;
  OnX_NSApplicationDelegate* fAppDelegate;
  NSText* fConsole;
  NSWindow* fMessageWindow;
  NSAlert* fAlert;
  std::list<OnX::Page> fPages;
  int fExitCode;
};

}

#endif

