#ifndef OnX_QTk_h
#define OnX_QTk_h 

#include <qsplitter.h>
#include <qgl.h>

class QPopupMenu;
class QCallbackData;

namespace Lib {
  class IMessage;
}

namespace OnX {
  class Action;
  class QtUI;
}

class OnX_QOpenGLArea : public QGLWidget {
  Q_OBJECT
public:
  OnX_QOpenGLArea(QWidget*,const char*);
  virtual ~OnX_QOpenGLArea();
  virtual void initialieGL();
  virtual void resizeGL(int,int);
  virtual void paintGL();
  virtual void mousePressEvent(QMouseEvent*);
  virtual void mouseReleaseEvent(QMouseEvent*);
  virtual void mouseMoveEvent(QMouseEvent*);
signals:
  void paint(QCallbackData&);
  void eventHandler(QCallbackData&);
};

// To handle splitPosition at startup.
class OnX_QSplitter : public QSplitter {
  Q_OBJECT
public:
  OnX_QSplitter(Qt::Orientation,QWidget*,const char*);
  virtual ~OnX_QSplitter();
  void setSplitPosition(int);
  virtual void showEvent(QShowEvent*);
private:
  int fSplitPosition;
};

//WARNING : can't be put in the main panel. The show 
// on the main panel will show it and then we will
// be blocked in the QColorDialog::getColor().

//class OnX_QColorDialog : public QColorDialog {
class OnX_QColorDialog : public QWidget {
  Q_OBJECT
public:
  OnX_QColorDialog(const char*);
  virtual ~OnX_QColorDialog();
  virtual void show();
  virtual void hide();
  const std::string value() const;
signals:
  void ok();
private:
  std::string fValue;
};

class OnX_QMenu : public QWidget {
  Q_OBJECT
public:
  OnX_QMenu(const char*,QPopupMenu&);
  virtual ~OnX_QMenu();
  QPopupMenu& popupMenu() const;
private:
  QPopupMenu& fPopup;
};

class OnX_QMenuItem : public QWidget {
  Q_OBJECT
public:
  OnX_QMenuItem(const char*,QPopupMenu&,int = 0);
  virtual ~OnX_QMenuItem();
  QPopupMenu& popupMenu() const;
private:
  QPopupMenu& fPopup;
  int fID;
};

class OnX_QToggleMenuItem : public OnX_QMenuItem {
  Q_OBJECT
public:
  OnX_QToggleMenuItem(const char*,QPopupMenu&,int = 0);
  virtual ~OnX_QToggleMenuItem();
};

class QtCallback : public QObject {
  Q_OBJECT
public:
  QtCallback(OnX::Action&);
  virtual ~QtCallback();
  OnX::Action& action() const;
public slots:
  void destroy();
  void execute();
  void execute2(QCallbackData&);
private:
  OnX::Action& fAction;
};

namespace OnX {
class QtMessage {
public:
  QtMessage(QtUI&,Lib::IMessage*);
  ~QtMessage();
public:
  QtUI& fUI;
  Lib::IMessage* fMessage;
};
}

class OnX_QMessageWidget : public QWidget {
protected:
  void customEvent(QCustomEvent*);
};

#endif
