#ifndef OnX_QtUI_h
#define OnX_QtUI_h 

// Inheritance :
#include <OnX/Core/BaseUI.h>

class QApplication;
class QWidget;
class QPopupMenu;
class QMessageBox;
class QMetaObject;

#include <OnX/Inventor/Page.h> //FIXME

class QtCallback;
class OnX_QMessageWidget;
class QListViewItem;

#include <vector>

namespace OnX {

class QtUI : public BaseUI {
public: //IUI
  virtual void* cast(const std::string&) const;
  virtual void put(const std::string&);
  virtual int steer();
  virtual void synchronize();
  virtual void exit(int = 0);
  virtual bool lock();
  virtual bool unlock();

  virtual bool setCallback(const std::string&,
                           const std::string&,
                           const std::string&,
                           const std::string&,
                           int = 0);
  virtual void show(const std::string& = "");
  virtual void hide(const std::string& = "");
  virtual void warn(const std::string&);
  virtual bool refresh(const std::string&);
  virtual bool size(const std::string&,unsigned int&,unsigned int&);
  virtual bool position(const std::string&,int&,int&);

  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool parameterValue(const std::string&,std::string&);
  virtual bool write(const std::string&,const std::string&,const std::string&);
  virtual IPage* findPage(const std::string&);
  virtual ICyclic* createCyclic(const std::string&,
                                int,const std::string&,const std::string&);
  virtual bool notify(Lib::IMessage*);
public: //OnX::BaseUI
  virtual bool createWidget(Lib::ItemML&);
  virtual bool destroyWidget(Lib::ItemML&);
  virtual Action* addCallback(Lib::ItemML&,
                           const std::string&,
                           const std::string&,
                           const std::string&);
  virtual void deleteCallback(Lib::ItemML&,
                              const std::string&,
                              const Action&);
  virtual std::string widgetName(Lib::ItemML&);
  virtual void showAll(Lib::ItemML&);
public:
  QtUI(ISession&,const std::string&,int,char**);
  virtual ~QtUI();
  QApplication* getQApplication() const;
private:
  bool createTree(QWidget*,Lib::ItemML*);
  //bool setExpand(QWidget*,const std::string);
  //unsigned int getQtDecor(const std::string&,const std::string&);
  int getCompound(const std::string&);
  std::string getType(const std::string&);
  bool destroyOneWidget(Lib::ItemML&);
  void addCallbacks(Lib::ItemML&,QWidget*,
                    const std::string&,
                    const std::string&,
                    const std::string& = "");
  void addCallbacks(Lib::ItemML&,QPopupMenu*,int aActualID,const std::string&);
  std::string getCallbackName(const QMetaObject&,const std::string&);
  static std::string getXMLTree(QListViewItem*);
  IPage* getIPage(SoPage*);
protected:
  QApplication* fQApplication;
private:
  bool fFirstWindow;
  // Console panel :
  QWidget* fConsoleWindow;
  QWidget* fConsoleText;
  QMessageBox* fWarningDialog;
  int fCounter;
  int fMenuItemCounter;
  std::vector<QtCallback*> fQtCallbacks;
  OnX_QMessageWidget* fMessageWidget;
  std::list<QWidget*> fWidgets;
  std::list<OnX::Page> fPages;
};

}

#endif

