#ifndef OnX_WinUI_h
#define OnX_WinUI_h 

// Inheritance :
#include <OnX/Core/BaseUI.h>

#include <windows.h>
#include <commctrl.h>

namespace WinTk {
  class Component;
  class MessageWindow;
  class ScrolledText;
  class MenuItem;
  class CallbackData;
  class Tree;
  class WarningDialog;
}

#include <OnX/Inventor/Page.h> 

namespace OnX {

class WinUI : public BaseUI {
public: //IUI
  virtual void* cast(const std::string&) const;
  virtual void put(const std::string&);
  virtual int steer();
  virtual void synchronize();
  virtual void exit(int = 0);
  virtual bool lock();
  virtual bool unlock();

  virtual void show(const std::string& = "");
  virtual void hide(const std::string& = "");
  virtual void warn(const std::string&);
  virtual bool refresh(const std::string&);
  virtual bool size(const std::string&,unsigned int&,unsigned int&);
  virtual bool position(const std::string&,int&,int&);
  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool parameterValue(const std::string&,std::string&);
  virtual bool write(const std::string&,const std::string&,const std::string&);
  virtual IPage* findPage(const std::string&);
  virtual ICyclic* createCyclic(const std::string&,
                                int,const std::string&,const std::string&);
  virtual bool notify(Lib::IMessage*);
public: //OnX::BaseUI
  virtual bool createWidget(Lib::ItemML&);
  virtual bool destroyWidget(Lib::ItemML&);
  virtual Action* addCallback(Lib::ItemML&,
                           const std::string&,
                           const std::string&,
                           const std::string&);
  virtual void deleteCallback(Lib::ItemML&,
                              const std::string&,
                              const Action&);
  virtual std::string widgetName(Lib::ItemML&);
  virtual void showAll(Lib::ItemML&);
public:
  WinUI(ISession&,const std::string&,int,char**);
  virtual ~WinUI();
  bool createTree(WinTk::Tree*,Lib::ItemML*);
  WinTk::Component* topWidget() const;
private:
  static void callback(WinTk::Component&,WinTk::CallbackData&,void*);
  static void destroyCallback(WinTk::Component&,WinTk::CallbackData&,void*);
  static void messageCallback(WinTk::Component&,WinTk::CallbackData&,void*);
  static void windowCallback(WinTk::Component&,WinTk::CallbackData&,void*);
  static LRESULT CALLBACK Win32WindowProc(HWND,UINT,WPARAM,LPARAM);
  static std::string getXMLTree(HWND,HTREEITEM,int);
private:
  bool destroyOneWidget(Lib::ItemML&);
  int getType(const std::string&);
  void addCallbacks(Lib::ItemML&,WinTk::Component*,const std::string&);
  WinTk::MenuItem* findMenuItem(int);
  IPage* getIPage(SoPage*);
protected:
  WinTk::Component* fTopWidget;
private:
  std::list<WinTk::Component*> fWidgets;
  // Console panel :
  WinTk::Component* fConsoleWindow;
  WinTk::ScrolledText* fConsoleText;
  int fActionID;
  WinTk::MessageWindow* fMessageWindow;
  WinTk::WarningDialog* fWarningDialog;
  std::list< std::pair<int,WinTk::MenuItem*> > fMenuItems;
  std::list<OnX::Page> fPages;
};

}

#endif

