#ifndef OnX_XtUI_h
#define OnX_XtUI_h 

// Inheritance :
#include <OnX/Core/BaseUI.h>

#include <X11/Intrinsic.h>
typedef Widget XtWidget;

#include <Xm/Xm.h>

#include <OnX/Interfaces/IUI.h>

#include <OnX/Inventor/Page.h> //FIXME

namespace OnX {

class XtUI : public BaseUI {
public: //IUI
  virtual void* cast(const std::string&) const;
  virtual void put(const std::string&);
  virtual int steer();
  virtual void synchronize();
  virtual void exit(int = 0);
  virtual bool lock();
  virtual bool unlock();
  virtual void show(const std::string& = "");
  virtual void hide(const std::string& = "");
  virtual void warn(const std::string&);
  virtual bool refresh(const std::string&);
  virtual bool size(const std::string&,unsigned int&,unsigned int&);
  virtual bool position(const std::string&,int&,int&);
  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool parameterValue(const std::string&,std::string&);
  virtual bool write(const std::string&,const std::string&,const std::string&);
  virtual IPage* findPage(const std::string&);
  virtual ICyclic* createCyclic(const std::string&,
                                int,const std::string&,const std::string&);
  virtual bool notify(Lib::IMessage*);
public: //OnX::BaseUI
  virtual bool createWidget(Lib::ItemML&);
  virtual bool destroyWidget(Lib::ItemML&);
  virtual Action* addCallback(Lib::ItemML&,
                           const std::string&,
                           const std::string&,
                           const std::string&);
  virtual void deleteCallback(Lib::ItemML&,
                              const std::string&,
                              const Action&);
  virtual std::string widgetName(Lib::ItemML&);
  virtual void showAll(Lib::ItemML&);
public:
  /** 
   * @param aSession : a valid ISession.
   * @param aArgc : coming from main, needed to initialize Xt.
   * @param aArgv : coming from main, needed to initialize Xt.
   */
  XtUI(ISession&,const std::string&,int,char**);

  virtual ~XtUI();
  bool createTree(XtWidget,Lib::ItemML*);
  XtAppContext appContext() const;
  XtWidget appWidget() const;
private:
  static void callback(XtWidget,XtPointer,XtPointer);
  static void destroyCallback(XtWidget,XtPointer,XtPointer);
  static void warningCallback(XtWidget,XtPointer,XtPointer);
private:
  //bool setSplitPosition(XtWidget,int,const std::string);
  int getCompound(const std::string&);
  WidgetClass getType(const std::string&);
  std::string getCallbackName(WidgetClass,const std::string&);
  void addCallbacks(Lib::ItemML&,XtWidget,const std::string&,const std::string&);
  IPage* getIPage(SoPage*);
protected:
  XtWidget fAppWidget;
  XtAppContext fAppContext;
private:
  std::list<XtWidget> fWidgets;
  // Console panel :
  XtWidget fConsoleWindow;
  XtWidget fConsoleText;
  XtWidget fWarningDialog;
  std::list<OnX::Page> fPages;
};

}

#endif

