#/////////////////////////////////////////////////////////////////////////////
# OnX module :
#/////////////////////////////////////////////////////////////////////////////

# Module OnX_SWIG is produced by SWIG.
import OnX_SWIG
from OnX_SWIG import *

#/////////////////////////////////////////////////////////////////////////////
# Usefull variables :
#/////////////////////////////////////////////////////////////////////////////

session = OnX_getSession()

#print 'debug : OnX.py'

ui = IManager_to_IUI(session.findManager('UI_Manager'))
printer = session.printer()
out = printer # For backward compatibility.
typeManager = IManager_to_ITypeManager(session.findManager('TypeManager'))
scriptManager = IManager_to_IScriptManager(session.findManager('ScriptManager'))

# Redirect Python output :
import sys
onx_stderrSave = sys.stderr
onx_stdoutSave = sys.stdout
sys.stdout = session.printer()
sys.stderr = session.printer()

#FIXME : execute OnX_end at exit only if Python had been loaded.
def OnX_end():
  import sys
  # Restore Python output redirection :
  sys.stdout = onx_stdoutSave
  sys.stderr = onx_stderrSave
  printer.disable()

#/////////////////////////////////////////////////////////////////////////////
# sys procedures :
#/////////////////////////////////////////////////////////////////////////////

def sys_import(aModule):
  import sys
  if sys.modules.has_key(aModule):
    reload(sys.modules[aModule])
  else:
    exec('import %s' % aModule)

def sys_dump_main():
  import sys
  print dir(sys.modules['__main__'])

def sys_exists(name):
  try:
    eval(name)
    return 1
  except NameError:
    return 0

#/////////////////////////////////////////////////////////////////////////////
# std_string procedures :
#/////////////////////////////////////////////////////////////////////////////

def smanip_first_line(aString):
  vs = std_vector_string()
  Lib_smanip_lines(aString,vs)
  if vs.size() >=1 :
    return vs.operator_bracket(0).c_str()
  else :
    return ""

def smanip_last_line(aString):
  vs = std_vector_string()
  Lib_smanip_lines(aString,vs)
  if vs.size() >=1 :
    return vs.operator_bracket(vs.size()-1).c_str()
  else :
    return ""

def s2d(aString):
  v = OnX_Double()
  if Lib_smanip_todouble(aString,v.p()) == 0 :
    ui.echo('Cannot convert string to double.')
    return 0
  else :
    return v.v()       

def s2i(aString):
  v = OnX_Long()
  if Lib_smanip_tolong(aString,v.p()) == 0 :
    ui.echo('Cannot convert string to long.')
    return 0
  else :
    return v.v()       

def s2b(aString):
  v = OnX_Boolean()
  if Lib_smanip_tobool(aString,v.p()) == 0 :
    ui.echo('Cannot convert string to bool.')
    return 0
  else :
    return v.v()       

#/////////////////////////////////////////////////////////////////////////////
# session procedures :
#/////////////////////////////////////////////////////////////////////////////

def session_setColor(aRGB):
  session.setParameter('modeling.color',aRGB)

def session_parameterValue(aWhat):
  ss = std_string('')
  if session.parameterValue(aWhat,ss) == 0 : return ''
  cs = ss.c_str()
  del ss
  return cs

#/////////////////////////////////////////////////////////////////////////////
# out procedures :
#/////////////////////////////////////////////////////////////////////////////

def out_file(aFile):
  if session_parameterValue('session.helpOnDialog') == 'yes':
    out_file_dialog(aFile)
  else:
    out_file_console(aFile)

def out_file_console(aFile):
  import sys
  input = open(aFile,'r')
  for line in input.readlines() :
    printer.write(line)
  del input

def out_file_dialog(aFile):
  # Expand env variables :
  fname = Lib_fmanip_name(aFile).c_str()
  #print 'debug : "' + aFile + '" "' + fname + '"'
  import sys
  input = open(fname,'r')
  lines = ''
  for line in input.readlines() :
    lines = lines + line
  del input
  import os
  ui.load('main',os.environ['ONXROOT']+'/scripts/OnX/HelpDialog.onx')
  ui.setParameter('OnX_HelpDialog_text.value',lines)
  ui.show('OnX_HelpDialog_dialog')

#/////////////////////////////////////////////////////////////////////////////
# typeManager procedures :
#/////////////////////////////////////////////////////////////////////////////

def data_collect(aWhat,aCuts = ''):
  ss = std_vector_string()
  ss.push_back(aWhat)
  ss.push_back(aCuts)
  typeManager.execute('collect',ss)
  del ss

def data_visualize():
  ss = std_vector_string()
  typeManager.execute('visualize',ss)
  del ss

def data_filter(aWhat,aFilter = ''):
  ss = std_vector_string()
  ss.push_back(aWhat)
  ss.push_back(aFilter)
  typeManager.execute('filter',ss)
  del ss

def data_destroy():
  ss = std_vector_string()
  typeManager.execute('destroy',ss)
  del ss

def data_dump(aMode = 'table'):
  ss = std_vector_string()
  ss.push_back(aMode)
  typeManager.execute('dump',ss)
  del ss

def data_number():
  ss = std_vector_string()
  sret = typeManager.execute('number',ss).c_str()
  del ss
  return sret

def data_set(aWhat,aValue):
  ss = std_vector_string()
  ss.push_back(aWhat)
  ss.push_back(aValue)
  typeManager.execute('set',ss)
  del ss

def data_values(aWhat):
  ss = std_vector_string()
  ss.push_back(aWhat)
  sret = typeManager.execute('values',ss).c_str()
  del ss
  return sret

def tree_set_from_strings(aParameter,aStrings):
  s = Lib_smanip_tostring(aStrings,'\n').c_str()
  tree_set(aParameter,s)

def tree_set(aParameter,aString):
  # Example : tree_set('mainTree.items','aa\nbb')
  vs = std_vector_string()
  Lib_smanip_lines(aString,vs)
  sout = '<tree>'
  for index in range(0,vs.size()):
    s = vs.operator_bracket(index).c_str()
    sout += '<treeItem><label>'+s+'</label></treeItem>'
  sout += '</tree>'
  ui.setParameter(aParameter,sout)

#/////////////////////////////////////////////////////////////////////////////
# Inventor procedures :
#/////////////////////////////////////////////////////////////////////////////

def iv_dumpNode( aNode ):
  ss = SbString('')
  aNode.get(ss)
  ui.echo(ss.getString())
  del ss

#/////////////////////////////////////////////////////////////////////////////
# ui procedures :
#/////////////////////////////////////////////////////////////////////////////

def ui_parameterValue(aWhat):
  ss = std_string('')
  assert ui.parameterValue(aWhat,ss) == 1
  cs = ss.c_str()
  del ss
  return cs

def ui_setFileChooser(aProc,aDirMask='*',aDirectory=''):
  ui.setCallback('fileChooser','ok','Python','ui.hide("fileChooser")\n'+aProc+'(ui.callbackValue().c_str())')
  ui.setCallback('fileChooser','cancel','Python','ui.hide("fileChooser")')
  ui.setParameter('fileChooser.dirMask',aDirMask)
  ui.setParameter('fileChooser.directory',aDirectory)
  ui.setParameter('fileChooser.mode','open')

def ui_setColorChooser():
  ui.setCallback('colorChooser','ok','Python','session_setColor(ui.callbackValue().c_str())')

def ui_setTreeItems(aWidget,aFile):
  import os
  lines = std_vector_string()
  if Lib_fmanip_read(aFile,lines) == 0:
    ui.echo("Can't open Tree.xml file." )
  else:
    s = Lib_smanip_tostring(lines,'')
    ui.setParameter(aWidget+'.items',s.c_str())
    del s
  del lines

#/////////////////////////////////////////////////////////////////////////////
# viewer procedures :
#/////////////////////////////////////////////////////////////////////////////

def viewer_collect():
  data_collect('SceneGraph','highlight==true')
  data_filter('name')
  data_dumpMode = session_parameterValue('TypeManager.dumpMode')
  data_dump(data_dumpMode)

def viewer_popup():
  item = ui.callbackValue().c_str()
  if item == "Current region" :
    x = ui.callbackData().x()
    y = ui.callbackData().y()
    index = ui.currentPage().positionRegion(x,y)
    ui.currentPage().setCurrentRegion(index)
  elif item == "No highlighted" :
    ui.currentPage().setParameter('highlighted','off')

#/////////////////////////////////////////////////////////////////////////////
# Inventor procedures :
#/////////////////////////////////////////////////////////////////////////////
def sokit_parameterValue(aNode,aWhat):
  ss = SbString(aWhat)
#FIXME : below crashes with Coin
#  aNode.get(ss)
  SoTools_getKitPartValue(aNode,SbString(aWhat),ss)
  return ss.getString()

#/////////////////////////////////////////////////////////////////////////////
# SoPage procedures :
#/////////////////////////////////////////////////////////////////////////////
def ui_SoPage(aUI):
  widget = ui.findWidget(ui.currentViewer().c_str())
  assert widget
  pageViewer = IWidget_to_IPageViewer(widget)
  assert pageViewer
  return pageViewer.soPage()

def sopage_setCurrentRegion(aPage,aIndex):
  soRegion = aPage.getRootRegion(aIndex)
  if soRegion == 0 : return
  aPage.highlightRegion(soRegion)

def sopage_currentRegion(aPage):
  highlighted = aPage.getHighlightedRegion()
  assert highlighted
  if aPage.isRootRegion(highlighted) == 0 : return 0
  return highlighted

def sopage_createRegion(aPage,aClass,aX,aY,aWidth,aHeight):
  soRegion = aPage.createRegion(SbString(aClass),aX,aY,aWidth,aHeight)
  if aPage.getNumberOfRootRegions() == 1 : aPage.highlightRegion(soRegion)
  return soRegion

#/////////////////////////////////////////////////////////////////////////////
# SoRegion procedures :
#/////////////////////////////////////////////////////////////////////////////

def soregion_staticNode(aRegion):
  return SoNode_to_SoSeparator(aRegion.findNode(SbString("part"),SbString("staticScene")))

def soregion_dynamicNode(aRegion):
  return SoNode_to_SoSeparator(aRegion.findNode(SbString("part"),SbString("dynamicScene")))

def soregion_readInventorFile(aRegion,aFile):
  name = Lib_fmanip_name(aFile)
  soInput = SoInput()
  if soInput.openFile(name.c_str()) == 0 : return
  soSep = SoDB_readAll(soInput)
  if soSep == 0 : return
  doIt = SbAddNode(soSep,SbString("staticScene"))
  if aRegion.doIt(doIt) == 0 : 
    soSep.unref()
    return

