#/////////////////////////////////////////////////////////////////////////////
# OnX module :
#/////////////////////////////////////////////////////////////////////////////

# For info, in tcl "puts" write something on stdout.
# puts hello

# Linux, OSF1 : the tcl "load" do a dlopen
#  with RTLD_GLOBAL option that induces problems if 
#  loaded code contains stream manipulations.
#  To bypass this problem, we first dlopen
#  (through the loadLib command and then 
#  Lib::Library class) without RTLD_GLOBAL mode that seems
#  to cure the problem.
# KCC & CINT & KUIP : the same trick permits to 
#  have CINT and KUIP working with KCC for OnX.
#  But we loose dynamic_cast info for KUIP at the level of Lab...
proc loadLibrary { aFile aPackage } {
  # For info, env is an associative array in the global namespace.
  if { [info exists ::env($aFile) ] } {
    # OnX loadLib :
    loadLib $::env($aFile)
    # Tcl load :
    load $::env($aFile) $aPackage
  } else {
    # OnX loadLib :
    loadLib $aFile
    # Tcl load :
    load $aFile $aPackage
  }
}

proc dllName { aDLL } {
  if { [info exists ::env(OSTYPE) ] } {
    # UNIX :
    set os $::env(OSTYPE)
    #puts $::env(OSTYPE)
    if { $os == "darwin" } {
      return [format "lib%s.dylib" $aDLL]
    }
    # linux, osf1
    return [format "lib%s.so" $aDLL]
  }
  # Windows 
  return $aDLL
}

loadLibrary [dllName OnXTclSWIG] OnX_SWIG

#/////////////////////////////////////////////////////////////////////////////
# OnX tcl global variables (session, out, ui, data) :
#/////////////////////////////////////////////////////////////////////////////

ISession session -this [OnX_getSession]
IPrinter printer -this [session printer]

if { [session findManager UI_Manager] == "NULL" } {
 printer write "UI manager not found !\n"
 return    
}

IUI ui -this [IManager_to_IUI [session findManager UI_Manager]]

# [db cget -this] is the SWIG way to retreive the address of an object !!!
# We create a '&' procedure to hide that :
proc & {aName} { $aName cget -this }

# Procedure to delete an existing "command object".
# Contrary to "rename", it does not produce an error 
# if object does not exist. 
proc delete {aObject} {
 if { [info commands $aObject] != "" } { rename $aObject ""}
}

# TypeManager facilities :
Lib_ITypeManager typeManager -this [IManager_to_ITypeManager [session findManager TypeManager]]

#IRepresenter representer -this [IManager_to_IRepresenter [session findManager Representer]]

#/////////////////////////////////////////////////////////////////////////////
# session procedures :
#/////////////////////////////////////////////////////////////////////////////

proc session_setColor { rgb  } {
 session setParameter modeling.color $rgb
}

#/////////////////////////////////////////////////////////////////////////////
# typeManager procedures :
#/////////////////////////////////////////////////////////////////////////////

proc data_collect { aWhat { aCuts "" } } {
 std_vector_string ss
 ss push_back $aWhat
 ss push_back $aCuts
 typeManager execute collect [& ss]
 delete ss
}
proc data_vis {} {
 std_vector_string ss
 typeManager execute visualize [& ss]
 delete ss
}
proc data_show {} {
 std_vector_string ss
 typeManager execute visualize [& ss]
 delete ss
}
proc data_filter { aProperty } {
 std_vector_string ss
 ss push_back $aProperty
 typeManager execute filter [& ss]
 delete ss
}
proc data_dump {} {
 std_vector_string ss
 typeManager execute dump [& ss]
 delete ss
}
proc data_number {} {
 std_vector_string ss
 typeManager execute number [& ss]
 delete ss
}
proc data_destroy {} {
 std_vector_string ss
 typeManager execute destroy [& ss]
 delete ss
}
proc data_setValue { aWhat aValue} {
 std_vector_string ss
 ss push_back $aWhat
 ss push_back $aValue
 typeManager execute collect [& ss]
 delete ss
}
proc data_values { aName } {
 std_vector_string ss
 ss push_back $aName
 std_string s -this [typeManager execute values [& ss]]
 delete ss
 set cs [s c_str]
 delete s
 return $cs
}

#/////////////////////////////////////////////////////////////////////////////
# ui procedures :
#/////////////////////////////////////////////////////////////////////////////
proc ui_parameterValue { aWhat } {
# Exa : echo [ ui_parameterValue mainTree.selectedItems ]
  std_string ss ""
  ui parameterValue $aWhat [& ss]
  set sret [ ss c_str ]
  delete ss
  return $sret
}


#/////////////////////////////////////////////////////////////////////////////
# SoPage procedures :
#/////////////////////////////////////////////////////////////////////////////
proc ui_SoPage {} {
  std_string s -this [ui currentViewer]
  IWidget widget -this [ ui findWidget [s c_str] ]
  IPageViewer pageViewer -this [IWidget_to_IPageViewer [& widget]]
  return [pageViewer soPage]
}

proc sopage_currentRegion { aPage } {
  SoRegion highlighted -this [$aPage getHighlightedRegion]
# if { [$aPage isRootRegion $highlighted] == 0 } return 0
  return [& highlighted]
}

#/////////////////////////////////////////////////////////////////////////////
# SoRegion procedures :
#/////////////////////////////////////////////////////////////////////////////

proc soregion_staticNode { aRegion } {
  SbString a0 "part"
  SbString a1 "staticScene"
  SoNode node -this [$aRegion findNode [& a0] [& a1]]
  delete a0
  delete a1
  return [SoNode_to_SoSeparator [& node]]
}
